/* Generated by Cython 0.29.29 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "__LITTLE_ENDIAN__",
                "1"
            ]
        ],
        "depends": [
            "ddtrace/internal/_utils.h",
            "ddtrace/internal/pack.h"
        ],
        "include_dirs": [
            "ddtrace/internal",
            "./ddtrace/internal",
            "."
        ],
        "name": "ddtrace.internal._encoding",
        "sources": [
            "ddtrace/internal/_encoding.pyx"
        ]
    },
    "module_name": "ddtrace.internal._encoding"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_29"
#define CYTHON_HEX_VERSION 0x001D1DF0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__ddtrace__internal___encoding
#define __PYX_HAVE_API__ddtrace__internal___encoding
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#include <stdint.h>
#include "_utils.h"
#include "pack.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "ddtrace/internal/_encoding.pyx",
  "stringsource",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable;
struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable;
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable;
struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder;
struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder;
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;
struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer;

/* "ddtrace/internal/_encoding.pyx":145
 * 
 * 
 * cdef class StringTable(object):             # <<<<<<<<<<<<<<
 *     cdef dict _table
 *     cdef stdint.uint32_t _next_id
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable {
  PyObject_HEAD
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *__pyx_vtab;
  PyObject *_table;
  uint32_t _next_id;
};


/* "ddtrace/internal/_encoding.pyx":188
 * 
 * 
 * cdef class ListStringTable(StringTable):             # <<<<<<<<<<<<<<
 *     cdef list _list
 * 
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable __pyx_base;
  PyObject *_list;
};


/* "ddtrace/internal/_encoding.pyx":202
 * 
 * 
 * cdef class MsgpackStringTable(StringTable):             # <<<<<<<<<<<<<<
 *     cdef msgpack_packer pk
 *     cdef int max_size
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable __pyx_base;
  struct msgpack_packer pk;
  int max_size;
  int _sp_len;
  uint32_t _sp_id;
  PyObject *_lock;
  size_t _reset_size;
};


/* "ddtrace/internal/_encoding.pyx":302
 * 
 * 
 * cdef class BufferedEncoder(object):             # <<<<<<<<<<<<<<
 *     content_type: str = None
 * 
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder {
  PyObject_HEAD
  size_t max_size;
  size_t max_item_size;
  PyObject *_lock;
};


/* "ddtrace/internal/_encoding.pyx":323
 * 
 * 
 * cdef class ListBufferedEncoder(BufferedEncoder):             # <<<<<<<<<<<<<<
 *     cdef list _buffer
 *     cdef Py_ssize_t _size
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder __pyx_base;
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_vtab;
  PyObject *_buffer;
  Py_ssize_t _size;
};


/* "ddtrace/internal/_encoding.pyx":369
 * 
 * 
 * cdef class MsgpackEncoderBase(BufferedEncoder):             # <<<<<<<<<<<<<<
 *     content_type = "application/msgpack"
 * 
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder __pyx_base;
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_vtab;
  struct msgpack_packer pk;
  uint32_t _count;
};


/* "ddtrace/internal/_encoding.pyx":500
 * 
 * 
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):             # <<<<<<<<<<<<<<
 *     cpdef flush(self):
 *         with self._lock:
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase __pyx_base;
};


/* "ddtrace/internal/_encoding.pyx":645
 * 
 * 
 * cdef class MsgpackEncoderV05(MsgpackEncoderBase):             # <<<<<<<<<<<<<<
 *     cdef MsgpackStringTable _st
 * 
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase __pyx_base;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *_st;
};


/* "ddtrace/internal/_encoding.pyx":746
 * 
 * 
 * cdef class Packer(object):             # <<<<<<<<<<<<<<
 *     """Slightly modified version of the v0.6.2 msgpack Packer
 *     which only supports basic Python types (int, bool, float, dict, list).
 */
struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer {
  PyObject_HEAD
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *__pyx_vtab;
  struct msgpack_packer pk;
  PyObject *_default;
  PyObject *_berrors;
  char const *encoding;
  char const *unicode_errors;
};



/* "ddtrace/internal/_encoding.pyx":145
 * 
 * 
 * cdef class StringTable(object):             # <<<<<<<<<<<<<<
 *     cdef dict _table
 *     cdef stdint.uint32_t _next_id
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable {
  PyObject *(*insert)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *);
  uint32_t (*_index)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *);
  PyObject *(*index)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*reset)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *__pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable;


/* "ddtrace/internal/_encoding.pyx":188
 * 
 * 
 * cdef class ListStringTable(StringTable):             # <<<<<<<<<<<<<<
 *     cdef list _list
 * 
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListStringTable {
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable __pyx_base;
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_vtabptr_7ddtrace_8internal_9_encoding_ListStringTable;


/* "ddtrace/internal/_encoding.pyx":202
 * 
 * 
 * cdef class MsgpackStringTable(StringTable):             # <<<<<<<<<<<<<<
 *     cdef msgpack_packer pk
 *     cdef int max_size
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable {
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable __pyx_base;
  PyObject *(*savepoint)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *);
  PyObject *(*rollback)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *);
  PyObject *(*get_bytes)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *);
  PyObject *(*append_raw)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *, long, Py_ssize_t);
  PyObject *(*flush)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackStringTable;


/* "ddtrace/internal/_encoding.pyx":323
 * 
 * 
 * cdef class ListBufferedEncoder(BufferedEncoder):             # <<<<<<<<<<<<<<
 *     cdef list _buffer
 *     cdef Py_ssize_t _size
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListBufferedEncoder {
  PyObject *(*put)(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get)(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_vtabptr_7ddtrace_8internal_9_encoding_ListBufferedEncoder;


/* "ddtrace/internal/_encoding.pyx":369
 * 
 * 
 * cdef class MsgpackEncoderBase(BufferedEncoder):             # <<<<<<<<<<<<<<
 *     content_type = "application/msgpack"
 * 
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase {
  PyObject *(*_decode)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_reset_buffer)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *);
  PyObject *(*encode)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch);
  int (*_update_array_len)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *);
  PyObject *(*get_bytes)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *);
  char *(*get_buffer)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *);
  void *(*get_dd_origin_ref)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *);
  int (*_pack_trace)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *);
  PyObject *(*put)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*flush)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch);
  int (*pack_span)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, void *);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *);
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__pack_trace(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *);


/* "ddtrace/internal/_encoding.pyx":500
 * 
 * 
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):             # <<<<<<<<<<<<<<
 *     cpdef flush(self):
 *         with self._lock:
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 {
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase __pyx_base;
  int (*_pack_meta)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *, char *);
  int (*_pack_metrics)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_meta(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *, char *);
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_metrics(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *);


/* "ddtrace/internal/_encoding.pyx":645
 * 
 * 
 * cdef class MsgpackEncoderV05(MsgpackEncoderBase):             # <<<<<<<<<<<<<<
 *     cdef MsgpackStringTable _st
 * 
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 {
  struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase __pyx_base;
  int (*_pack_string)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *, PyObject *);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *, PyObject *);


/* "ddtrace/internal/_encoding.pyx":746
 * 
 * 
 * cdef class Packer(object):             # <<<<<<<<<<<<<<
 *     """Slightly modified version of the v0.6.2 msgpack Packer
 *     which only supports basic Python types (int, bool, float, dict, list).
 */

struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer {
  int (*_pack)(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *, PyObject *);
  PyObject *(*pack)(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *__pyx_vtabptr_7ddtrace_8internal_9_encoding_Packer;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* KeywordStringCheck.proto */
static int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyInstance_Check(obj)))
        return __Pyx_PyObject_GetAttrStr(obj, attr_name);
#endif
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#else
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* py_dict_items.proto */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* UnpackUnboundCMethod.proto */
typedef struct {
    PyObject *type;
    PyObject **method_name;
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;

/* CallUnboundCMethod0.proto */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CallUnboundCMethod0(cfunc, self)\
    (likely((cfunc)->func) ?\
        (likely((cfunc)->flag == METH_NOARGS) ?  (*((cfunc)->func))(self, NULL) :\
         (PY_VERSION_HEX >= 0x030600B1 && likely((cfunc)->flag == METH_FASTCALL) ?\
            (PY_VERSION_HEX >= 0x030700A0 ?\
                (*(__Pyx_PyCFunctionFast)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0) :\
                (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL)) :\
          (PY_VERSION_HEX >= 0x030700A0 && (cfunc)->flag == (METH_FASTCALL | METH_KEYWORDS) ?\
            (*(__Pyx_PyCFunctionFastWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, &__pyx_empty_tuple, 0, NULL) :\
            (likely((cfunc)->flag == (METH_VARARGS | METH_KEYWORDS)) ?  ((*(PyCFunctionWithKeywords)(void*)(PyCFunction)(cfunc)->func)(self, __pyx_empty_tuple, NULL)) :\
               ((cfunc)->flag == METH_VARARGS ?  (*((cfunc)->func))(self, __pyx_empty_tuple) :\
               __Pyx__CallUnboundCMethod0(cfunc, self)))))) :\
        __Pyx__CallUnboundCMethod0(cfunc, self))
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_As_PY_LONG_LONG(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int64_t __Pyx_PyInt_As_int64_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_insert(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_string); /* proto*/
static uint32_t __pyx_f_7ddtrace_8internal_9_encoding_11StringTable__index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_reset(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_15ListStringTable_insert(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self, PyObject *__pyx_v_string); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_insert(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, PyObject *__pyx_v_string); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_savepoint(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_rollback(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_get_bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_append_raw(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, long __pyx_v_src, Py_ssize_t __pyx_v_size); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_reset(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, PyObject *__pyx_v_item, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_get(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__decode(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__reset_buffer(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_encode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto*/
static char *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_buffer(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto*/
static void *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_dd_origin_ref(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_dd_origin); /* proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__pack_trace(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_flush(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_pack_span(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_span, CYTHON_UNUSED void *__pyx_v_dd_origin); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static void *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_get_dd_origin_ref(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_dd_origin); /* proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_meta(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_meta, char *__pyx_v_dd_origin); /* proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_metrics(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_metrics); /* proto*/
static int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_pack_span(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_span, void *__pyx_v_dd_origin); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_trace, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_string); /* proto*/
static void *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_get_dd_origin_ref(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_dd_origin); /* proto*/
static int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_pack_span(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_span, void *__pyx_v_dd_origin); /* proto*/
static int __pyx_f_7ddtrace_8internal_9_encoding_6Packer__pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_o); /* proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_6Packer_pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.bytearray' */

/* Module declarations from 'libc' */

/* Module declarations from 'libc.stdint' */

/* Module declarations from 'ddtrace.internal._utils' */

/* Module declarations from 'ddtrace.internal._encoding' */
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_StringTable = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_ListBufferedEncoder = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 = 0;
static PyTypeObject *__pyx_ptype_7ddtrace_8internal_9_encoding_Packer = 0;
static PY_LONG_LONG __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT;
static char const *__pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY;
static size_t __pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY_LEN;
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_PyBytesLike_CheckExact(PyObject *); /*proto*/
static CYTHON_INLINE char const *__pyx_f_7ddtrace_8internal_9_encoding_string_to_buff(PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(uint32_t); /*proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes(struct msgpack_packer *, char *, Py_ssize_t); /*proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_number(struct msgpack_packer *, PyObject *); /*proto*/
static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_text(struct msgpack_packer *, PyObject *); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_StringTable__set_state(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_ListStringTable__set_state(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "ddtrace.internal._encoding"
extern int __pyx_module_is_main_ddtrace__internal___encoding;
int __pyx_module_is_main_ddtrace__internal___encoding = 0;

/* Implementation of 'ddtrace.internal._encoding' */
static PyObject *__pyx_builtin_OverflowError;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_RuntimeError;
static PyObject *__pyx_builtin_NotImplementedError;
static const char __pyx_k_o[] = "o";
static const char __pyx_k__3[] = "";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_put[] = "put";
static const char __pyx_k_raw[] = "raw";
static const char __pyx_k_Lock[] = "Lock";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_exit[] = "__exit__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_meta[] = "_meta";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_pack[] = "pack";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_RLock[] = "RLock";
static const char __pyx_k_enter[] = "__enter__";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_flush[] = "flush";
static const char __pyx_k_index[] = "index";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_packb[] = "packb";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_Packer[] = "Packer";
static const char __pyx_k_decode[] = "_decode";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_kwargs[] = "kwargs";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_context[] = "context";
static const char __pyx_k_default[] = "default";
static const char __pyx_k_metrics[] = "_metrics";
static const char __pyx_k_msgpack[] = "msgpack";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_service[] = "service";
static const char __pyx_k_span_id[] = "span_id";
static const char __pyx_k_unpackb[] = "unpackb";
static const char __pyx_k_version[] = "version";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_max_size[] = "max_size";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_resource[] = "resource";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_start_ns[] = "start_ns";
static const char __pyx_k_trace_id[] = "trace_id";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_dd_origin[] = "dd_origin";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_parent_id[] = "parent_id";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_span_type[] = "span_type";
static const char __pyx_k_threading[] = "threading";
static const char __pyx_k_BufferFull[] = "BufferFull";
static const char __pyx_k_ORIGIN_KEY[] = "ORIGIN_KEY";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_StringTable[] = "StringTable";
static const char __pyx_k_duration_ns[] = "duration_ns";
static const char __pyx_k_encode_item[] = "encode_item";
static const char __pyx_k_RuntimeError[] = "RuntimeError";
static const char __pyx_k_content_type[] = "content_type";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_OverflowError[] = "OverflowError";
static const char __pyx_k_max_item_size[] = "max_item_size";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_internal_error[] = "internal error";
static const char __pyx_k_BufferedEncoder[] = "BufferedEncoder";
static const char __pyx_k_ListStringTable[] = "ListStringTable";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_MsgpackEncoderV03[] = "MsgpackEncoderV03";
static const char __pyx_k_MsgpackEncoderV05[] = "MsgpackEncoderV05";
static const char __pyx_k_ddtrace_constants[] = "ddtrace.constants";
static const char __pyx_k_dict_is_too_large[] = "dict is too large";
static const char __pyx_k_list_is_too_large[] = "list is too large";
static const char __pyx_k_BufferItemTooLarge[] = "BufferItemTooLarge";
static const char __pyx_k_Couldn_t_pack_span[] = "Couldn't pack span";
static const char __pyx_k_MsgpackEncoderBase[] = "MsgpackEncoderBase";
static const char __pyx_k_MsgpackStringTable[] = "MsgpackStringTable";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_Couldn_t_pack_trace[] = "Couldn't pack trace";
static const char __pyx_k_ListBufferedEncoder[] = "ListBufferedEncoder";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_application_msgpack[] = "application/msgpack";
static const char __pyx_k_Unhandled_meta_type_r[] = "Unhandled meta type: %r";
static const char __pyx_k_Unhandled_text_type_r[] = "Unhandled text type: %r";
static const char __pyx_k_Unhandled_metrics_type_r[] = "Unhandled metrics type: %r";
static const char __pyx_k_Unhandled_numeric_type_r[] = "Unhandled numeric type: %r";
static const char __pyx_k_pyx_unpickle_StringTable[] = "__pyx_unpickle_StringTable";
static const char __pyx_k_Integer_value_out_of_range[] = "Integer value out of range";
static const char __pyx_k_ddtrace_internal__encoding[] = "ddtrace.internal._encoding";
static const char __pyx_k_default_must_be_a_callable[] = "default must be a callable.";
static const char __pyx_k_unicode_string_is_too_large[] = "unicode string is too large";
static const char __pyx_k_pyx_unpickle_ListStringTable[] = "__pyx_unpickle_ListStringTable";
static const char __pyx_k_ddtrace_internal__encoding_pyx[] = "ddtrace/internal/_encoding.pyx";
static const char __pyx_k_Cannot_insert_s_string_table_is[] = "Cannot insert '%s': string table is full (current size: %d, max size: %d).";
static const char __pyx_k_Failed_to_add_string_to_msgpack[] = "Failed to add string to msgpack string table";
static const char __pyx_k_Pickling_of_struct_members_such[] = "Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)";
static const char __pyx_k_Failed_to_append_raw_bytes_to_ms[] = "Failed to append raw bytes to msgpack string table";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0[] = "Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))";
static const char __pyx_k_Unable_to_allocate_internal_buff[] = "Unable to allocate internal buffer.";
static const char __pyx_k_no_default___reduce___due_to_non[] = "no default __reduce__ due to non-trivial __cinit__";
static const char __pyx_k_Incompatible_checksums_0x_x_vs_0_2[] = "Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))";
static PyObject *__pyx_n_s_BufferFull;
static PyObject *__pyx_n_s_BufferItemTooLarge;
static PyObject *__pyx_n_s_BufferedEncoder;
static PyObject *__pyx_kp_s_Cannot_insert_s_string_table_is;
static PyObject *__pyx_kp_s_Couldn_t_pack_span;
static PyObject *__pyx_kp_s_Couldn_t_pack_trace;
static PyObject *__pyx_kp_s_Failed_to_add_string_to_msgpack;
static PyObject *__pyx_kp_s_Failed_to_append_raw_bytes_to_ms;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0;
static PyObject *__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2;
static PyObject *__pyx_kp_s_Integer_value_out_of_range;
static PyObject *__pyx_n_s_ListBufferedEncoder;
static PyObject *__pyx_n_s_ListStringTable;
static PyObject *__pyx_n_s_Lock;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_MsgpackEncoderBase;
static PyObject *__pyx_n_s_MsgpackEncoderV03;
static PyObject *__pyx_n_s_MsgpackEncoderV05;
static PyObject *__pyx_n_s_MsgpackStringTable;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_ORIGIN_KEY;
static PyObject *__pyx_n_s_OverflowError;
static PyObject *__pyx_n_s_Packer;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_kp_s_Pickling_of_struct_members_such;
static PyObject *__pyx_n_s_RLock;
static PyObject *__pyx_n_s_RuntimeError;
static PyObject *__pyx_n_s_StringTable;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Unable_to_allocate_internal_buff;
static PyObject *__pyx_kp_s_Unhandled_meta_type_r;
static PyObject *__pyx_kp_s_Unhandled_metrics_type_r;
static PyObject *__pyx_kp_s_Unhandled_numeric_type_r;
static PyObject *__pyx_kp_s_Unhandled_text_type_r;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__3;
static PyObject *__pyx_kp_s_application_msgpack;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_content_type;
static PyObject *__pyx_n_s_context;
static PyObject *__pyx_n_s_dd_origin;
static PyObject *__pyx_n_s_ddtrace_constants;
static PyObject *__pyx_n_s_ddtrace_internal__encoding;
static PyObject *__pyx_kp_s_ddtrace_internal__encoding_pyx;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_default;
static PyObject *__pyx_kp_s_default_must_be_a_callable;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_kp_s_dict_is_too_large;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_duration_ns;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_encode_item;
static PyObject *__pyx_n_s_enter;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_s_exit;
static PyObject *__pyx_n_s_flush;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_index;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_kp_s_internal_error;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_kwargs;
static PyObject *__pyx_kp_s_list_is_too_large;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_max_item_size;
static PyObject *__pyx_n_s_max_size;
static PyObject *__pyx_n_s_meta;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_metrics;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_msgpack;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_kp_s_no_default___reduce___due_to_non;
static PyObject *__pyx_n_s_o;
static PyObject *__pyx_n_s_pack;
static PyObject *__pyx_n_s_packb;
static PyObject *__pyx_n_s_parent_id;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_put;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_ListStringTable;
static PyObject *__pyx_n_s_pyx_unpickle_StringTable;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_raw;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_resource;
static PyObject *__pyx_n_s_service;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_span_id;
static PyObject *__pyx_n_s_span_type;
static PyObject *__pyx_n_s_start_ns;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_threading;
static PyObject *__pyx_n_s_trace_id;
static PyObject *__pyx_kp_s_unicode_string_is_too_large;
static PyObject *__pyx_n_s_unpackb;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_version;
static int __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_2index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string); /* proto */
static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_4__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_6__contains__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_8__reduce_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_10__setstate_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_2__iter__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_4__reduce_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_6__setstate_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, PyObject *__pyx_v_max_size); /* proto */
static void __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_2__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, size_t __pyx_v_max_size, size_t __pyx_v_max_item_size); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_2put(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_4encode(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_2__set__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_2__set__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED size_t __pyx_v_max_size, CYTHON_UNUSED size_t __pyx_v_max_item_size); /* proto */
static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_2__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4put(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_6get(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_8encode_item(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_item); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, size_t __pyx_v_max_size, size_t __pyx_v_max_item_size); /* proto */
static void __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_2__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_6_decode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_8encode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_10put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_12flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, size_t __pyx_v_max_size, CYTHON_UNUSED size_t __pyx_v_max_item_size); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_2flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_trace); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_6Packer___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self); /* proto */
static int __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_2__init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_default); /* proto */
static void __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_4__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_6pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_obj); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_8bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_packb(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_o, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_2__pyx_unpickle_StringTable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_4__pyx_unpickle_ListStringTable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_StringTable(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_ListStringTable(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackStringTable(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_BufferedEncoder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_ListBufferedEncoder(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV03(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV05(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_Packer(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items = {0, &__pyx_n_s_items, 0, 0, 0};
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_6;
static PyObject *__pyx_int_31856863;
static PyObject *__pyx_int_66476266;
static PyObject *__pyx_int_88997951;
static PyObject *__pyx_int_182609841;
static PyObject *__pyx_int_194841817;
static PyObject *__pyx_int_228126377;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__14;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_codeobj__33;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
/* Late includes */

/* "ddtrace/internal/_encoding.pyx":46
 * 
 * 
 * cdef inline int PyBytesLike_CheckExact(object o):             # <<<<<<<<<<<<<<
 *     return PyBytes_CheckExact(o) or PyByteArray_CheckExact(o)
 * 
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_PyBytesLike_CheckExact(PyObject *__pyx_v_o) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("PyBytesLike_CheckExact", 0);

  /* "ddtrace/internal/_encoding.pyx":47
 * 
 * cdef inline int PyBytesLike_CheckExact(object o):
 *     return PyBytes_CheckExact(o) or PyByteArray_CheckExact(o)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = PyBytes_CheckExact(__pyx_v_o);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = PyByteArray_CheckExact(__pyx_v_o);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":46
 * 
 * 
 * cdef inline int PyBytesLike_CheckExact(object o):             # <<<<<<<<<<<<<<
 *     return PyBytes_CheckExact(o) or PyByteArray_CheckExact(o)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":58
 * 
 * 
 * cdef inline const char * string_to_buff(str s):             # <<<<<<<<<<<<<<
 *     IF PY_MAJOR_VERSION >= 3:
 *         return PyUnicode_AsUTF8(s)
 */

static CYTHON_INLINE char const *__pyx_f_7ddtrace_8internal_9_encoding_string_to_buff(PyObject *__pyx_v_s) {
  char const *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("string_to_buff", 0);

  /* "ddtrace/internal/_encoding.pyx":60
 * cdef inline const char * string_to_buff(str s):
 *     IF PY_MAJOR_VERSION >= 3:
 *         return PyUnicode_AsUTF8(s)             # <<<<<<<<<<<<<<
 *     ELSE:
 *         return <const char *> s
 */
  __pyx_r = PyUnicode_AsUTF8(__pyx_v_s);
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":58
 * 
 * 
 * cdef inline const char * string_to_buff(str s):             # <<<<<<<<<<<<<<
 *     IF PY_MAJOR_VERSION >= 3:
 *         return PyUnicode_AsUTF8(s)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":71
 * 
 * 
 * cdef inline int array_prefix_size(stdint.uint32_t l):             # <<<<<<<<<<<<<<
 *     if l < 16:
 *         return 1
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(uint32_t __pyx_v_l) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("array_prefix_size", 0);

  /* "ddtrace/internal/_encoding.pyx":72
 * 
 * cdef inline int array_prefix_size(stdint.uint32_t l):
 *     if l < 16:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif l < (2<<16):
 */
  __pyx_t_1 = ((__pyx_v_l < 16) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":73
 * cdef inline int array_prefix_size(stdint.uint32_t l):
 *     if l < 16:
 *         return 1             # <<<<<<<<<<<<<<
 *     elif l < (2<<16):
 *         return 3
 */
    __pyx_r = 1;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":72
 * 
 * cdef inline int array_prefix_size(stdint.uint32_t l):
 *     if l < 16:             # <<<<<<<<<<<<<<
 *         return 1
 *     elif l < (2<<16):
 */
  }

  /* "ddtrace/internal/_encoding.pyx":74
 *     if l < 16:
 *         return 1
 *     elif l < (2<<16):             # <<<<<<<<<<<<<<
 *         return 3
 *     return MSGPACK_ARRAY_LENGTH_PREFIX_SIZE
 */
  __pyx_t_1 = ((__pyx_v_l < 0x20000) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":75
 *         return 1
 *     elif l < (2<<16):
 *         return 3             # <<<<<<<<<<<<<<
 *     return MSGPACK_ARRAY_LENGTH_PREFIX_SIZE
 * 
 */
    __pyx_r = 3;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":74
 *     if l < 16:
 *         return 1
 *     elif l < (2<<16):             # <<<<<<<<<<<<<<
 *         return 3
 *     return MSGPACK_ARRAY_LENGTH_PREFIX_SIZE
 */
  }

  /* "ddtrace/internal/_encoding.pyx":76
 *     elif l < (2<<16):
 *         return 3
 *     return MSGPACK_ARRAY_LENGTH_PREFIX_SIZE             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 5;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":71
 * 
 * 
 * cdef inline int array_prefix_size(stdint.uint32_t l):             # <<<<<<<<<<<<<<
 *     if l < 16:
 *         return 1
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":79
 * 
 * 
 * cdef inline int pack_bytes(msgpack_packer *pk, char *bs, Py_ssize_t l):             # <<<<<<<<<<<<<<
 *     cdef int ret
 * 
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes(struct msgpack_packer *__pyx_v_pk, char *__pyx_v_bs, Py_ssize_t __pyx_v_l) {
  int __pyx_v_ret;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("pack_bytes", 0);

  /* "ddtrace/internal/_encoding.pyx":82
 *     cdef int ret
 * 
 *     ret = msgpack_pack_raw(pk, l)             # <<<<<<<<<<<<<<
 *     if ret == 0:
 *         ret = msgpack_pack_raw_body(pk, bs, l)
 */
  __pyx_v_ret = msgpack_pack_raw(__pyx_v_pk, __pyx_v_l);

  /* "ddtrace/internal/_encoding.pyx":83
 * 
 *     ret = msgpack_pack_raw(pk, l)
 *     if ret == 0:             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_raw_body(pk, bs, l)
 *     return ret
 */
  __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":84
 *     ret = msgpack_pack_raw(pk, l)
 *     if ret == 0:
 *         ret = msgpack_pack_raw_body(pk, bs, l)             # <<<<<<<<<<<<<<
 *     return ret
 * 
 */
    __pyx_v_ret = msgpack_pack_raw_body(__pyx_v_pk, __pyx_v_bs, __pyx_v_l);

    /* "ddtrace/internal/_encoding.pyx":83
 * 
 *     ret = msgpack_pack_raw(pk, l)
 *     if ret == 0:             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_raw_body(pk, bs, l)
 *     return ret
 */
  }

  /* "ddtrace/internal/_encoding.pyx":85
 *     if ret == 0:
 *         ret = msgpack_pack_raw_body(pk, bs, l)
 *     return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":79
 * 
 * 
 * cdef inline int pack_bytes(msgpack_packer *pk, char *bs, Py_ssize_t l):             # <<<<<<<<<<<<<<
 *     cdef int ret
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":88
 * 
 * 
 * cdef inline int pack_number(msgpack_packer *pk, object n) except? -1:             # <<<<<<<<<<<<<<
 *     if n is None:
 *         return msgpack_pack_nil(pk)
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_number(struct msgpack_packer *__pyx_v_pk, PyObject *__pyx_v_n) {
  CYTHON_UNUSED PyObject *__pyx_v_oe = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned PY_LONG_LONG __pyx_t_7;
  PY_LONG_LONG __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  long __pyx_t_13;
  double __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_number", 0);

  /* "ddtrace/internal/_encoding.pyx":89
 * 
 * cdef inline int pack_number(msgpack_packer *pk, object n) except? -1:
 *     if n is None:             # <<<<<<<<<<<<<<
 *         return msgpack_pack_nil(pk)
 * 
 */
  __pyx_t_1 = (__pyx_v_n == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":90
 * cdef inline int pack_number(msgpack_packer *pk, object n) except? -1:
 *     if n is None:
 *         return msgpack_pack_nil(pk)             # <<<<<<<<<<<<<<
 * 
 *     if PyLong_Check(n):
 */
    __pyx_r = msgpack_pack_nil(__pyx_v_pk);
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":89
 * 
 * cdef inline int pack_number(msgpack_packer *pk, object n) except? -1:
 *     if n is None:             # <<<<<<<<<<<<<<
 *         return msgpack_pack_nil(pk)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":92
 *         return msgpack_pack_nil(pk)
 * 
 *     if PyLong_Check(n):             # <<<<<<<<<<<<<<
 *         # PyInt_Check(long) is True for Python 3.
 *         # So we should test long before int.
 */
  __pyx_t_2 = (PyLong_Check(__pyx_v_n) != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":95
 *         # PyInt_Check(long) is True for Python 3.
 *         # So we should test long before int.
 *         try:             # <<<<<<<<<<<<<<
 *             if n > 0:
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "ddtrace/internal/_encoding.pyx":96
 *         # So we should test long before int.
 *         try:
 *             if n > 0:             # <<<<<<<<<<<<<<
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 *             return msgpack_pack_long_long(pk, <long long> n)
 */
        __pyx_t_6 = PyObject_RichCompare(__pyx_v_n, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 96, __pyx_L5_error)
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 96, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (__pyx_t_2) {

          /* "ddtrace/internal/_encoding.pyx":97
 *         try:
 *             if n > 0:
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)             # <<<<<<<<<<<<<<
 *             return msgpack_pack_long_long(pk, <long long> n)
 *         except OverflowError as oe:
 */
          __pyx_t_7 = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(__pyx_v_n); if (unlikely((__pyx_t_7 == (unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 97, __pyx_L5_error)
          __pyx_r = msgpack_pack_unsigned_long_long(__pyx_v_pk, ((unsigned PY_LONG_LONG)__pyx_t_7));
          goto __pyx_L9_try_return;

          /* "ddtrace/internal/_encoding.pyx":96
 *         # So we should test long before int.
 *         try:
 *             if n > 0:             # <<<<<<<<<<<<<<
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 *             return msgpack_pack_long_long(pk, <long long> n)
 */
        }

        /* "ddtrace/internal/_encoding.pyx":98
 *             if n > 0:
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 *             return msgpack_pack_long_long(pk, <long long> n)             # <<<<<<<<<<<<<<
 *         except OverflowError as oe:
 *             raise OverflowError("Integer value out of range")
 */
        __pyx_t_8 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_n); if (unlikely((__pyx_t_8 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 98, __pyx_L5_error)
        __pyx_r = msgpack_pack_long_long(__pyx_v_pk, ((PY_LONG_LONG)__pyx_t_8));
        goto __pyx_L9_try_return;

        /* "ddtrace/internal/_encoding.pyx":95
 *         # PyInt_Check(long) is True for Python 3.
 *         # So we should test long before int.
 *         try:             # <<<<<<<<<<<<<<
 *             if n > 0:
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 */
      }
      __pyx_L5_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "ddtrace/internal/_encoding.pyx":99
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 *             return msgpack_pack_long_long(pk, <long long> n)
 *         except OverflowError as oe:             # <<<<<<<<<<<<<<
 *             raise OverflowError("Integer value out of range")
 * 
 */
      __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_OverflowError);
      if (__pyx_t_9) {
        __Pyx_AddTraceback("ddtrace.internal._encoding.pack_number", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_10, &__pyx_t_11) < 0) __PYX_ERR(0, 99, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __pyx_v_oe = __pyx_t_10;

        /* "ddtrace/internal/_encoding.pyx":100
 *             return msgpack_pack_long_long(pk, <long long> n)
 *         except OverflowError as oe:
 *             raise OverflowError("Integer value out of range")             # <<<<<<<<<<<<<<
 * 
 *     if PyInt_Check(n):
 */
        __pyx_t_12 = __Pyx_PyObject_Call(__pyx_builtin_OverflowError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 100, __pyx_L7_except_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_Raise(__pyx_t_12, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __PYX_ERR(0, 100, __pyx_L7_except_error)
      }
      goto __pyx_L7_except_error;
      __pyx_L7_except_error:;

      /* "ddtrace/internal/_encoding.pyx":95
 *         # PyInt_Check(long) is True for Python 3.
 *         # So we should test long before int.
 *         try:             # <<<<<<<<<<<<<<
 *             if n > 0:
 *                 return msgpack_pack_unsigned_long_long(pk, <unsigned long long> n)
 */
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L9_try_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":92
 *         return msgpack_pack_nil(pk)
 * 
 *     if PyLong_Check(n):             # <<<<<<<<<<<<<<
 *         # PyInt_Check(long) is True for Python 3.
 *         # So we should test long before int.
 */
  }

  /* "ddtrace/internal/_encoding.pyx":102
 *             raise OverflowError("Integer value out of range")
 * 
 *     if PyInt_Check(n):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_long(pk, <long> n)
 * 
 */
  __pyx_t_2 = (PyInt_Check(__pyx_v_n) != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":103
 * 
 *     if PyInt_Check(n):
 *         return msgpack_pack_long(pk, <long> n)             # <<<<<<<<<<<<<<
 * 
 *     if PyFloat_Check(n):
 */
    __pyx_t_13 = __Pyx_PyInt_As_long(__pyx_v_n); if (unlikely((__pyx_t_13 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 103, __pyx_L1_error)
    __pyx_r = msgpack_pack_long(__pyx_v_pk, ((long)__pyx_t_13));
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":102
 *             raise OverflowError("Integer value out of range")
 * 
 *     if PyInt_Check(n):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_long(pk, <long> n)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":105
 *         return msgpack_pack_long(pk, <long> n)
 * 
 *     if PyFloat_Check(n):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_double(pk, <double> n)
 * 
 */
  __pyx_t_2 = (PyFloat_Check(__pyx_v_n) != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":106
 * 
 *     if PyFloat_Check(n):
 *         return msgpack_pack_double(pk, <double> n)             # <<<<<<<<<<<<<<
 * 
 *     raise TypeError("Unhandled numeric type: %r" % type(n))
 */
    __pyx_t_14 = __pyx_PyFloat_AsDouble(__pyx_v_n); if (unlikely((__pyx_t_14 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 106, __pyx_L1_error)
    __pyx_r = msgpack_pack_double(__pyx_v_pk, ((double)__pyx_t_14));
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":105
 *         return msgpack_pack_long(pk, <long> n)
 * 
 *     if PyFloat_Check(n):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_double(pk, <double> n)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":108
 *         return msgpack_pack_double(pk, <double> n)
 * 
 *     raise TypeError("Unhandled numeric type: %r" % type(n))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_v_n); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = __Pyx_PyString_Format(__pyx_kp_s_Unhandled_numeric_type_r, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_Raise(__pyx_t_11, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __PYX_ERR(0, 108, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":88
 * 
 * 
 * cdef inline int pack_number(msgpack_packer *pk, object n) except? -1:             # <<<<<<<<<<<<<<
 *     if n is None:
 *         return msgpack_pack_nil(pk)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("ddtrace.internal._encoding.pack_number", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_oe);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":111
 * 
 * 
 * cdef inline int pack_text(msgpack_packer *pk, object text) except? -1:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t L
 *     cdef int ret
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_pack_text(struct msgpack_packer *__pyx_v_pk, PyObject *__pyx_v_text) {
  Py_ssize_t __pyx_v_L;
  int __pyx_v_ret;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4;
  char *__pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_text", 0);

  /* "ddtrace/internal/_encoding.pyx":115
 *     cdef int ret
 * 
 *     if text is None:             # <<<<<<<<<<<<<<
 *         return msgpack_pack_nil(pk)
 * 
 */
  __pyx_t_1 = (__pyx_v_text == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":116
 * 
 *     if text is None:
 *         return msgpack_pack_nil(pk)             # <<<<<<<<<<<<<<
 * 
 *     if PyBytesLike_Check(text):
 */
    __pyx_r = msgpack_pack_nil(__pyx_v_pk);
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":115
 *     cdef int ret
 * 
 *     if text is None:             # <<<<<<<<<<<<<<
 *         return msgpack_pack_nil(pk)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":118
 *         return msgpack_pack_nil(pk)
 * 
 *     if PyBytesLike_Check(text):             # <<<<<<<<<<<<<<
 *         L = len(text)
 *         if L > ITEM_LIMIT:
 */
  __pyx_t_2 = (PyBytesLike_Check(__pyx_v_text) != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":119
 * 
 *     if PyBytesLike_Check(text):
 *         L = len(text)             # <<<<<<<<<<<<<<
 *         if L > ITEM_LIMIT:
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 */
    __pyx_t_3 = PyObject_Length(__pyx_v_text); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 119, __pyx_L1_error)
    __pyx_v_L = __pyx_t_3;

    /* "ddtrace/internal/_encoding.pyx":120
 *     if PyBytesLike_Check(text):
 *         L = len(text)
 *         if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 *         ret = msgpack_pack_raw(pk, L)
 */
    __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":121
 *         L = len(text)
 *         if L > ITEM_LIMIT:
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_raw(pk, L)
 *         if ret == 0:
 */
      __pyx_t_4 = PyErr_Format(__pyx_builtin_ValueError, ((char *)"%.200s object is too large"), Py_TYPE(__pyx_v_text)->tp_name); if (unlikely(__pyx_t_4 == ((PyObject *)NULL))) __PYX_ERR(0, 121, __pyx_L1_error)

      /* "ddtrace/internal/_encoding.pyx":120
 *     if PyBytesLike_Check(text):
 *         L = len(text)
 *         if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 *         ret = msgpack_pack_raw(pk, L)
 */
    }

    /* "ddtrace/internal/_encoding.pyx":122
 *         if L > ITEM_LIMIT:
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 *         ret = msgpack_pack_raw(pk, L)             # <<<<<<<<<<<<<<
 *         if ret == 0:
 *             ret = msgpack_pack_raw_body(pk, <char *> text, L)
 */
    __pyx_v_ret = msgpack_pack_raw(__pyx_v_pk, __pyx_v_L);

    /* "ddtrace/internal/_encoding.pyx":123
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 *         ret = msgpack_pack_raw(pk, L)
 *         if ret == 0:             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_raw_body(pk, <char *> text, L)
 *         return ret
 */
    __pyx_t_2 = ((__pyx_v_ret == 0) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":124
 *         ret = msgpack_pack_raw(pk, L)
 *         if ret == 0:
 *             ret = msgpack_pack_raw_body(pk, <char *> text, L)             # <<<<<<<<<<<<<<
 *         return ret
 * 
 */
      __pyx_t_5 = __Pyx_PyObject_AsWritableString(__pyx_v_text); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 124, __pyx_L1_error)
      __pyx_v_ret = msgpack_pack_raw_body(__pyx_v_pk, ((char *)__pyx_t_5), __pyx_v_L);

      /* "ddtrace/internal/_encoding.pyx":123
 *             PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(text).tp_name)
 *         ret = msgpack_pack_raw(pk, L)
 *         if ret == 0:             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_raw_body(pk, <char *> text, L)
 *         return ret
 */
    }

    /* "ddtrace/internal/_encoding.pyx":125
 *         if ret == 0:
 *             ret = msgpack_pack_raw_body(pk, <char *> text, L)
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     if PyUnicode_Check(text):
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":118
 *         return msgpack_pack_nil(pk)
 * 
 *     if PyBytesLike_Check(text):             # <<<<<<<<<<<<<<
 *         L = len(text)
 *         if L > ITEM_LIMIT:
 */
  }

  /* "ddtrace/internal/_encoding.pyx":127
 *         return ret
 * 
 *     if PyUnicode_Check(text):             # <<<<<<<<<<<<<<
 *         IF PY_MAJOR_VERSION >= 3:
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 */
  __pyx_t_2 = (PyUnicode_Check(__pyx_v_text) != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":129
 *     if PyUnicode_Check(text):
 *         IF PY_MAJOR_VERSION >= 3:
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)             # <<<<<<<<<<<<<<
 *             if ret == -2:
 *                 raise ValueError("unicode string is too large")
 */
    __pyx_v_ret = msgpack_pack_unicode(__pyx_v_pk, __pyx_v_text, __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT);

    /* "ddtrace/internal/_encoding.pyx":130
 *         IF PY_MAJOR_VERSION >= 3:
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 *             if ret == -2:             # <<<<<<<<<<<<<<
 *                 raise ValueError("unicode string is too large")
 *         ELSE:
 */
    __pyx_t_2 = ((__pyx_v_ret == -2L) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "ddtrace/internal/_encoding.pyx":131
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 *             if ret == -2:
 *                 raise ValueError("unicode string is too large")             # <<<<<<<<<<<<<<
 *         ELSE:
 *             text = PyUnicode_AsEncodedString(text, "utf-8", NULL)
 */
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 131, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 131, __pyx_L1_error)

      /* "ddtrace/internal/_encoding.pyx":130
 *         IF PY_MAJOR_VERSION >= 3:
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 *             if ret == -2:             # <<<<<<<<<<<<<<
 *                 raise ValueError("unicode string is too large")
 *         ELSE:
 */
    }

    /* "ddtrace/internal/_encoding.pyx":140
 *             if ret == 0:
 *                 ret = msgpack_pack_raw_body(pk, <char *> text, L)
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     raise TypeError("Unhandled text type: %r" % type(text))
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":127
 *         return ret
 * 
 *     if PyUnicode_Check(text):             # <<<<<<<<<<<<<<
 *         IF PY_MAJOR_VERSION >= 3:
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":142
 *         return ret
 * 
 *     raise TypeError("Unhandled text type: %r" % type(text))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_v_text); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_Unhandled_text_type_r, __pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_7); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 142, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_Raise(__pyx_t_6, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __PYX_ERR(0, 142, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":111
 * 
 * 
 * cdef inline int pack_text(msgpack_packer *pk, object text) except? -1:             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t L
 *     cdef int ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("ddtrace.internal._encoding.pack_text", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":149
 *     cdef stdint.uint32_t _next_id
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._table = {"": 0}
 *         self.insert("")
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable___init__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ddtrace/internal/_encoding.pyx":150
 * 
 *     def __init__(self):
 *         self._table = {"": 0}             # <<<<<<<<<<<<<<
 *         self.insert("")
 *         self._next_id = 1
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_t_1, __pyx_kp_s__3, __pyx_int_0) < 0) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_table);
  __Pyx_DECREF(__pyx_v_self->_table);
  __pyx_v_self->_table = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":151
 *     def __init__(self):
 *         self._table = {"": 0}
 *         self.insert("")             # <<<<<<<<<<<<<<
 *         self._next_id = 1
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->__pyx_vtab)->insert(__pyx_v_self, __pyx_kp_s__3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":152
 *         self._table = {"": 0}
 *         self.insert("")
 *         self._next_id = 1             # <<<<<<<<<<<<<<
 * 
 *     cdef insert(self, object string):
 */
  __pyx_v_self->_next_id = 1;

  /* "ddtrace/internal/_encoding.pyx":149
 *     cdef stdint.uint32_t _next_id
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._table = {"": 0}
 *         self.insert("")
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":154
 *         self._next_id = 1
 * 
 *     cdef insert(self, object string):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_insert(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("insert", 0);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":157
 *         pass
 * 
 *     cdef stdint.uint32_t _index(self, object string):             # <<<<<<<<<<<<<<
 *         cdef stdint.uint32_t _id
 * 
 */

static uint32_t __pyx_f_7ddtrace_8internal_9_encoding_11StringTable__index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string) {
  uint32_t __pyx_v__id;
  uint32_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5;
  long __pyx_t_6;
  uint32_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_index", 0);

  /* "ddtrace/internal/_encoding.pyx":160
 *         cdef stdint.uint32_t _id
 * 
 *         if string is None:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  __pyx_t_1 = (__pyx_v_string == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":161
 * 
 *         if string is None:
 *             return 0             # <<<<<<<<<<<<<<
 * 
 *         if PyDict_Contains(self._table, string):
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":160
 *         cdef stdint.uint32_t _id
 * 
 *         if string is None:             # <<<<<<<<<<<<<<
 *             return 0
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":163
 *             return 0
 * 
 *         if PyDict_Contains(self._table, string):             # <<<<<<<<<<<<<<
 *             return PyLong_AsLong(<object>PyDict_GetItem(self._table, string))
 * 
 */
  __pyx_t_3 = __pyx_v_self->_table;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = PyDict_Contains(__pyx_t_3, __pyx_v_string); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = (__pyx_t_4 != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":164
 * 
 *         if PyDict_Contains(self._table, string):
 *             return PyLong_AsLong(<object>PyDict_GetItem(self._table, string))             # <<<<<<<<<<<<<<
 * 
 *         _id = self._next_id
 */
    __pyx_t_3 = __pyx_v_self->_table;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = PyDict_GetItem(__pyx_t_3, __pyx_v_string);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_6 = PyLong_AsLong(((PyObject *)__pyx_t_5)); if (unlikely(__pyx_t_6 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 164, __pyx_L1_error)
    __pyx_r = __pyx_t_6;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":163
 *             return 0
 * 
 *         if PyDict_Contains(self._table, string):             # <<<<<<<<<<<<<<
 *             return PyLong_AsLong(<object>PyDict_GetItem(self._table, string))
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":166
 *             return PyLong_AsLong(<object>PyDict_GetItem(self._table, string))
 * 
 *         _id = self._next_id             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(self._table, string, PyLong_FromLong(_id))
 *         self.insert(string)
 */
  __pyx_t_7 = __pyx_v_self->_next_id;
  __pyx_v__id = __pyx_t_7;

  /* "ddtrace/internal/_encoding.pyx":167
 * 
 *         _id = self._next_id
 *         PyDict_SetItem(self._table, string, PyLong_FromLong(_id))             # <<<<<<<<<<<<<<
 *         self.insert(string)
 *         self._next_id += 1
 */
  __pyx_t_3 = __pyx_v_self->_table;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_8 = PyLong_FromLong(__pyx_v__id); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = PyDict_SetItem(__pyx_t_3, __pyx_v_string, __pyx_t_8); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "ddtrace/internal/_encoding.pyx":168
 *         _id = self._next_id
 *         PyDict_SetItem(self._table, string, PyLong_FromLong(_id))
 *         self.insert(string)             # <<<<<<<<<<<<<<
 *         self._next_id += 1
 *         return _id
 */
  __pyx_t_8 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->__pyx_vtab)->insert(__pyx_v_self, __pyx_v_string); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 168, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "ddtrace/internal/_encoding.pyx":169
 *         PyDict_SetItem(self._table, string, PyLong_FromLong(_id))
 *         self.insert(string)
 *         self._next_id += 1             # <<<<<<<<<<<<<<
 *         return _id
 * 
 */
  __pyx_v_self->_next_id = (__pyx_v_self->_next_id + 1);

  /* "ddtrace/internal/_encoding.pyx":170
 *         self.insert(string)
 *         self._next_id += 1
 *         return _id             # <<<<<<<<<<<<<<
 * 
 *     cpdef index(self, object string):
 */
  __pyx_r = __pyx_v__id;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":157
 *         pass
 * 
 *     cdef stdint.uint32_t _index(self, object string):             # <<<<<<<<<<<<<<
 *         cdef stdint.uint32_t _id
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("ddtrace.internal._encoding.StringTable._index", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":172
 *         return _id
 * 
 *     cpdef index(self, object string):             # <<<<<<<<<<<<<<
 *         return self._index(string)
 * 
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_3index(PyObject *__pyx_v_self, PyObject *__pyx_v_string); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("index", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_3index)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_string) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_string);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 172, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":173
 * 
 *     cpdef index(self, object string):
 *         return self._index(string)             # <<<<<<<<<<<<<<
 * 
 *     cdef reset(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_uint32_t(((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->__pyx_vtab)->_index(__pyx_v_self, __pyx_v_string)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":172
 *         return _id
 * 
 *     cpdef index(self, object string):             # <<<<<<<<<<<<<<
 *         return self._index(string)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_3index(PyObject *__pyx_v_self, PyObject *__pyx_v_string); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_3index(PyObject *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("index (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_2index(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self), ((PyObject *)__pyx_v_string));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_2index(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("index", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_11StringTable_index(__pyx_v_self, __pyx_v_string, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":175
 *         return self._index(string)
 * 
 *     cdef reset(self):             # <<<<<<<<<<<<<<
 *         PyDict_Clear(self._table)
 *         PyDict_SetItem(self._table, "", 0)
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_reset(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reset", 0);

  /* "ddtrace/internal/_encoding.pyx":176
 * 
 *     cdef reset(self):
 *         PyDict_Clear(self._table)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(self._table, "", 0)
 *         self.insert("")
 */
  __pyx_t_1 = __pyx_v_self->_table;
  __Pyx_INCREF(__pyx_t_1);
  PyDict_Clear(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":177
 *     cdef reset(self):
 *         PyDict_Clear(self._table)
 *         PyDict_SetItem(self._table, "", 0)             # <<<<<<<<<<<<<<
 *         self.insert("")
 *         self._next_id = 1
 */
  __pyx_t_1 = __pyx_v_self->_table;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyDict_SetItem(__pyx_t_1, __pyx_kp_s__3, __pyx_int_0); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":178
 *         PyDict_Clear(self._table)
 *         PyDict_SetItem(self._table, "", 0)
 *         self.insert("")             # <<<<<<<<<<<<<<
 *         self._next_id = 1
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->__pyx_vtab)->insert(__pyx_v_self, __pyx_kp_s__3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":179
 *         PyDict_SetItem(self._table, "", 0)
 *         self.insert("")
 *         self._next_id = 1             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __pyx_v_self->_next_id = 1;

  /* "ddtrace/internal/_encoding.pyx":175
 *         return self._index(string)
 * 
 *     cdef reset(self):             # <<<<<<<<<<<<<<
 *         PyDict_Clear(self._table)
 *         PyDict_SetItem(self._table, "", 0)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":181
 *         self._next_id = 1
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return PyLong_FromLong(self._next_id)
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_5__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_5__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_4__len__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_4__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ddtrace/internal/_encoding.pyx":182
 * 
 *     def __len__(self):
 *         return PyLong_FromLong(self._next_id)             # <<<<<<<<<<<<<<
 * 
 *     def __contains__(self, object string):
 */
  __pyx_t_1 = PyLong_FromLong(__pyx_v_self->_next_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":181
 *         self._next_id = 1
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return PyLong_FromLong(self._next_id)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":184
 *         return PyLong_FromLong(self._next_id)
 * 
 *     def __contains__(self, object string):             # <<<<<<<<<<<<<<
 *         return PyBool_FromLong(PyDict_Contains(self._table, string))
 * 
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_7__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_string); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_7__contains__(PyObject *__pyx_v_self, PyObject *__pyx_v_string) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__contains__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_6__contains__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self), ((PyObject *)__pyx_v_string));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_6__contains__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v_string) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__contains__", 0);

  /* "ddtrace/internal/_encoding.pyx":185
 * 
 *     def __contains__(self, object string):
 *         return PyBool_FromLong(PyDict_Contains(self._table, string))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __pyx_v_self->_table;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyDict_Contains(__pyx_t_1, __pyx_v_string); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":184
 *         return PyLong_FromLong(self._next_id)
 * 
 *     def __contains__(self, object string):             # <<<<<<<<<<<<<<
 *         return PyBool_FromLong(PyDict_Contains(self._table, string))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.__contains__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_8__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_8__reduce_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._next_id, self._table)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_uint32_t(__pyx_v_self->_next_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->_table);
  __Pyx_GIVEREF(__pyx_v_self->_table);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->_table);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._table is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._table is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->_table != ((PyObject*)Py_None));
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._table is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self._table is not None
 *     if use_setstate:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_StringTable); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_int_194841817);
    __Pyx_GIVEREF(__pyx_int_194841817);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_194841817);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._table is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, None), state
 *     else:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StringTable__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_StringTable); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
    __Pyx_INCREF(__pyx_int_194841817);
    __Pyx_GIVEREF(__pyx_int_194841817);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_194841817);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StringTable__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_10__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_11StringTable_10__setstate_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StringTable__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_StringTable__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StringTable, (type(self), 0xb9d0cd9, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StringTable__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.StringTable.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":191
 *     cdef list _list
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._list = []
 *         super(ListStringTable, self).__init__()
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__init__", 0))) return -1;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable___init__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ddtrace/internal/_encoding.pyx":192
 * 
 *     def __init__(self):
 *         self._list = []             # <<<<<<<<<<<<<<
 *         super(ListStringTable, self).__init__()
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 192, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_list);
  __Pyx_DECREF(__pyx_v_self->_list);
  __pyx_v_self->_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":193
 *     def __init__(self):
 *         self._list = []
 *         super(ListStringTable, self).__init__()             # <<<<<<<<<<<<<<
 * 
 *     cdef insert(self, object string):
 */
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":191
 *     cdef list _list
 * 
 *     def __init__(self):             # <<<<<<<<<<<<<<
 *         self._list = []
 *         super(ListStringTable, self).__init__()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListStringTable.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":195
 *         super(ListStringTable, self).__init__()
 * 
 *     cdef insert(self, object string):             # <<<<<<<<<<<<<<
 *         PyList_Append(self._list, string)
 * 
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_15ListStringTable_insert(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self, PyObject *__pyx_v_string) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert", 0);

  /* "ddtrace/internal/_encoding.pyx":196
 * 
 *     cdef insert(self, object string):
 *         PyList_Append(self._list, string)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __pyx_t_1 = __pyx_v_self->_list;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_Append(__pyx_t_1, __pyx_v_string); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 196, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":195
 *         super(ListStringTable, self).__init__()
 * 
 *     cdef insert(self, object string):             # <<<<<<<<<<<<<<
 *         PyList_Append(self._list, string)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListStringTable.insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":198
 *         PyList_Append(self._list, string)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self._list)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_3__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_2__iter__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_2__iter__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "ddtrace/internal/_encoding.pyx":199
 * 
 *     def __iter__(self):
 *         return iter(self._list)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->_list;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 199, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":198
 *         PyList_Append(self._list, string)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self._list)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListStringTable.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_5__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_4__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_4__reduce_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._list, self._next_id, self._table)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_uint32_t(__pyx_v_self->__pyx_base._next_id); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_list);
  __Pyx_GIVEREF(__pyx_v_self->_list);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_self->_list);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._table);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._table);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_self->__pyx_base._table);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._list, self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._list, self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._list is not None or self._table is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._list, self._next_id, self._table)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._list is not None or self._table is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, None), state
 */
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->_list != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._table != ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._list is not None or self._table is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self._list is not None or self._table is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_ListStringTable); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __Pyx_INCREF(__pyx_int_182609841);
    __Pyx_GIVEREF(__pyx_int_182609841);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_int_182609841);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_6, 2, Py_None);
    __pyx_t_2 = 0;
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_6);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_6 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._list is not None or self._table is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, None), state
 *     else:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ListStringTable__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_ListStringTable); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_INCREF(__pyx_int_182609841);
    __Pyx_GIVEREF(__pyx_int_182609841);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_182609841);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_6 = 0;
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListStringTable.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ListStringTable__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_7__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_6__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15ListStringTable_6__setstate_cython__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ListStringTable__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_ListStringTable__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ListStringTable, (type(self), 0xae267b1, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ListStringTable__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListStringTable.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":210
 *     cdef size_t _reset_size
 * 
 *     def __init__(self, max_size):             # <<<<<<<<<<<<<<
 *         self.pk.buf_size = min(max_size, 1 << 20)
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_max_size = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_size,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 210, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_max_size = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 210, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable___init__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self), __pyx_v_max_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable___init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, PyObject *__pyx_v_max_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  size_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ddtrace/internal/_encoding.pyx":211
 * 
 *     def __init__(self, max_size):
 *         self.pk.buf_size = min(max_size, 1 << 20)             # <<<<<<<<<<<<<<
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 *         if self.pk.buf == NULL:
 */
  __pyx_t_1 = 0x100000;
  __Pyx_INCREF(__pyx_v_max_size);
  __pyx_t_2 = __pyx_v_max_size;
  __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__pyx_t_6) {
    __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_t_5;
    __pyx_t_5 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = __pyx_t_2;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __Pyx_PyInt_As_size_t(__pyx_t_3); if (unlikely((__pyx_t_7 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->pk.buf_size = __pyx_t_7;

  /* "ddtrace/internal/_encoding.pyx":212
 *     def __init__(self, max_size):
 *         self.pk.buf_size = min(max_size, 1 << 20)
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)             # <<<<<<<<<<<<<<
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")
 */
  __pyx_v_self->pk.buf = ((char *)PyMem_Malloc(__pyx_v_self->pk.buf_size));

  /* "ddtrace/internal/_encoding.pyx":213
 *         self.pk.buf_size = min(max_size, 1 << 20)
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.max_size = max_size
 */
  __pyx_t_6 = ((__pyx_v_self->pk.buf == NULL) != 0);
  if (unlikely(__pyx_t_6)) {

    /* "ddtrace/internal/_encoding.pyx":214
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")             # <<<<<<<<<<<<<<
 *         self.max_size = max_size
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 214, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":213
 *         self.pk.buf_size = min(max_size, 1 << 20)
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.max_size = max_size
 */
  }

  /* "ddtrace/internal/_encoding.pyx":215
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.max_size = max_size             # <<<<<<<<<<<<<<
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE
 *         self._sp_len = 0
 */
  __pyx_t_8 = __Pyx_PyInt_As_int(__pyx_v_max_size); if (unlikely((__pyx_t_8 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 215, __pyx_L1_error)
  __pyx_v_self->max_size = __pyx_t_8;

  /* "ddtrace/internal/_encoding.pyx":216
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.max_size = max_size
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE             # <<<<<<<<<<<<<<
 *         self._sp_len = 0
 *         self._lock = threading.RLock()
 */
  __pyx_v_self->pk.length = 6;

  /* "ddtrace/internal/_encoding.pyx":217
 *         self.max_size = max_size
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE
 *         self._sp_len = 0             # <<<<<<<<<<<<<<
 *         self._lock = threading.RLock()
 *         super(MsgpackStringTable, self).__init__()
 */
  __pyx_v_self->_sp_len = 0;

  /* "ddtrace/internal/_encoding.pyx":218
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE
 *         self._sp_len = 0
 *         self._lock = threading.RLock()             # <<<<<<<<<<<<<<
 *         super(MsgpackStringTable, self).__init__()
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_RLock); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 218, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->_lock);
  __Pyx_DECREF(__pyx_v_self->_lock);
  __pyx_v_self->_lock = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "ddtrace/internal/_encoding.pyx":219
 *         self._sp_len = 0
 *         self._lock = threading.RLock()
 *         super(MsgpackStringTable, self).__init__()             # <<<<<<<<<<<<<<
 * 
 *         assert self.index(ORIGIN_KEY) == 1
 */
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable));
  PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_5, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "ddtrace/internal/_encoding.pyx":221
 *         super(MsgpackStringTable, self).__init__()
 * 
 *         assert self.index(ORIGIN_KEY) == 1             # <<<<<<<<<<<<<<
 *         self._reset_size = self.pk.length
 * 
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_ORIGIN_KEY); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.index(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self), __pyx_t_3, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyInt_EqObjC(__pyx_t_5, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_6)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 221, __pyx_L1_error)
    }
  }
  #endif

  /* "ddtrace/internal/_encoding.pyx":222
 * 
 *         assert self.index(ORIGIN_KEY) == 1
 *         self._reset_size = self.pk.length             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_7 = __pyx_v_self->pk.length;
  __pyx_v_self->_reset_size = __pyx_t_7;

  /* "ddtrace/internal/_encoding.pyx":210
 *     cdef size_t _reset_size
 * 
 *     def __init__(self, max_size):             # <<<<<<<<<<<<<<
 *         self.pk.buf_size = min(max_size, 1 << 20)
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":224
 *         self._reset_size = self.pk.length
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

/* Python wrapper */
static void __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_2__dealloc__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_2__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ddtrace/internal/_encoding.pyx":225
 * 
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)             # <<<<<<<<<<<<<<
 *         self.pk.buf = NULL
 * 
 */
  PyMem_Free(__pyx_v_self->pk.buf);

  /* "ddtrace/internal/_encoding.pyx":226
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef insert(self, object string):
 */
  __pyx_v_self->pk.buf = NULL;

  /* "ddtrace/internal/_encoding.pyx":224
 *         self._reset_size = self.pk.length
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "ddtrace/internal/_encoding.pyx":228
 *         self.pk.buf = NULL
 * 
 *     cdef insert(self, object string):             # <<<<<<<<<<<<<<
 *         cdef int ret
 * 
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_insert(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, PyObject *__pyx_v_string) {
  int __pyx_v_ret;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("insert", 0);

  /* "ddtrace/internal/_encoding.pyx":231
 *         cdef int ret
 * 
 *         if self.pk.length + len(string) > self.max_size:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "Cannot insert '%s': string table is full (current size: %d, max size: %d)." % (
 */
  __pyx_t_1 = PyObject_Length(__pyx_v_string); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 231, __pyx_L1_error)
  __pyx_t_2 = (((__pyx_v_self->pk.length + __pyx_t_1) > __pyx_v_self->max_size) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ddtrace/internal/_encoding.pyx":234
 *             raise ValueError(
 *                 "Cannot insert '%s': string table is full (current size: %d, max size: %d)." % (
 *                     string, self.pk.length, self.max_size             # <<<<<<<<<<<<<<
 *                 )
 *             )
 */
    __pyx_t_3 = __Pyx_PyInt_FromSize_t(__pyx_v_self->pk.length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_self->max_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 234, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v_string);
    __Pyx_GIVEREF(__pyx_v_string);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_string);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;

    /* "ddtrace/internal/_encoding.pyx":233
 *         if self.pk.length + len(string) > self.max_size:
 *             raise ValueError(
 *                 "Cannot insert '%s': string table is full (current size: %d, max size: %d)." % (             # <<<<<<<<<<<<<<
 *                     string, self.pk.length, self.max_size
 *                 )
 */
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Cannot_insert_s_string_table_is, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ddtrace/internal/_encoding.pyx":232
 * 
 *         if self.pk.length + len(string) > self.max_size:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "Cannot insert '%s': string table is full (current size: %d, max size: %d)." % (
 *                     string, self.pk.length, self.max_size
 */
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 232, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 232, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":231
 *         cdef int ret
 * 
 *         if self.pk.length + len(string) > self.max_size:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "Cannot insert '%s': string table is full (current size: %d, max size: %d)." % (
 */
  }

  /* "ddtrace/internal/_encoding.pyx":238
 *             )
 * 
 *         ret = pack_text(&self.pk, string)             # <<<<<<<<<<<<<<
 *         if ret != 0:
 *             raise RuntimeError("Failed to add string to msgpack string table")
 */
  __pyx_t_6 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->pk), __pyx_v_string); if (unlikely(__pyx_t_6 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 238, __pyx_L1_error)
  __pyx_v_ret = __pyx_t_6;

  /* "ddtrace/internal/_encoding.pyx":239
 * 
 *         ret = pack_text(&self.pk, string)
 *         if ret != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to add string to msgpack string table")
 * 
 */
  __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ddtrace/internal/_encoding.pyx":240
 *         ret = pack_text(&self.pk, string)
 *         if ret != 0:
 *             raise RuntimeError("Failed to add string to msgpack string table")             # <<<<<<<<<<<<<<
 * 
 *     cdef savepoint(self):
 */
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 240, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 240, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":239
 * 
 *         ret = pack_text(&self.pk, string)
 *         if ret != 0:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Failed to add string to msgpack string table")
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":228
 *         self.pk.buf = NULL
 * 
 *     cdef insert(self, object string):             # <<<<<<<<<<<<<<
 *         cdef int ret
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.insert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":242
 *             raise RuntimeError("Failed to add string to msgpack string table")
 * 
 *     cdef savepoint(self):             # <<<<<<<<<<<<<<
 *         self._sp_len = self.pk.length
 *         self._sp_id = self._next_id
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_savepoint(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  uint32_t __pyx_t_2;
  __Pyx_RefNannySetupContext("savepoint", 0);

  /* "ddtrace/internal/_encoding.pyx":243
 * 
 *     cdef savepoint(self):
 *         self._sp_len = self.pk.length             # <<<<<<<<<<<<<<
 *         self._sp_id = self._next_id
 * 
 */
  __pyx_t_1 = __pyx_v_self->pk.length;
  __pyx_v_self->_sp_len = __pyx_t_1;

  /* "ddtrace/internal/_encoding.pyx":244
 *     cdef savepoint(self):
 *         self._sp_len = self.pk.length
 *         self._sp_id = self._next_id             # <<<<<<<<<<<<<<
 * 
 *     cdef rollback(self):
 */
  __pyx_t_2 = __pyx_v_self->__pyx_base._next_id;
  __pyx_v_self->_sp_id = __pyx_t_2;

  /* "ddtrace/internal/_encoding.pyx":242
 *             raise RuntimeError("Failed to add string to msgpack string table")
 * 
 *     cdef savepoint(self):             # <<<<<<<<<<<<<<
 *         self._sp_len = self.pk.length
 *         self._sp_id = self._next_id
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":246
 *         self._sp_id = self._next_id
 * 
 *     cdef rollback(self):             # <<<<<<<<<<<<<<
 *         if self._sp_len > 0:
 *             self.pk.length = self._sp_len
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_rollback(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  uint32_t __pyx_t_3;
  __Pyx_RefNannySetupContext("rollback", 0);

  /* "ddtrace/internal/_encoding.pyx":247
 * 
 *     cdef rollback(self):
 *         if self._sp_len > 0:             # <<<<<<<<<<<<<<
 *             self.pk.length = self._sp_len
 *             self._next_id = self._sp_id
 */
  __pyx_t_1 = ((__pyx_v_self->_sp_len > 0) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":248
 *     cdef rollback(self):
 *         if self._sp_len > 0:
 *             self.pk.length = self._sp_len             # <<<<<<<<<<<<<<
 *             self._next_id = self._sp_id
 * 
 */
    __pyx_t_2 = __pyx_v_self->_sp_len;
    __pyx_v_self->pk.length = __pyx_t_2;

    /* "ddtrace/internal/_encoding.pyx":249
 *         if self._sp_len > 0:
 *             self.pk.length = self._sp_len
 *             self._next_id = self._sp_id             # <<<<<<<<<<<<<<
 * 
 *     cdef get_bytes(self):
 */
    __pyx_t_3 = __pyx_v_self->_sp_id;
    __pyx_v_self->__pyx_base._next_id = __pyx_t_3;

    /* "ddtrace/internal/_encoding.pyx":247
 * 
 *     cdef rollback(self):
 *         if self._sp_len > 0:             # <<<<<<<<<<<<<<
 *             self.pk.length = self._sp_len
 *             self._next_id = self._sp_id
 */
  }

  /* "ddtrace/internal/_encoding.pyx":246
 *         self._sp_id = self._next_id
 * 
 *     cdef rollback(self):             # <<<<<<<<<<<<<<
 *         if self._sp_len > 0:
 *             self.pk.length = self._sp_len
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":251
 *             self._next_id = self._sp_id
 * 
 *     cdef get_bytes(self):             # <<<<<<<<<<<<<<
 *         cdef int ret;
 *         cdef stdint.uint32_t l = self._next_id
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_get_bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  int __pyx_v_ret;
  uint32_t __pyx_v_l;
  int __pyx_v_offset;
  int __pyx_v_old_pos;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  uint32_t __pyx_t_1;
  size_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  long __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_bytes", 0);

  /* "ddtrace/internal/_encoding.pyx":253
 *     cdef get_bytes(self):
 *         cdef int ret;
 *         cdef stdint.uint32_t l = self._next_id             # <<<<<<<<<<<<<<
 *         cdef int offset = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE - array_prefix_size(l)
 *         cdef int old_pos = self.pk.length
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base._next_id;
  __pyx_v_l = __pyx_t_1;

  /* "ddtrace/internal/_encoding.pyx":254
 *         cdef int ret;
 *         cdef stdint.uint32_t l = self._next_id
 *         cdef int offset = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE - array_prefix_size(l)             # <<<<<<<<<<<<<<
 *         cdef int old_pos = self.pk.length
 * 
 */
  __pyx_v_offset = (6 - __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(__pyx_v_l));

  /* "ddtrace/internal/_encoding.pyx":255
 *         cdef stdint.uint32_t l = self._next_id
 *         cdef int offset = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE - array_prefix_size(l)
 *         cdef int old_pos = self.pk.length             # <<<<<<<<<<<<<<
 * 
 *         with self._lock:
 */
  __pyx_t_2 = __pyx_v_self->pk.length;
  __pyx_v_old_pos = __pyx_t_2;

  /* "ddtrace/internal/_encoding.pyx":257
 *         cdef int old_pos = self.pk.length
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             # Update table size prefix
 *             self.pk.length = offset
 */
  /*with:*/ {
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 257, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 257, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 257, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    /*try:*/ {
      {
        (void)__pyx_t_7; (void)__pyx_t_8; (void)__pyx_t_9; /* mark used */
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":259
 *         with self._lock:
 *             # Update table size prefix
 *             self.pk.length = offset             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_array(&self.pk, l)
 *             if ret:
 */
          __pyx_v_self->pk.length = __pyx_v_offset;

          /* "ddtrace/internal/_encoding.pyx":260
 *             # Update table size prefix
 *             self.pk.length = offset
 *             ret = msgpack_pack_array(&self.pk, l)             # <<<<<<<<<<<<<<
 *             if ret:
 *                 return None
 */
          __pyx_v_ret = msgpack_pack_array((&__pyx_v_self->pk), __pyx_v_l);

          /* "ddtrace/internal/_encoding.pyx":261
 *             self.pk.length = offset
 *             ret = msgpack_pack_array(&self.pk, l)
 *             if ret:             # <<<<<<<<<<<<<<
 *                 return None
 *             # Add root array size prefix
 */
          __pyx_t_10 = (__pyx_v_ret != 0);
          if (__pyx_t_10) {

            /* "ddtrace/internal/_encoding.pyx":262
 *             ret = msgpack_pack_array(&self.pk, l)
 *             if ret:
 *                 return None             # <<<<<<<<<<<<<<
 *             # Add root array size prefix
 *             self.pk.length = offset = offset - 1
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_r = Py_None; __Pyx_INCREF(Py_None);
            goto __pyx_L11_try_return;

            /* "ddtrace/internal/_encoding.pyx":261
 *             self.pk.length = offset
 *             ret = msgpack_pack_array(&self.pk, l)
 *             if ret:             # <<<<<<<<<<<<<<
 *                 return None
 *             # Add root array size prefix
 */
          }

          /* "ddtrace/internal/_encoding.pyx":264
 *                 return None
 *             # Add root array size prefix
 *             self.pk.length = offset = offset - 1             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_array(&self.pk, 2)
 *             if ret:
 */
          __pyx_t_11 = (__pyx_v_offset - 1);
          __pyx_v_self->pk.length = __pyx_t_11;
          __pyx_v_offset = __pyx_t_11;

          /* "ddtrace/internal/_encoding.pyx":265
 *             # Add root array size prefix
 *             self.pk.length = offset = offset - 1
 *             ret = msgpack_pack_array(&self.pk, 2)             # <<<<<<<<<<<<<<
 *             if ret:
 *                 return None
 */
          __pyx_v_ret = msgpack_pack_array((&__pyx_v_self->pk), 2);

          /* "ddtrace/internal/_encoding.pyx":266
 *             self.pk.length = offset = offset - 1
 *             ret = msgpack_pack_array(&self.pk, 2)
 *             if ret:             # <<<<<<<<<<<<<<
 *                 return None
 *             self.pk.length = old_pos
 */
          __pyx_t_10 = (__pyx_v_ret != 0);
          if (__pyx_t_10) {

            /* "ddtrace/internal/_encoding.pyx":267
 *             ret = msgpack_pack_array(&self.pk, 2)
 *             if ret:
 *                 return None             # <<<<<<<<<<<<<<
 *             self.pk.length = old_pos
 * 
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_r = Py_None; __Pyx_INCREF(Py_None);
            goto __pyx_L11_try_return;

            /* "ddtrace/internal/_encoding.pyx":266
 *             self.pk.length = offset = offset - 1
 *             ret = msgpack_pack_array(&self.pk, 2)
 *             if ret:             # <<<<<<<<<<<<<<
 *                 return None
 *             self.pk.length = old_pos
 */
          }

          /* "ddtrace/internal/_encoding.pyx":268
 *             if ret:
 *                 return None
 *             self.pk.length = old_pos             # <<<<<<<<<<<<<<
 * 
 *         return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 */
          __pyx_v_self->pk.length = __pyx_v_old_pos;

          /* "ddtrace/internal/_encoding.pyx":257
 *         cdef int old_pos = self.pk.length
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             # Update table size prefix
 *             self.pk.length = offset
 */
        }
        goto __pyx_L12_try_end;
        __pyx_L11_try_return:;
        goto __pyx_L4_return;
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_3) {
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 257, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_9 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_3) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 257, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_r = __pyx_t_9;
        __pyx_t_9 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L15;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L1_error;
    __pyx_L15:;
  }

  /* "ddtrace/internal/_encoding.pyx":270
 *             self.pk.length = old_pos
 * 
 *         return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyBytes_FromStringAndSize((__pyx_v_self->pk.buf + __pyx_v_offset), (__pyx_v_self->pk.length - __pyx_v_offset)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":251
 *             self._next_id = self._sp_id
 * 
 *     cdef get_bytes(self):             # <<<<<<<<<<<<<<
 *         cdef int ret;
 *         cdef stdint.uint32_t l = self._next_id
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.get_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":273
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ddtrace/internal/_encoding.pyx":274
 *     @property
 *     def size(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 274, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 274, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":275
 *     def size(self):
 *         with self._lock:
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)             # <<<<<<<<<<<<<<
 * 
 *     cdef append_raw(self, long src, Py_ssize_t size):
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = __Pyx_PyInt_FromSize_t(((__pyx_v_self->pk.length - 5) + __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(__pyx_v_self->__pyx_base._next_id))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":274
 *     @property
 *     def size(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 * 
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 274, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 274, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 274, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 274, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 274, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 274, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 274, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "ddtrace/internal/_encoding.pyx":273
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":277
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 * 
 *     cdef append_raw(self, long src, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         cdef int res
 *         with self._lock:
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_append_raw(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, long __pyx_v_src, Py_ssize_t __pyx_v_size) {
  int __pyx_v_res;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  long __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append_raw", 0);

  /* "ddtrace/internal/_encoding.pyx":279
 *     cdef append_raw(self, long src, Py_ssize_t size):
 *         cdef int res
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             assert self.size + size <= self.max_size
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 279, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 279, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":280
 *         cdef int res
 *         with self._lock:
 *             assert self.size + size <= self.max_size             # <<<<<<<<<<<<<<
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 *             if res != 0:
 */
          #ifndef CYTHON_WITHOUT_ASSERTIONS
          if (unlikely(!Py_OptimizeFlag)) {
            __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = PyInt_FromSsize_t(__pyx_v_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_4);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_self->max_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_2 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_LE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 280, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            if (unlikely(!__pyx_t_8)) {
              PyErr_SetNone(PyExc_AssertionError);
              __PYX_ERR(0, 280, __pyx_L7_error)
            }
          }
          #endif

          /* "ddtrace/internal/_encoding.pyx":281
 *         with self._lock:
 *             assert self.size + size <= self.max_size
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)             # <<<<<<<<<<<<<<
 *             if res != 0:
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")
 */
          __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v_src); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_9 = PyLong_AsLong(__pyx_t_2); if (unlikely(__pyx_t_9 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 281, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_v_res = msgpack_pack_raw_body((&__pyx_v_self->pk), ((char *)__pyx_t_9), __pyx_v_size);

          /* "ddtrace/internal/_encoding.pyx":282
 *             assert self.size + size <= self.max_size
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 *             if res != 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")
 * 
 */
          __pyx_t_8 = ((__pyx_v_res != 0) != 0);
          if (unlikely(__pyx_t_8)) {

            /* "ddtrace/internal/_encoding.pyx":283
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 *             if res != 0:
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")             # <<<<<<<<<<<<<<
 * 
 *     cdef reset(self):
 */
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L7_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 283, __pyx_L7_error)

            /* "ddtrace/internal/_encoding.pyx":282
 *             assert self.size + size <= self.max_size
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 *             if res != 0:             # <<<<<<<<<<<<<<
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")
 * 
 */
          }

          /* "ddtrace/internal/_encoding.pyx":279
 *     cdef append_raw(self, long src, Py_ssize_t size):
 *         cdef int res
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             assert self.size + size <= self.max_size
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 */
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.append_raw", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 279, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 279, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 279, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_8 < 0) __PYX_ERR(0, 279, __pyx_L9_except_error)
          __pyx_t_12 = ((!(__pyx_t_8 != 0)) != 0);
          if (__pyx_t_12) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 279, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 279, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "ddtrace/internal/_encoding.pyx":277
 *             return self.pk.length - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE + array_prefix_size(self._next_id)
 * 
 *     cdef append_raw(self, long src, Py_ssize_t size):             # <<<<<<<<<<<<<<
 *         cdef int res
 *         with self._lock:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.append_raw", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":285
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")
 * 
 *     cdef reset(self):             # <<<<<<<<<<<<<<
 *         StringTable.reset(self)
 *         assert self._next_id == 1
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_reset(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("reset", 0);

  /* "ddtrace/internal/_encoding.pyx":286
 * 
 *     cdef reset(self):
 *         StringTable.reset(self)             # <<<<<<<<<<<<<<
 *         assert self._next_id == 1
 * 
 */
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_11StringTable_reset(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":287
 *     cdef reset(self):
 *         StringTable.reset(self)
 *         assert self._next_id == 1             # <<<<<<<<<<<<<<
 * 
 *         PyDict_SetItem(self._table, ORIGIN_KEY, 1)
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!((__pyx_v_self->__pyx_base._next_id == 1) != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 287, __pyx_L1_error)
    }
  }
  #endif

  /* "ddtrace/internal/_encoding.pyx":289
 *         assert self._next_id == 1
 * 
 *         PyDict_SetItem(self._table, ORIGIN_KEY, 1)             # <<<<<<<<<<<<<<
 *         self._next_id = 2
 *         self.pk.length = self._reset_size
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base._table;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_ORIGIN_KEY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyDict_SetItem(__pyx_t_1, __pyx_t_2, __pyx_int_1); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":290
 * 
 *         PyDict_SetItem(self._table, ORIGIN_KEY, 1)
 *         self._next_id = 2             # <<<<<<<<<<<<<<
 *         self.pk.length = self._reset_size
 *         self._sp_len = 0
 */
  __pyx_v_self->__pyx_base._next_id = 2;

  /* "ddtrace/internal/_encoding.pyx":291
 *         PyDict_SetItem(self._table, ORIGIN_KEY, 1)
 *         self._next_id = 2
 *         self.pk.length = self._reset_size             # <<<<<<<<<<<<<<
 *         self._sp_len = 0
 * 
 */
  __pyx_t_4 = __pyx_v_self->_reset_size;
  __pyx_v_self->pk.length = __pyx_t_4;

  /* "ddtrace/internal/_encoding.pyx":292
 *         self._next_id = 2
 *         self.pk.length = self._reset_size
 *         self._sp_len = 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef flush(self):
 */
  __pyx_v_self->_sp_len = 0;

  /* "ddtrace/internal/_encoding.pyx":285
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")
 * 
 *     cdef reset(self):             # <<<<<<<<<<<<<<
 *         StringTable.reset(self)
 *         assert self._next_id == 1
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.reset", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":294
 *         self._sp_len = 0
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_5flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  int __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_5flush)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":295
 * 
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return self.get_bytes()
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->_lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":296
 *     cpdef flush(self):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 return self.get_bytes()
 *             finally:
 */
          /*try:*/ {

            /* "ddtrace/internal/_encoding.pyx":297
 *         with self._lock:
 *             try:
 *                 return self.get_bytes()             # <<<<<<<<<<<<<<
 *             finally:
 *                 self.reset()
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_bytes(__pyx_v_self); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_r = __pyx_t_1;
            __pyx_t_1 = 0;
            goto __pyx_L13_return;
          }

          /* "ddtrace/internal/_encoding.pyx":299
 *                 return self.get_bytes()
 *             finally:
 *                 self.reset()             # <<<<<<<<<<<<<<
 * 
 * 
 */
          /*finally:*/ {
            __pyx_L14_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
              if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_15);
              __Pyx_XGOTREF(__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_17);
              __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
              {
                __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reset(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L17_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              }
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
              goto __pyx_L7_error;
              __pyx_L17_error:;
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              goto __pyx_L7_error;
            }
            __pyx_L13_return: {
              __pyx_t_17 = __pyx_r;
              __pyx_r = 0;
              __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.reset(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_r = __pyx_t_17;
              __pyx_t_17 = 0;
              goto __pyx_L11_try_return;
            }
          }

          /* "ddtrace/internal/_encoding.pyx":295
 * 
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return self.get_bytes()
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 295, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 295, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 295, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (__pyx_t_18 < 0) __PYX_ERR(0, 295, __pyx_L9_except_error)
          __pyx_t_19 = ((!(__pyx_t_18 != 0)) != 0);
          if (__pyx_t_19) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 295, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 295, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 295, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "ddtrace/internal/_encoding.pyx":294
 *         self._sp_len = 0
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_5flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_5flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("flush (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4flush(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_flush(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_6__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_8__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackStringTable_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackStringTable.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":309
 *     cdef object _lock
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self.max_size = max_size
 *         self.max_item_size = max_item_size
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_max_size;
  size_t __pyx_v_max_item_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_size,&__pyx_n_s_max_item_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_item_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(0, 309, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 309, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_max_size = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_max_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L3_error)
    __pyx_v_max_item_size = __Pyx_PyInt_As_size_t(values[1]); if (unlikely((__pyx_v_max_item_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 309, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder___cinit__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self), __pyx_v_max_size, __pyx_v_max_item_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, size_t __pyx_v_max_size, size_t __pyx_v_max_item_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ddtrace/internal/_encoding.pyx":310
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         self.max_size = max_size             # <<<<<<<<<<<<<<
 *         self.max_item_size = max_item_size
 *         self._lock = threading.Lock()
 */
  __pyx_v_self->max_size = __pyx_v_max_size;

  /* "ddtrace/internal/_encoding.pyx":311
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         self.max_size = max_size
 *         self.max_item_size = max_item_size             # <<<<<<<<<<<<<<
 *         self._lock = threading.Lock()
 * 
 */
  __pyx_v_self->max_item_size = __pyx_v_max_item_size;

  /* "ddtrace/internal/_encoding.pyx":312
 *         self.max_size = max_size
 *         self.max_item_size = max_item_size
 *         self._lock = threading.Lock()             # <<<<<<<<<<<<<<
 * 
 *     # ---- Abstract methods ----
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_threading); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Lock); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_lock);
  __Pyx_DECREF(__pyx_v_self->_lock);
  __pyx_v_self->_lock = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":309
 *     cdef object _lock
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self.max_size = max_size
 *         self.max_item_size = max_item_size
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":316
 *     # ---- Abstract methods ----
 * 
 *     def put(self, item):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_3put(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_3put(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_2put(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_2put(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);

  /* "ddtrace/internal/_encoding.pyx":317
 * 
 *     def put(self, item):
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 *     def encode(self):
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 317, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":316
 *     # ---- Abstract methods ----
 * 
 *     def put(self, item):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":319
 *         raise NotImplementedError()
 * 
 *     def encode(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_5encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_5encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("encode (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_4encode(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_4encode(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode", 0);

  /* "ddtrace/internal/_encoding.pyx":320
 * 
 *     def encode(self):
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 320, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":319
 *         raise NotImplementedError()
 * 
 *     def encode(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":305
 *     content_type: str = None
 * 
 *     cdef public size_t max_size             # <<<<<<<<<<<<<<
 *     cdef public size_t max_item_size
 *     cdef object _lock
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->max_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.max_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_2__set__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_2__set__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)
  __pyx_v_self->max_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.max_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":306
 * 
 *     cdef public size_t max_size
 *     cdef public size_t max_item_size             # <<<<<<<<<<<<<<
 *     cdef object _lock
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->max_item_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.max_item_size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_2__set__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_2__set__(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_size_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 306, __pyx_L1_error)
  __pyx_v_self->max_item_size = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.max_item_size.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_6__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_15BufferedEncoder_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.BufferedEncoder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":327
 *     cdef Py_ssize_t _size
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self._buffer = []
 *         self._size = 0
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED size_t __pyx_v_max_size;
  CYTHON_UNUSED size_t __pyx_v_max_item_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_size,&__pyx_n_s_max_item_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_item_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(0, 327, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 327, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_max_size = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_max_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 327, __pyx_L3_error)
    __pyx_v_max_item_size = __Pyx_PyInt_As_size_t(values[1]); if (unlikely((__pyx_v_max_item_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 327, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 327, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder___cinit__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self), __pyx_v_max_size, __pyx_v_max_item_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED size_t __pyx_v_max_size, CYTHON_UNUSED size_t __pyx_v_max_item_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ddtrace/internal/_encoding.pyx":328
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         self._buffer = []             # <<<<<<<<<<<<<<
 *         self._size = 0
 * 
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 328, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_buffer);
  __Pyx_DECREF(__pyx_v_self->_buffer);
  __pyx_v_self->_buffer = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":329
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         self._buffer = []
 *         self._size = 0             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __pyx_v_self->_size = 0;

  /* "ddtrace/internal/_encoding.pyx":327
 *     cdef Py_ssize_t _size
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self._buffer = []
 *         self._size = 0
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":331
 *         self._size = 0
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self._buffer)
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_3__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_3__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_2__len__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_2__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ddtrace/internal/_encoding.pyx":332
 * 
 *     def __len__(self):
 *         return len(self._buffer)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_t_1 = __pyx_v_self->_buffer;
  __Pyx_INCREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 332, __pyx_L1_error)
  }
  __pyx_t_2 = PyList_GET_SIZE(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":331
 *         self._size = 0
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return len(self._buffer)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":335
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             return self._size
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ddtrace/internal/_encoding.pyx":336
 *     @property
 *     def size(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self._size
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 336, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 336, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 336, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":337
 *     def size(self):
 *         with self._lock:
 *             return self._size             # <<<<<<<<<<<<<<
 * 
 *     cpdef put(self, item):
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = PyInt_FromSsize_t(__pyx_v_self->_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 337, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":336
 *     @property
 *     def size(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self._size
 * 
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 336, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 336, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 336, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 336, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 336, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 336, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 336, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "ddtrace/internal/_encoding.pyx":335
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             return self._size
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":339
 *             return self._size
 * 
 *     cpdef put(self, item):             # <<<<<<<<<<<<<<
 *         """Put an item to be serialized in the buffer."""
 *         cdef int item_len
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, PyObject *__pyx_v_item, int __pyx_skip_dispatch) {
  int __pyx_v_item_len;
  PyObject *__pyx_v_encoded_item = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_put); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_5put)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_item) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_item);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 339, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":343
 *         cdef int item_len
 * 
 *         encoded_item = self.encode_item(item)             # <<<<<<<<<<<<<<
 *         item_len = len(encoded_item)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_encode_item); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_item) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_item);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_encoded_item = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":344
 * 
 *         encoded_item = self.encode_item(item)
 *         item_len = len(encoded_item)             # <<<<<<<<<<<<<<
 * 
 *         if item_len > self.max_item_size or item_len > self.max_size:
 */
  __pyx_t_5 = PyObject_Length(__pyx_v_encoded_item); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 344, __pyx_L1_error)
  __pyx_v_item_len = __pyx_t_5;

  /* "ddtrace/internal/_encoding.pyx":346
 *         item_len = len(encoded_item)
 * 
 *         if item_len > self.max_item_size or item_len > self.max_size:             # <<<<<<<<<<<<<<
 *             raise BufferItemTooLarge(item_len)
 * 
 */
  __pyx_t_7 = ((__pyx_v_item_len > __pyx_v_self->__pyx_base.max_item_size) != 0);
  if (!__pyx_t_7) {
  } else {
    __pyx_t_6 = __pyx_t_7;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_7 = ((__pyx_v_item_len > __pyx_v_self->__pyx_base.max_size) != 0);
  __pyx_t_6 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_6)) {

    /* "ddtrace/internal/_encoding.pyx":347
 * 
 *         if item_len > self.max_item_size or item_len > self.max_size:
 *             raise BufferItemTooLarge(item_len)             # <<<<<<<<<<<<<<
 * 
 *         with self._lock:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_BufferItemTooLarge); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_item_len); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 347, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 347, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":346
 *         item_len = len(encoded_item)
 * 
 *         if item_len > self.max_item_size or item_len > self.max_size:             # <<<<<<<<<<<<<<
 *             raise BufferItemTooLarge(item_len)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":349
 *             raise BufferItemTooLarge(item_len)
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._size + item_len <= self.max_size:
 *                 self._buffer.append(encoded_item)
 */
  /*with:*/ {
    __pyx_t_8 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_11);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":350
 * 
 *         with self._lock:
 *             if self._size + item_len <= self.max_size:             # <<<<<<<<<<<<<<
 *                 self._buffer.append(encoded_item)
 *                 self._size += item_len
 */
          __pyx_t_6 = (((__pyx_v_self->_size + __pyx_v_item_len) <= __pyx_v_self->__pyx_base.max_size) != 0);
          if (likely(__pyx_t_6)) {

            /* "ddtrace/internal/_encoding.pyx":351
 *         with self._lock:
 *             if self._size + item_len <= self.max_size:
 *                 self._buffer.append(encoded_item)             # <<<<<<<<<<<<<<
 *                 self._size += item_len
 *             else:
 */
            if (unlikely(__pyx_v_self->_buffer == Py_None)) {
              PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
              __PYX_ERR(0, 351, __pyx_L10_error)
            }
            __pyx_t_12 = __Pyx_PyList_Append(__pyx_v_self->_buffer, __pyx_v_encoded_item); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 351, __pyx_L10_error)

            /* "ddtrace/internal/_encoding.pyx":352
 *             if self._size + item_len <= self.max_size:
 *                 self._buffer.append(encoded_item)
 *                 self._size += item_len             # <<<<<<<<<<<<<<
 *             else:
 *                 raise BufferFull(item_len)
 */
            __pyx_v_self->_size = (__pyx_v_self->_size + __pyx_v_item_len);

            /* "ddtrace/internal/_encoding.pyx":350
 * 
 *         with self._lock:
 *             if self._size + item_len <= self.max_size:             # <<<<<<<<<<<<<<
 *                 self._buffer.append(encoded_item)
 *                 self._size += item_len
 */
            goto __pyx_L16;
          }

          /* "ddtrace/internal/_encoding.pyx":354
 *                 self._size += item_len
 *             else:
 *                 raise BufferFull(item_len)             # <<<<<<<<<<<<<<
 * 
 *     cpdef get(self):
 */
          /*else*/ {
            __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_BufferFull); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_2);
            __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_item_len); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 354, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_3);
            __pyx_t_4 = NULL;
            if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
              if (likely(__pyx_t_4)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
                __Pyx_INCREF(__pyx_t_4);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_2, function);
              }
            }
            __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3);
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 354, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_Raise(__pyx_t_1, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __PYX_ERR(0, 354, __pyx_L10_error)
          }
          __pyx_L16:;

          /* "ddtrace/internal/_encoding.pyx":349
 *             raise BufferItemTooLarge(item_len)
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if self._size + item_len <= self.max_size:
 *                 self._buffer.append(encoded_item)
 */
        }
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L15_try_end;
        __pyx_L10_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 349, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 349, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 349, __pyx_L12_except_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_13);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (__pyx_t_6 < 0) __PYX_ERR(0, 349, __pyx_L12_except_error)
          __pyx_t_7 = ((!(__pyx_t_6 != 0)) != 0);
          if (__pyx_t_7) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 349, __pyx_L12_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L11_exception_handled;
        }
        __pyx_L12_except_error:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        goto __pyx_L1_error;
        __pyx_L11_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_XGIVEREF(__pyx_t_11);
        __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
        __pyx_L15_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_8) {
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 349, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        }
        goto __pyx_L9;
      }
      __pyx_L9:;
    }
    goto __pyx_L20;
    __pyx_L6_error:;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L1_error;
    __pyx_L20:;
  }

  /* "ddtrace/internal/_encoding.pyx":339
 *             return self._size
 * 
 *     cpdef put(self, item):             # <<<<<<<<<<<<<<
 *         """Put an item to be serialized in the buffer."""
 *         cdef int item_len
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoded_item);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static char __pyx_doc_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4put[] = "Put an item to be serialized in the buffer.";
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4put(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4put(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_put(__pyx_v_self, __pyx_v_item, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 339, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":356
 *                 raise BufferFull(item_len)
 * 
 *     cpdef get(self):             # <<<<<<<<<<<<<<
 *         """Get a copy of the buffer and clear it."""
 *         with self._lock:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_7get(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_get(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  int __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_7get)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":358
 *     cpdef get(self):
 *         """Get a copy of the buffer and clear it."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return list(self._buffer)
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":359
 *         """Get a copy of the buffer and clear it."""
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 return list(self._buffer)
 *             finally:
 */
          /*try:*/ {

            /* "ddtrace/internal/_encoding.pyx":360
 *         with self._lock:
 *             try:
 *                 return list(self._buffer)             # <<<<<<<<<<<<<<
 *             finally:
 *                 self._buffer[:] = []
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_1 = PySequence_List(__pyx_v_self->_buffer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_r = __pyx_t_1;
            __pyx_t_1 = 0;
            goto __pyx_L13_return;
          }

          /* "ddtrace/internal/_encoding.pyx":362
 *                 return list(self._buffer)
 *             finally:
 *                 self._buffer[:] = []             # <<<<<<<<<<<<<<
 *                 self._size = 0
 * 
 */
          /*finally:*/ {
            __pyx_L14_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
              if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_15);
              __Pyx_XGOTREF(__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_17);
              __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
              {
                __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L17_error)
                __Pyx_GOTREF(__pyx_t_1);
                if (unlikely(__pyx_v_self->_buffer == Py_None)) {
                  PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                  __PYX_ERR(0, 362, __pyx_L17_error)
                }
                if (__Pyx_PyObject_SetSlice(__pyx_v_self->_buffer, __pyx_t_1, 0, 0, NULL, NULL, NULL, 0, 0, 1) < 0) __PYX_ERR(0, 362, __pyx_L17_error)
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

                /* "ddtrace/internal/_encoding.pyx":363
 *             finally:
 *                 self._buffer[:] = []
 *                 self._size = 0             # <<<<<<<<<<<<<<
 * 
 *     def encode_item(self, item):
 */
                __pyx_v_self->_size = 0;
              }
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
              goto __pyx_L7_error;
              __pyx_L17_error:;
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              goto __pyx_L7_error;
            }
            __pyx_L13_return: {
              __pyx_t_17 = __pyx_r;
              __pyx_r = 0;

              /* "ddtrace/internal/_encoding.pyx":362
 *                 return list(self._buffer)
 *             finally:
 *                 self._buffer[:] = []             # <<<<<<<<<<<<<<
 *                 self._size = 0
 * 
 */
              __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
              if (unlikely(__pyx_v_self->_buffer == Py_None)) {
                PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
                __PYX_ERR(0, 362, __pyx_L7_error)
              }
              if (__Pyx_PyObject_SetSlice(__pyx_v_self->_buffer, __pyx_t_1, 0, 0, NULL, NULL, NULL, 0, 0, 1) < 0) __PYX_ERR(0, 362, __pyx_L7_error)
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "ddtrace/internal/_encoding.pyx":363
 *             finally:
 *                 self._buffer[:] = []
 *                 self._size = 0             # <<<<<<<<<<<<<<
 * 
 *     def encode_item(self, item):
 */
              __pyx_v_self->_size = 0;
              __pyx_r = __pyx_t_17;
              __pyx_t_17 = 0;
              goto __pyx_L11_try_return;
            }
          }

          /* "ddtrace/internal/_encoding.pyx":358
 *     cpdef get(self):
 *         """Get a copy of the buffer and clear it."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return list(self._buffer)
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 358, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 358, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 358, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (__pyx_t_18 < 0) __PYX_ERR(0, 358, __pyx_L9_except_error)
          __pyx_t_19 = ((!(__pyx_t_18 != 0)) != 0);
          if (__pyx_t_19) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 358, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 358, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 358, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "ddtrace/internal/_encoding.pyx":356
 *                 raise BufferFull(item_len)
 * 
 *     cpdef get(self):             # <<<<<<<<<<<<<<
 *         """Get a copy of the buffer and clear it."""
 *         with self._lock:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_7get(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_6get[] = "Get a copy of the buffer and clear it.";
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_7get(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_6get(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_6get(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_get(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":365
 *                 self._size = 0
 * 
 *     def encode_item(self, item):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_9encode_item(PyObject *__pyx_v_self, PyObject *__pyx_v_item); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_9encode_item(PyObject *__pyx_v_self, PyObject *__pyx_v_item) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("encode_item (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_8encode_item(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v_item));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_8encode_item(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_item) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode_item", 0);

  /* "ddtrace/internal/_encoding.pyx":366
 * 
 *     def encode_item(self, item):
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 366, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 366, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":365
 *                 self._size = 0
 * 
 *     def encode_item(self, item):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.encode_item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_10__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_12__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.ListBufferedEncoder.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":375
 *     cdef stdint.uint32_t _count
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_max_size;
  size_t __pyx_v_max_item_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_size,&__pyx_n_s_max_item_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_item_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(0, 375, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 375, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_max_size = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_max_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L3_error)
    __pyx_v_max_item_size = __Pyx_PyInt_As_size_t(values[1]); if (unlikely((__pyx_v_max_item_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 375, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 375, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase___cinit__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self), __pyx_v_max_size, __pyx_v_max_item_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, size_t __pyx_v_max_size, size_t __pyx_v_max_item_size) {
  int __pyx_v_buf_size;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ddtrace/internal/_encoding.pyx":376
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         cdef int buf_size = 1024*1024             # <<<<<<<<<<<<<<
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:
 */
  __pyx_v_buf_size = 0x100000;

  /* "ddtrace/internal/_encoding.pyx":377
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)             # <<<<<<<<<<<<<<
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")
 */
  __pyx_v_self->pk.buf = ((char *)PyMem_Malloc(__pyx_v_buf_size));

  /* "ddtrace/internal/_encoding.pyx":378
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 * 
 */
  __pyx_t_1 = ((__pyx_v_self->pk.buf == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ddtrace/internal/_encoding.pyx":379
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")             # <<<<<<<<<<<<<<
 * 
 *         self.max_size = max_size
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 379, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":378
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":381
 *             raise MemoryError("Unable to allocate internal buffer.")
 * 
 *         self.max_size = max_size             # <<<<<<<<<<<<<<
 *         self.pk.buf_size = buf_size
 *         self.max_item_size = max_item_size if max_item_size < max_size else max_size
 */
  __pyx_v_self->__pyx_base.max_size = __pyx_v_max_size;

  /* "ddtrace/internal/_encoding.pyx":382
 * 
 *         self.max_size = max_size
 *         self.pk.buf_size = buf_size             # <<<<<<<<<<<<<<
 *         self.max_item_size = max_item_size if max_item_size < max_size else max_size
 *         self._lock = threading.RLock()
 */
  __pyx_v_self->pk.buf_size = __pyx_v_buf_size;

  /* "ddtrace/internal/_encoding.pyx":383
 *         self.max_size = max_size
 *         self.pk.buf_size = buf_size
 *         self.max_item_size = max_item_size if max_item_size < max_size else max_size             # <<<<<<<<<<<<<<
 *         self._lock = threading.RLock()
 *         self._reset_buffer()
 */
  if (((__pyx_v_max_item_size < __pyx_v_max_size) != 0)) {
    __pyx_t_3 = __pyx_v_max_item_size;
  } else {
    __pyx_t_3 = __pyx_v_max_size;
  }
  __pyx_v_self->__pyx_base.max_item_size = __pyx_t_3;

  /* "ddtrace/internal/_encoding.pyx":384
 *         self.pk.buf_size = buf_size
 *         self.max_item_size = max_item_size if max_item_size < max_size else max_size
 *         self._lock = threading.RLock()             # <<<<<<<<<<<<<<
 *         self._reset_buffer()
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_threading); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_RLock); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 384, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._lock);
  __Pyx_DECREF(__pyx_v_self->__pyx_base._lock);
  __pyx_v_self->__pyx_base._lock = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":385
 *         self.max_item_size = max_item_size if max_item_size < max_size else max_size
 *         self._lock = threading.RLock()
 *         self._reset_buffer()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self->__pyx_vtab)->_reset_buffer(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 385, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":375
 *     cdef stdint.uint32_t _count
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":387
 *         self._reset_buffer()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

/* Python wrapper */
static void __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_3__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_2__dealloc__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_2__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ddtrace/internal/_encoding.pyx":388
 * 
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)             # <<<<<<<<<<<<<<
 *         self.pk.buf = NULL
 * 
 */
  PyMem_Free(__pyx_v_self->pk.buf);

  /* "ddtrace/internal/_encoding.pyx":389
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):  # TODO: Use a better name?
 */
  __pyx_v_self->pk.buf = NULL;

  /* "ddtrace/internal/_encoding.pyx":387
 *         self._reset_buffer()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "ddtrace/internal/_encoding.pyx":391
 *         self.pk.buf = NULL
 * 
 *     def __len__(self):  # TODO: Use a better name?             # <<<<<<<<<<<<<<
 *         return self._count
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_5__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_5__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4__len__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4__len__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "ddtrace/internal/_encoding.pyx":392
 * 
 *     def __len__(self):  # TODO: Use a better name?
 *         return self._count             # <<<<<<<<<<<<<<
 * 
 *     cpdef _decode(self, data):
 */
  __pyx_r = __pyx_v_self->_count;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":391
 *         self.pk.buf = NULL
 * 
 *     def __len__(self):  # TODO: Use a better name?             # <<<<<<<<<<<<<<
 *         return self._count
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":394
 *         return self._count
 * 
 *     cpdef _decode(self, data):             # <<<<<<<<<<<<<<
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_7_decode(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__decode(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_msgpack = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_decode", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_decode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_7_decode)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_data);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 394, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":395
 * 
 *     cpdef _decode(self, data):
 *         import msgpack             # <<<<<<<<<<<<<<
 *         if msgpack.version[:2] < (0, 6):
 *             return msgpack.unpackb(data)
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_msgpack, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 395, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_msgpack = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":396
 *     cpdef _decode(self, data):
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):             # <<<<<<<<<<<<<<
 *             return msgpack.unpackb(data)
 *         return msgpack.unpackb(data, raw=True)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_msgpack, __pyx_n_s_version); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetSlice(__pyx_t_1, 0, 2, NULL, NULL, &__pyx_slice__14, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_tuple__15, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "ddtrace/internal/_encoding.pyx":397
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):
 *             return msgpack.unpackb(data)             # <<<<<<<<<<<<<<
 *         return msgpack.unpackb(data, raw=True)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_msgpack, __pyx_n_s_unpackb); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_data) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_data);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":396
 *     cpdef _decode(self, data):
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):             # <<<<<<<<<<<<<<
 *             return msgpack.unpackb(data)
 *         return msgpack.unpackb(data, raw=True)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":398
 *         if msgpack.version[:2] < (0, 6):
 *             return msgpack.unpackb(data)
 *         return msgpack.unpackb(data, raw=True)             # <<<<<<<<<<<<<<
 * 
 *     cdef _reset_buffer(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_msgpack, __pyx_n_s_unpackb); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_data);
  __Pyx_GIVEREF(__pyx_v_data);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_data);
  __pyx_t_3 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_t_3, __pyx_n_s_raw, Py_True) < 0) __PYX_ERR(0, 398, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":394
 *         return self._count
 * 
 *     cpdef _decode(self, data):             # <<<<<<<<<<<<<<
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase._decode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_msgpack);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_7_decode(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_7_decode(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_decode (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_6_decode(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_6_decode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_decode", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__decode(__pyx_v_self, __pyx_v_data, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase._decode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":400
 *         return msgpack.unpackb(data, raw=True)
 * 
 *     cdef _reset_buffer(self):             # <<<<<<<<<<<<<<
 *         self._count = 0
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__reset_buffer(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_reset_buffer", 0);

  /* "ddtrace/internal/_encoding.pyx":401
 * 
 *     cdef _reset_buffer(self):
 *         self._count = 0             # <<<<<<<<<<<<<<
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix
 * 
 */
  __pyx_v_self->_count = 0;

  /* "ddtrace/internal/_encoding.pyx":402
 *     cdef _reset_buffer(self):
 *         self._count = 0
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix             # <<<<<<<<<<<<<<
 * 
 *     cpdef encode(self):
 */
  __pyx_v_self->pk.length = 5;

  /* "ddtrace/internal/_encoding.pyx":400
 *         return msgpack.unpackb(data, raw=True)
 * 
 *     cdef _reset_buffer(self):             # <<<<<<<<<<<<<<
 *         self._count = 0
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":404
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix
 * 
 *     cpdef encode(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             if not self._count:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_9encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_encode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_encode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_9encode)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":405
 * 
 *     cpdef encode(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if not self._count:
 *                 return None
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 405, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 405, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":406
 *     cpdef encode(self):
 *         with self._lock:
 *             if not self._count:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
          __pyx_t_9 = ((!(__pyx_v_self->_count != 0)) != 0);
          if (__pyx_t_9) {

            /* "ddtrace/internal/_encoding.pyx":407
 *         with self._lock:
 *             if not self._count:
 *                 return None             # <<<<<<<<<<<<<<
 * 
 *             return self.flush()
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_r = Py_None; __Pyx_INCREF(Py_None);
            goto __pyx_L11_try_return;

            /* "ddtrace/internal/_encoding.pyx":406
 *     cpdef encode(self):
 *         with self._lock:
 *             if not self._count:             # <<<<<<<<<<<<<<
 *                 return None
 * 
 */
          }

          /* "ddtrace/internal/_encoding.pyx":409
 *                 return None
 * 
 *             return self.flush()             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _update_array_len(self):
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self->__pyx_vtab)->flush(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 409, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_r = __pyx_t_1;
          __pyx_t_1 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":405
 * 
 *     cpdef encode(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             if not self._count:
 *                 return None
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 405, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 405, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 405, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (__pyx_t_9 < 0) __PYX_ERR(0, 405, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_9 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 405, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 405, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L17;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L17:;
  }

  /* "ddtrace/internal/_encoding.pyx":404
 *         self.pk.length = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE  # Leave room for array length prefix
 * 
 *     cpdef encode(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             if not self._count:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_9encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_9encode(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("encode (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_8encode(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_8encode(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("encode", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_encode(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.encode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":411
 *             return self.flush()
 * 
 *     cdef inline int _update_array_len(self):             # <<<<<<<<<<<<<<
 *         """Update traces array size prefix"""
 *         cdef int offset = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE - array_prefix_size(self._count)
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  int __pyx_v_offset;
  int __pyx_v_old_pos;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_update_array_len", 0);

  /* "ddtrace/internal/_encoding.pyx":413
 *     cdef inline int _update_array_len(self):
 *         """Update traces array size prefix"""
 *         cdef int offset = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE - array_prefix_size(self._count)             # <<<<<<<<<<<<<<
 *         cdef int old_pos = self.pk.length
 * 
 */
  __pyx_v_offset = (5 - __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(__pyx_v_self->_count));

  /* "ddtrace/internal/_encoding.pyx":414
 *         """Update traces array size prefix"""
 *         cdef int offset = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE - array_prefix_size(self._count)
 *         cdef int old_pos = self.pk.length             # <<<<<<<<<<<<<<
 * 
 *         with self._lock:
 */
  __pyx_t_1 = __pyx_v_self->pk.length;
  __pyx_v_old_pos = __pyx_t_1;

  /* "ddtrace/internal/_encoding.pyx":416
 *         cdef int old_pos = self.pk.length
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             self.pk.length = offset
 *             msgpack_pack_array(&self.pk, self._count)
 */
  /*with:*/ {
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 416, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 416, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 416, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    /*try:*/ {
      {
        (void)__pyx_t_6; (void)__pyx_t_7; (void)__pyx_t_8; /* mark used */
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":417
 * 
 *         with self._lock:
 *             self.pk.length = offset             # <<<<<<<<<<<<<<
 *             msgpack_pack_array(&self.pk, self._count)
 *             self.pk.length = old_pos
 */
          __pyx_v_self->pk.length = __pyx_v_offset;

          /* "ddtrace/internal/_encoding.pyx":418
 *         with self._lock:
 *             self.pk.length = offset
 *             msgpack_pack_array(&self.pk, self._count)             # <<<<<<<<<<<<<<
 *             self.pk.length = old_pos
 *             return offset
 */
          (void)(msgpack_pack_array((&__pyx_v_self->pk), __pyx_v_self->_count));

          /* "ddtrace/internal/_encoding.pyx":419
 *             self.pk.length = offset
 *             msgpack_pack_array(&self.pk, self._count)
 *             self.pk.length = old_pos             # <<<<<<<<<<<<<<
 *             return offset
 * 
 */
          __pyx_v_self->pk.length = __pyx_v_old_pos;

          /* "ddtrace/internal/_encoding.pyx":420
 *             msgpack_pack_array(&self.pk, self._count)
 *             self.pk.length = old_pos
 *             return offset             # <<<<<<<<<<<<<<
 * 
 *     cdef get_bytes(self):
 */
          __pyx_r = __pyx_v_offset;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":416
 *         cdef int old_pos = self.pk.length
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             self.pk.length = offset
 *             msgpack_pack_array(&self.pk, self._count)
 */
        }
        __pyx_L11_try_return:;
        goto __pyx_L4_return;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 416, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_9 = __pyx_r;
        if (__pyx_t_2) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 416, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __pyx_r = __pyx_t_9;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L13;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L1_error;
    __pyx_L13:;
  }

  /* "ddtrace/internal/_encoding.pyx":411
 *             return self.flush()
 * 
 *     cdef inline int _update_array_len(self):             # <<<<<<<<<<<<<<
 *         """Update traces array size prefix"""
 *         cdef int offset = MSGPACK_ARRAY_LENGTH_PREFIX_SIZE - array_prefix_size(self._count)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("ddtrace.internal._encoding.MsgpackEncoderBase._update_array_len", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":422
 *             return offset
 * 
 *     cdef get_bytes(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer contents as bytes object"""
 *         cdef int offset = self._update_array_len()
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  int __pyx_v_offset;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_bytes", 0);

  /* "ddtrace/internal/_encoding.pyx":424
 *     cdef get_bytes(self):
 *         """Return internal buffer contents as bytes object"""
 *         cdef int offset = self._update_array_len()             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 */
  __pyx_v_offset = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len(__pyx_v_self);

  /* "ddtrace/internal/_encoding.pyx":425
 *         """Return internal buffer contents as bytes object"""
 *         cdef int offset = self._update_array_len()
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 425, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 425, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 425, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":426
 *         cdef int offset = self._update_array_len()
 *         with self._lock:
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)             # <<<<<<<<<<<<<<
 * 
 *     cdef char * get_buffer(self):
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = PyBytes_FromStringAndSize((__pyx_v_self->pk.buf + __pyx_v_offset), (__pyx_v_self->pk.length - __pyx_v_offset)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":425
 *         """Return internal buffer contents as bytes object"""
 *         cdef int offset = self._update_array_len()
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 * 
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.get_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 425, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 425, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 425, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 425, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 425, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 425, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 425, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "ddtrace/internal/_encoding.pyx":422
 *             return offset
 * 
 *     cdef get_bytes(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer contents as bytes object"""
 *         cdef int offset = self._update_array_len()
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.get_bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":428
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 * 
 *     cdef char * get_buffer(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer."""
 *         return self.pk.buf + self._update_array_len()
 */

static char *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_buffer(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  char *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_buffer", 0);

  /* "ddtrace/internal/_encoding.pyx":430
 *     cdef char * get_buffer(self):
 *         """Return internal buffer."""
 *         return self.pk.buf + self._update_array_len()             # <<<<<<<<<<<<<<
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 */
  __pyx_r = (__pyx_v_self->pk.buf + __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len(__pyx_v_self));
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":428
 *             return PyBytes_FromStringAndSize(self.pk.buf + offset, self.pk.length - offset)
 * 
 *     cdef char * get_buffer(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer."""
 *         return self.pk.buf + self._update_array_len()
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":432
 *         return self.pk.buf + self._update_array_len()
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

static void *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_dd_origin_ref(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_dd_origin) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_dd_origin_ref", 0);

  /* "ddtrace/internal/_encoding.pyx":433
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _pack_trace(self, list trace) except? -1:
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 433, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":432
 *         return self.pk.buf + self._update_array_len()
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("ddtrace.internal._encoding.MsgpackEncoderBase.get_dd_origin_ref", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":435
 *         raise NotImplementedError()
 * 
 *     cdef inline int _pack_trace(self, list trace) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         cdef Py_ssize_t L
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__pack_trace(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace) {
  int __pyx_v_ret;
  Py_ssize_t __pyx_v_L;
  void *__pyx_v_dd_origin;
  PyObject *__pyx_v_span = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_pack_trace", 0);

  /* "ddtrace/internal/_encoding.pyx":438
 *         cdef int ret
 *         cdef Py_ssize_t L
 *         cdef void * dd_origin = NULL             # <<<<<<<<<<<<<<
 * 
 *         L = len(trace)
 */
  __pyx_v_dd_origin = NULL;

  /* "ddtrace/internal/_encoding.pyx":440
 *         cdef void * dd_origin = NULL
 * 
 *         L = len(trace)             # <<<<<<<<<<<<<<
 *         if L > ITEM_LIMIT:
 *             raise ValueError("list is too large")
 */
  if (unlikely(__pyx_v_trace == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 440, __pyx_L1_error)
  }
  __pyx_t_1 = PyList_GET_SIZE(__pyx_v_trace); if (unlikely(__pyx_t_1 == ((Py_ssize_t)-1))) __PYX_ERR(0, 440, __pyx_L1_error)
  __pyx_v_L = __pyx_t_1;

  /* "ddtrace/internal/_encoding.pyx":441
 * 
 *         L = len(trace)
 *         if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *             raise ValueError("list is too large")
 * 
 */
  __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
  if (unlikely(__pyx_t_2)) {

    /* "ddtrace/internal/_encoding.pyx":442
 *         L = len(trace)
 *         if L > ITEM_LIMIT:
 *             raise ValueError("list is too large")             # <<<<<<<<<<<<<<
 * 
 *         ret = msgpack_pack_array(&self.pk, L)
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 442, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":441
 * 
 *         L = len(trace)
 *         if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *             raise ValueError("list is too large")
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":444
 *             raise ValueError("list is too large")
 * 
 *         ret = msgpack_pack_array(&self.pk, L)             # <<<<<<<<<<<<<<
 *         if ret != 0: raise RuntimeError("Couldn't pack trace")
 * 
 */
  __pyx_v_ret = msgpack_pack_array((&__pyx_v_self->pk), __pyx_v_L);

  /* "ddtrace/internal/_encoding.pyx":445
 * 
 *         ret = msgpack_pack_array(&self.pk, L)
 *         if ret != 0: raise RuntimeError("Couldn't pack trace")             # <<<<<<<<<<<<<<
 * 
 *         if L > 0 and trace[0].context is not None and trace[0].context.dd_origin is not None:
 */
  __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
  if (unlikely(__pyx_t_2)) {
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 445, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 445, __pyx_L1_error)
  }

  /* "ddtrace/internal/_encoding.pyx":447
 *         if ret != 0: raise RuntimeError("Couldn't pack trace")
 * 
 *         if L > 0 and trace[0].context is not None and trace[0].context.dd_origin is not None:             # <<<<<<<<<<<<<<
 *             dd_origin = self.get_dd_origin_ref(trace[0].context.dd_origin)
 * 
 */
  __pyx_t_4 = ((__pyx_v_L > 0) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L6_bool_binop_done;
  }
  if (unlikely(__pyx_v_trace == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 447, __pyx_L1_error)
  }
  __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_trace, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_context); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4 = (__pyx_t_5 != Py_None);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6 = (__pyx_t_4 != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_2 = __pyx_t_6;
    goto __pyx_L6_bool_binop_done;
  }
  if (unlikely(__pyx_v_trace == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 447, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_trace, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_context); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dd_origin); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (__pyx_t_5 != Py_None);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_4 = (__pyx_t_6 != 0);
  __pyx_t_2 = __pyx_t_4;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":448
 * 
 *         if L > 0 and trace[0].context is not None and trace[0].context.dd_origin is not None:
 *             dd_origin = self.get_dd_origin_ref(trace[0].context.dd_origin)             # <<<<<<<<<<<<<<
 * 
 *         for span in trace:
 */
    if (unlikely(__pyx_v_trace == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 448, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_GetItemInt_List(__pyx_v_trace, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_context); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_dd_origin); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 448, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_5))||((__pyx_t_5) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_5)->tp_name), 0))) __PYX_ERR(0, 448, __pyx_L1_error)
    __pyx_v_dd_origin = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self->__pyx_vtab)->get_dd_origin_ref(__pyx_v_self, ((PyObject*)__pyx_t_5));
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "ddtrace/internal/_encoding.pyx":447
 *         if ret != 0: raise RuntimeError("Couldn't pack trace")
 * 
 *         if L > 0 and trace[0].context is not None and trace[0].context.dd_origin is not None:             # <<<<<<<<<<<<<<
 *             dd_origin = self.get_dd_origin_ref(trace[0].context.dd_origin)
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":450
 *             dd_origin = self.get_dd_origin_ref(trace[0].context.dd_origin)
 * 
 *         for span in trace:             # <<<<<<<<<<<<<<
 *             ret = self.pack_span(span, dd_origin)
 *             if ret != 0: raise RuntimeError("Couldn't pack span")
 */
  if (unlikely(__pyx_v_trace == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 450, __pyx_L1_error)
  }
  __pyx_t_5 = __pyx_v_trace; __Pyx_INCREF(__pyx_t_5); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_1 >= PyList_GET_SIZE(__pyx_t_5)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_1); __Pyx_INCREF(__pyx_t_3); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 450, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_5, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_span, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "ddtrace/internal/_encoding.pyx":451
 * 
 *         for span in trace:
 *             ret = self.pack_span(span, dd_origin)             # <<<<<<<<<<<<<<
 *             if ret != 0: raise RuntimeError("Couldn't pack span")
 * 
 */
    __pyx_t_7 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self->__pyx_vtab)->pack_span(__pyx_v_self, __pyx_v_span, __pyx_v_dd_origin); if (unlikely(__pyx_t_7 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 451, __pyx_L1_error)
    __pyx_v_ret = __pyx_t_7;

    /* "ddtrace/internal/_encoding.pyx":452
 *         for span in trace:
 *             ret = self.pack_span(span, dd_origin)
 *             if ret != 0: raise RuntimeError("Couldn't pack span")             # <<<<<<<<<<<<<<
 * 
 *         return ret
 */
    __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
    if (unlikely(__pyx_t_2)) {
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 452, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 452, __pyx_L1_error)
    }

    /* "ddtrace/internal/_encoding.pyx":450
 *             dd_origin = self.get_dd_origin_ref(trace[0].context.dd_origin)
 * 
 *         for span in trace:             # <<<<<<<<<<<<<<
 *             ret = self.pack_span(span, dd_origin)
 *             if ret != 0: raise RuntimeError("Couldn't pack span")
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "ddtrace/internal/_encoding.pyx":454
 *             if ret != 0: raise RuntimeError("Couldn't pack span")
 * 
 *         return ret             # <<<<<<<<<<<<<<
 * 
 *     cpdef put(self, list trace):
 */
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":435
 *         raise NotImplementedError()
 * 
 *     cdef inline int _pack_trace(self, list trace) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         cdef Py_ssize_t L
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase._pack_trace", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_span);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":456
 *         return ret
 * 
 *     cpdef put(self, list trace):             # <<<<<<<<<<<<<<
 *         """Put a trace (i.e. a list of spans) in the buffer."""
 *         cdef int ret
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_11put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace, int __pyx_skip_dispatch) {
  int __pyx_v_ret;
  size_t __pyx_v_len_before;
  PyObject *__pyx_v_size_before = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_put); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_11put)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_trace) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_trace);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 456, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":460
 *         cdef int ret
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             len_before = self.pk.length
 *             size_before = self.size
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 460, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 460, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":461
 * 
 *         with self._lock:
 *             len_before = self.pk.length             # <<<<<<<<<<<<<<
 *             size_before = self.size
 *             try:
 */
          __pyx_t_9 = __pyx_v_self->pk.length;
          __pyx_v_len_before = __pyx_t_9;

          /* "ddtrace/internal/_encoding.pyx":462
 *         with self._lock:
 *             len_before = self.pk.length
 *             size_before = self.size             # <<<<<<<<<<<<<<
 *             try:
 *                 ret = self._pack_trace(trace)
 */
          __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_v_size_before = __pyx_t_1;
          __pyx_t_1 = 0;

          /* "ddtrace/internal/_encoding.pyx":463
 *             len_before = self.pk.length
 *             size_before = self.size
 *             try:             # <<<<<<<<<<<<<<
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.
 */
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_10, &__pyx_t_11, &__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_12);
            /*try:*/ {

              /* "ddtrace/internal/_encoding.pyx":464
 *             size_before = self.size
 *             try:
 *                 ret = self._pack_trace(trace)             # <<<<<<<<<<<<<<
 *                 if ret:  # should not happen.
 *                     raise RuntimeError("internal error")
 */
              __pyx_t_13 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__pack_trace(__pyx_v_self, __pyx_v_trace); if (unlikely(__pyx_t_13 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 464, __pyx_L13_error)
              __pyx_v_ret = __pyx_t_13;

              /* "ddtrace/internal/_encoding.pyx":465
 *             try:
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("internal error")
 * 
 */
              __pyx_t_14 = (__pyx_v_ret != 0);
              if (unlikely(__pyx_t_14)) {

                /* "ddtrace/internal/_encoding.pyx":466
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.
 *                     raise RuntimeError("internal error")             # <<<<<<<<<<<<<<
 * 
 *                 # DEV: msgpack avoids buffer overflows by calling PyMem_Realloc so
 */
                __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_Raise(__pyx_t_1, 0, 0, 0);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __PYX_ERR(0, 466, __pyx_L13_error)

                /* "ddtrace/internal/_encoding.pyx":465
 *             try:
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.             # <<<<<<<<<<<<<<
 *                     raise RuntimeError("internal error")
 * 
 */
              }

              /* "ddtrace/internal/_encoding.pyx":473
 *                 # grow arbitrarily because of the PyMem_Realloc and if it does then
 *                 # free and reallocate with the appropriate size.
 *                 if self.size - size_before > self.max_item_size:             # <<<<<<<<<<<<<<
 *                     raise BufferItemTooLarge(self.size - size_before)
 * 
 */
              __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_2 = PyNumber_Subtract(__pyx_t_1, __pyx_v_size_before); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 473, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.max_item_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 473, __pyx_L13_error)
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 473, __pyx_L13_error)
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(__pyx_t_14)) {

                /* "ddtrace/internal/_encoding.pyx":474
 *                 # free and reallocate with the appropriate size.
 *                 if self.size - size_before > self.max_item_size:
 *                     raise BufferItemTooLarge(self.size - size_before)             # <<<<<<<<<<<<<<
 * 
 *                 if self.size > self.max_size:
 */
                __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BufferItemTooLarge); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 474, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_1);
                __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 474, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_2);
                __pyx_t_4 = PyNumber_Subtract(__pyx_t_2, __pyx_v_size_before); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 474, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                __pyx_t_2 = NULL;
                if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
                  __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
                  if (likely(__pyx_t_2)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                    __Pyx_INCREF(__pyx_t_2);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_1, function);
                  }
                }
                __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_2, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4);
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_Raise(__pyx_t_3, 0, 0, 0);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __PYX_ERR(0, 474, __pyx_L13_error)

                /* "ddtrace/internal/_encoding.pyx":473
 *                 # grow arbitrarily because of the PyMem_Realloc and if it does then
 *                 # free and reallocate with the appropriate size.
 *                 if self.size - size_before > self.max_item_size:             # <<<<<<<<<<<<<<
 *                     raise BufferItemTooLarge(self.size - size_before)
 * 
 */
              }

              /* "ddtrace/internal/_encoding.pyx":476
 *                     raise BufferItemTooLarge(self.size - size_before)
 * 
 *                 if self.size > self.max_size:             # <<<<<<<<<<<<<<
 *                     raise BufferFull(self.size - size_before)
 * 
 */
              __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_3);
              __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_self->__pyx_base.max_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
              __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_t_1, Py_GT); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 476, __pyx_L13_error)
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 476, __pyx_L13_error)
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (unlikely(__pyx_t_14)) {

                /* "ddtrace/internal/_encoding.pyx":477
 * 
 *                 if self.size > self.max_size:
 *                     raise BufferFull(self.size - size_before)             # <<<<<<<<<<<<<<
 * 
 *                 self._count += 1
 */
                __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_BufferFull); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_1);
                __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_3);
                __pyx_t_2 = PyNumber_Subtract(__pyx_t_3, __pyx_v_size_before); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 477, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                __pyx_t_3 = NULL;
                if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
                  __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
                  if (likely(__pyx_t_3)) {
                    PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
                    __Pyx_INCREF(__pyx_t_3);
                    __Pyx_INCREF(function);
                    __Pyx_DECREF_SET(__pyx_t_1, function);
                  }
                }
                __pyx_t_4 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2);
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 477, __pyx_L13_error)
                __Pyx_GOTREF(__pyx_t_4);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_Raise(__pyx_t_4, 0, 0, 0);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                __PYX_ERR(0, 477, __pyx_L13_error)

                /* "ddtrace/internal/_encoding.pyx":476
 *                     raise BufferItemTooLarge(self.size - size_before)
 * 
 *                 if self.size > self.max_size:             # <<<<<<<<<<<<<<
 *                     raise BufferFull(self.size - size_before)
 * 
 */
              }

              /* "ddtrace/internal/_encoding.pyx":479
 *                     raise BufferFull(self.size - size_before)
 * 
 *                 self._count += 1             # <<<<<<<<<<<<<<
 *             except:
 *                 # rollback
 */
              __pyx_v_self->_count = (__pyx_v_self->_count + 1);

              /* "ddtrace/internal/_encoding.pyx":463
 *             len_before = self.pk.length
 *             size_before = self.size
 *             try:             # <<<<<<<<<<<<<<
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.
 */
            }
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            goto __pyx_L18_try_end;
            __pyx_L13_error:;
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "ddtrace/internal/_encoding.pyx":480
 * 
 *                 self._count += 1
 *             except:             # <<<<<<<<<<<<<<
 *                 # rollback
 *                 self.pk.length = len_before
 */
            /*except:*/ {
              __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_1, &__pyx_t_2) < 0) __PYX_ERR(0, 480, __pyx_L15_except_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_GOTREF(__pyx_t_2);

              /* "ddtrace/internal/_encoding.pyx":482
 *             except:
 *                 # rollback
 *                 self.pk.length = len_before             # <<<<<<<<<<<<<<
 *                 raise
 * 
 */
              __pyx_v_self->pk.length = __pyx_v_len_before;

              /* "ddtrace/internal/_encoding.pyx":483
 *                 # rollback
 *                 self.pk.length = len_before
 *                 raise             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
              __Pyx_GIVEREF(__pyx_t_4);
              __Pyx_GIVEREF(__pyx_t_1);
              __Pyx_XGIVEREF(__pyx_t_2);
              __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_1, __pyx_t_2);
              __pyx_t_4 = 0; __pyx_t_1 = 0; __pyx_t_2 = 0; 
              __PYX_ERR(0, 483, __pyx_L15_except_error)
            }
            __pyx_L15_except_error:;

            /* "ddtrace/internal/_encoding.pyx":463
 *             len_before = self.pk.length
 *             size_before = self.size
 *             try:             # <<<<<<<<<<<<<<
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.
 */
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_ExceptionReset(__pyx_t_10, __pyx_t_11, __pyx_t_12);
            goto __pyx_L7_error;
            __pyx_L18_try_end:;
          }

          /* "ddtrace/internal/_encoding.pyx":460
 *         cdef int ret
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             len_before = self.pk.length
 *             size_before = self.size
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_4) < 0) __PYX_ERR(0, 460, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_3 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_1, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 460, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_3, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 460, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_12);
          __pyx_t_14 = __Pyx_PyObject_IsTrue(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (__pyx_t_14 < 0) __PYX_ERR(0, 460, __pyx_L9_except_error)
          __pyx_t_15 = ((!(__pyx_t_14 != 0)) != 0);
          if (__pyx_t_15) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_1, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_1 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 460, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 460, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L27;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L27:;
  }

  /* "ddtrace/internal/_encoding.pyx":456
 *         return ret
 * 
 *     cpdef put(self, list trace):             # <<<<<<<<<<<<<<
 *         """Put a trace (i.e. a list of spans) in the buffer."""
 *         cdef int ret
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_size_before);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_11put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace); /*proto*/
static char __pyx_doc_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_10put[] = "Put a trace (i.e. a list of spans) in the buffer.";
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_11put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_trace), (&PyList_Type), 1, "trace", 1))) __PYX_ERR(0, 456, __pyx_L1_error)
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_10put(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self), ((PyObject*)__pyx_v_trace));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_10put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, PyObject *__pyx_v_trace) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_put(__pyx_v_self, __pyx_v_trace, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":486
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ddtrace/internal/_encoding.pyx":488
 *     def size(self):
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self.pk.length + array_prefix_size(self._count) - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 488, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 488, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":489
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 *             return self.pk.length + array_prefix_size(self._count) - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE             # <<<<<<<<<<<<<<
 * 
 *     # ---- Abstract methods ----
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = __Pyx_PyInt_FromSize_t(((__pyx_v_self->pk.length + __pyx_f_7ddtrace_8internal_9_encoding_array_prefix_size(__pyx_v_self->_count)) - 5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 489, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_r = __pyx_t_2;
          __pyx_t_2 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":488
 *     def size(self):
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self.pk.length + array_prefix_size(self._count) - MSGPACK_ARRAY_LENGTH_PREFIX_SIZE
 * 
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_4) < 0) __PYX_ERR(0, 488, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 488, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 488, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 488, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_4);
            __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_4);
            __pyx_t_2 = 0; __pyx_t_3 = 0; __pyx_t_4 = 0; 
            __PYX_ERR(0, 488, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 488, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 488, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "ddtrace/internal/_encoding.pyx":486
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":493
 *     # ---- Abstract methods ----
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_13flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_flush(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 493, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_13flush)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 493, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":494
 * 
 *     cpdef flush(self):
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 494, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":493
 *     # ---- Abstract methods ----
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_13flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_13flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("flush (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_12flush(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_12flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_flush(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":496
 *         raise NotImplementedError()
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

static int __pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_pack_span(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_span, CYTHON_UNUSED void *__pyx_v_dd_origin) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_span", 0);

  /* "ddtrace/internal/_encoding.pyx":497
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:
 *         raise NotImplementedError()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_builtin_NotImplementedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(0, 497, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":496
 *         raise NotImplementedError()
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         raise NotImplementedError()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.pack_span", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_15__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_14__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_17__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_16__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":501
 * 
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_1flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  char const *__pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  int __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 501, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_1flush)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 501, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":502
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return self.get_bytes()
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 502, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":503
 *     cpdef flush(self):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 return self.get_bytes()
 *             finally:
 */
          /*try:*/ {

            /* "ddtrace/internal/_encoding.pyx":504
 *         with self._lock:
 *             try:
 *                 return self.get_bytes()             # <<<<<<<<<<<<<<
 *             finally:
 *                 self._reset_buffer()
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get_bytes(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_r = __pyx_t_1;
            __pyx_t_1 = 0;
            goto __pyx_L13_return;
          }

          /* "ddtrace/internal/_encoding.pyx":506
 *                 return self.get_bytes()
 *             finally:
 *                 self._reset_buffer()             # <<<<<<<<<<<<<<
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 */
          /*finally:*/ {
            __pyx_L14_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_15, &__pyx_t_16, &__pyx_t_17);
              if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14) < 0)) __Pyx_ErrFetch(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_15);
              __Pyx_XGOTREF(__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_17);
              __pyx_t_9 = __pyx_lineno; __pyx_t_10 = __pyx_clineno; __pyx_t_11 = __pyx_filename;
              {
                __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._reset_buffer(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L17_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              }
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ErrRestore(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_t_12 = 0; __pyx_t_13 = 0; __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              __pyx_lineno = __pyx_t_9; __pyx_clineno = __pyx_t_10; __pyx_filename = __pyx_t_11;
              goto __pyx_L7_error;
              __pyx_L17_error:;
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_15);
                __Pyx_XGIVEREF(__pyx_t_16);
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_16, __pyx_t_17);
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0;
              goto __pyx_L7_error;
            }
            __pyx_L13_return: {
              __pyx_t_17 = __pyx_r;
              __pyx_r = 0;
              __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._reset_buffer(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_r = __pyx_t_17;
              __pyx_t_17 = 0;
              goto __pyx_L11_try_return;
            }
          }

          /* "ddtrace/internal/_encoding.pyx":502
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 return self.get_bytes()
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 502, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 502, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_17 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 502, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_17);
          __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
          if (__pyx_t_18 < 0) __PYX_ERR(0, 502, __pyx_L9_except_error)
          __pyx_t_19 = ((!(__pyx_t_18 != 0)) != 0);
          if (__pyx_t_19) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 502, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 502, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 502, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "ddtrace/internal/_encoding.pyx":501
 * 
 * cdef class MsgpackEncoderV03(MsgpackEncoderBase):
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_1flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_1flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("flush (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 501, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":508
 *                 self._reset_buffer()
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         return string_to_buff(dd_origin)
 * 
 */

static void *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_get_dd_origin_ref(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_dd_origin) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_dd_origin_ref", 0);

  /* "ddtrace/internal/_encoding.pyx":509
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 *         return string_to_buff(dd_origin)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _pack_meta(self, object meta, char *dd_origin) except? -1:
 */
  __pyx_r = __pyx_f_7ddtrace_8internal_9_encoding_string_to_buff(__pyx_v_dd_origin);
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":508
 *                 self._reset_buffer()
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         return string_to_buff(dd_origin)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":511
 *         return string_to_buff(dd_origin)
 * 
 *     cdef inline int _pack_meta(self, object meta, char *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t L
 *         cdef int ret
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_meta(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_meta, char *__pyx_v_dd_origin) {
  Py_ssize_t __pyx_v_L;
  int __pyx_v_ret;
  PyObject *__pyx_v_d = 0;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_pack_meta", 0);

  /* "ddtrace/internal/_encoding.pyx":516
 *         cdef dict d
 * 
 *         if PyDict_CheckExact(meta):             # <<<<<<<<<<<<<<
 *             d = <dict> meta
 *             L = len(d)
 */
  __pyx_t_1 = (PyDict_CheckExact(__pyx_v_meta) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":517
 * 
 *         if PyDict_CheckExact(meta):
 *             d = <dict> meta             # <<<<<<<<<<<<<<
 *             L = len(d)
 *             if dd_origin is not NULL:
 */
    __pyx_t_2 = __pyx_v_meta;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_d = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "ddtrace/internal/_encoding.pyx":518
 *         if PyDict_CheckExact(meta):
 *             d = <dict> meta
 *             L = len(d)             # <<<<<<<<<<<<<<
 *             if dd_origin is not NULL:
 *                 L += 1
 */
    if (unlikely(__pyx_v_d == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 518, __pyx_L1_error)
    }
    __pyx_t_3 = PyDict_Size(__pyx_v_d); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 518, __pyx_L1_error)
    __pyx_v_L = __pyx_t_3;

    /* "ddtrace/internal/_encoding.pyx":519
 *             d = <dict> meta
 *             L = len(d)
 *             if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *                 L += 1
 *             if L > ITEM_LIMIT:
 */
    __pyx_t_1 = ((__pyx_v_dd_origin != NULL) != 0);
    if (__pyx_t_1) {

      /* "ddtrace/internal/_encoding.pyx":520
 *             L = len(d)
 *             if dd_origin is not NULL:
 *                 L += 1             # <<<<<<<<<<<<<<
 *             if L > ITEM_LIMIT:
 *                 raise ValueError("dict is too large")
 */
      __pyx_v_L = (__pyx_v_L + 1);

      /* "ddtrace/internal/_encoding.pyx":519
 *             d = <dict> meta
 *             L = len(d)
 *             if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *                 L += 1
 *             if L > ITEM_LIMIT:
 */
    }

    /* "ddtrace/internal/_encoding.pyx":521
 *             if dd_origin is not NULL:
 *                 L += 1
 *             if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                 raise ValueError("dict is too large")
 * 
 */
    __pyx_t_1 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "ddtrace/internal/_encoding.pyx":522
 *                 L += 1
 *             if L > ITEM_LIMIT:
 *                 raise ValueError("dict is too large")             # <<<<<<<<<<<<<<
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 522, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 522, __pyx_L1_error)

      /* "ddtrace/internal/_encoding.pyx":521
 *             if dd_origin is not NULL:
 *                 L += 1
 *             if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                 raise ValueError("dict is too large")
 * 
 */
    }

    /* "ddtrace/internal/_encoding.pyx":524
 *                 raise ValueError("dict is too large")
 * 
 *             ret = msgpack_pack_map(&self.pk, L)             # <<<<<<<<<<<<<<
 *             if ret == 0:
 *                 for k, v in d.items():
 */
    __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->__pyx_base.pk), __pyx_v_L);

    /* "ddtrace/internal/_encoding.pyx":525
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 */
    __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
    if (__pyx_t_1) {

      /* "ddtrace/internal/_encoding.pyx":526
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:
 *                 for k, v in d.items():             # <<<<<<<<<<<<<<
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 */
      if (unlikely(__pyx_v_d == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 526, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Items(__pyx_v_d); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
        __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_3 = 0;
        __pyx_t_5 = NULL;
      } else {
        __pyx_t_3 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 526, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (;;) {
        if (likely(!__pyx_t_5)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 526, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 526, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_5(__pyx_t_4);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 526, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
          PyObject* sequence = __pyx_t_2;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 526, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          #else
          __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 526, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
          index = 0; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L9_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_6);
          index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L9_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_7);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 526, __pyx_L1_error)
          __pyx_t_9 = NULL;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          goto __pyx_L10_unpacking_done;
          __pyx_L9_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_9 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 526, __pyx_L1_error)
          __pyx_L10_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "ddtrace/internal/_encoding.pyx":527
 *             if ret == 0:
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)             # <<<<<<<<<<<<<<
 *                     if ret != 0: break
 *                     ret = pack_text(&self.pk, v)
 */
        __pyx_t_10 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_v_k); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 527, __pyx_L1_error)
        __pyx_v_ret = __pyx_t_10;

        /* "ddtrace/internal/_encoding.pyx":528
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break             # <<<<<<<<<<<<<<
 *                     ret = pack_text(&self.pk, v)
 *                     if ret != 0: break
 */
        __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
        if (__pyx_t_1) {
          goto __pyx_L8_break;
        }

        /* "ddtrace/internal/_encoding.pyx":529
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 *                     ret = pack_text(&self.pk, v)             # <<<<<<<<<<<<<<
 *                     if ret != 0: break
 *                 if dd_origin is not NULL:
 */
        __pyx_t_10 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_v_v); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 529, __pyx_L1_error)
        __pyx_v_ret = __pyx_t_10;

        /* "ddtrace/internal/_encoding.pyx":530
 *                     if ret != 0: break
 *                     ret = pack_text(&self.pk, v)
 *                     if ret != 0: break             # <<<<<<<<<<<<<<
 *                 if dd_origin is not NULL:
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 */
        __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
        if (__pyx_t_1) {
          goto __pyx_L8_break;
        }

        /* "ddtrace/internal/_encoding.pyx":526
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:
 *                 for k, v in d.items():             # <<<<<<<<<<<<<<
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 */
      }
      __pyx_L8_break:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ddtrace/internal/_encoding.pyx":531
 *                     ret = pack_text(&self.pk, v)
 *                     if ret != 0: break
 *                 if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 *                     if ret == 0:
 */
      __pyx_t_1 = ((__pyx_v_dd_origin != NULL) != 0);
      if (__pyx_t_1) {

        /* "ddtrace/internal/_encoding.pyx":532
 *                     if ret != 0: break
 *                 if dd_origin is not NULL:
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)             # <<<<<<<<<<<<<<
 *                     if ret == 0:
 *                         ret = pack_bytes(&self.pk, dd_origin, strlen(dd_origin))
 */
        __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), __pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY, __pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY_LEN);

        /* "ddtrace/internal/_encoding.pyx":533
 *                 if dd_origin is not NULL:
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 *                     if ret == 0:             # <<<<<<<<<<<<<<
 *                         ret = pack_bytes(&self.pk, dd_origin, strlen(dd_origin))
 *             return ret
 */
        __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
        if (__pyx_t_1) {

          /* "ddtrace/internal/_encoding.pyx":534
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 *                     if ret == 0:
 *                         ret = pack_bytes(&self.pk, dd_origin, strlen(dd_origin))             # <<<<<<<<<<<<<<
 *             return ret
 * 
 */
          __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), __pyx_v_dd_origin, strlen(__pyx_v_dd_origin));

          /* "ddtrace/internal/_encoding.pyx":533
 *                 if dd_origin is not NULL:
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 *                     if ret == 0:             # <<<<<<<<<<<<<<
 *                         ret = pack_bytes(&self.pk, dd_origin, strlen(dd_origin))
 *             return ret
 */
        }

        /* "ddtrace/internal/_encoding.pyx":531
 *                     ret = pack_text(&self.pk, v)
 *                     if ret != 0: break
 *                 if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *                     ret = pack_bytes(&self.pk, _ORIGIN_KEY, _ORIGIN_KEY_LEN)
 *                     if ret == 0:
 */
      }

      /* "ddtrace/internal/_encoding.pyx":525
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 */
    }

    /* "ddtrace/internal/_encoding.pyx":535
 *                     if ret == 0:
 *                         ret = pack_bytes(&self.pk, dd_origin, strlen(dd_origin))
 *             return ret             # <<<<<<<<<<<<<<
 * 
 *         raise TypeError("Unhandled meta type: %r" % type(meta))
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":516
 *         cdef dict d
 * 
 *         if PyDict_CheckExact(meta):             # <<<<<<<<<<<<<<
 *             d = <dict> meta
 *             L = len(d)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":537
 *             return ret
 * 
 *         raise TypeError("Unhandled meta type: %r" % type(meta))             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _pack_metrics(self, object metrics) except? -1:
 */
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_v_meta); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Unhandled_meta_type_r, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 537, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 537, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":511
 *         return string_to_buff(dd_origin)
 * 
 *     cdef inline int _pack_meta(self, object meta, char *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t L
 *         cdef int ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03._pack_meta", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":539
 *         raise TypeError("Unhandled meta type: %r" % type(meta))
 * 
 *     cdef inline int _pack_metrics(self, object metrics) except? -1:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t L
 *         cdef int ret
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_metrics(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_metrics) {
  Py_ssize_t __pyx_v_L;
  int __pyx_v_ret;
  PyObject *__pyx_v_d = 0;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_pack_metrics", 0);

  /* "ddtrace/internal/_encoding.pyx":544
 *         cdef dict d
 * 
 *         if PyDict_CheckExact(metrics):             # <<<<<<<<<<<<<<
 *             d = <dict> metrics
 *             L = len(d)
 */
  __pyx_t_1 = (PyDict_CheckExact(__pyx_v_metrics) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":545
 * 
 *         if PyDict_CheckExact(metrics):
 *             d = <dict> metrics             # <<<<<<<<<<<<<<
 *             L = len(d)
 *             if L > ITEM_LIMIT:
 */
    __pyx_t_2 = __pyx_v_metrics;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_d = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "ddtrace/internal/_encoding.pyx":546
 *         if PyDict_CheckExact(metrics):
 *             d = <dict> metrics
 *             L = len(d)             # <<<<<<<<<<<<<<
 *             if L > ITEM_LIMIT:
 *                 raise ValueError("dict is too large")
 */
    if (unlikely(__pyx_v_d == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 546, __pyx_L1_error)
    }
    __pyx_t_3 = PyDict_Size(__pyx_v_d); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(0, 546, __pyx_L1_error)
    __pyx_v_L = __pyx_t_3;

    /* "ddtrace/internal/_encoding.pyx":547
 *             d = <dict> metrics
 *             L = len(d)
 *             if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                 raise ValueError("dict is too large")
 * 
 */
    __pyx_t_1 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
    if (unlikely(__pyx_t_1)) {

      /* "ddtrace/internal/_encoding.pyx":548
 *             L = len(d)
 *             if L > ITEM_LIMIT:
 *                 raise ValueError("dict is too large")             # <<<<<<<<<<<<<<
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 548, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 548, __pyx_L1_error)

      /* "ddtrace/internal/_encoding.pyx":547
 *             d = <dict> metrics
 *             L = len(d)
 *             if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                 raise ValueError("dict is too large")
 * 
 */
    }

    /* "ddtrace/internal/_encoding.pyx":550
 *                 raise ValueError("dict is too large")
 * 
 *             ret = msgpack_pack_map(&self.pk, L)             # <<<<<<<<<<<<<<
 *             if ret == 0:
 *                 for k, v in d.items():
 */
    __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->__pyx_base.pk), __pyx_v_L);

    /* "ddtrace/internal/_encoding.pyx":551
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 */
    __pyx_t_1 = ((__pyx_v_ret == 0) != 0);
    if (__pyx_t_1) {

      /* "ddtrace/internal/_encoding.pyx":552
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:
 *                 for k, v in d.items():             # <<<<<<<<<<<<<<
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 */
      if (unlikely(__pyx_v_d == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 552, __pyx_L1_error)
      }
      __pyx_t_2 = __Pyx_PyDict_Items(__pyx_v_d); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
        __pyx_t_4 = __pyx_t_2; __Pyx_INCREF(__pyx_t_4); __pyx_t_3 = 0;
        __pyx_t_5 = NULL;
      } else {
        __pyx_t_3 = -1; __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 552, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext; if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 552, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      for (;;) {
        if (likely(!__pyx_t_5)) {
          if (likely(PyList_CheckExact(__pyx_t_4))) {
            if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyList_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 552, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          } else {
            if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_4)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_3); __Pyx_INCREF(__pyx_t_2); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 552, __pyx_L1_error)
            #else
            __pyx_t_2 = PySequence_ITEM(__pyx_t_4, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            #endif
          }
        } else {
          __pyx_t_2 = __pyx_t_5(__pyx_t_4);
          if (unlikely(!__pyx_t_2)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 552, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_2);
        }
        if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
          PyObject* sequence = __pyx_t_2;
          Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
          if (unlikely(size != 2)) {
            if (size > 2) __Pyx_RaiseTooManyValuesError(2);
            else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
            __PYX_ERR(0, 552, __pyx_L1_error)
          }
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          if (likely(PyTuple_CheckExact(sequence))) {
            __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
            __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
          } else {
            __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
            __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
          }
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_7);
          #else
          __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 552, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 552, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          #endif
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        } else {
          Py_ssize_t index = -1;
          __pyx_t_8 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 552, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
          index = 0; __pyx_t_6 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_6)) goto __pyx_L8_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_6);
          index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L8_unpacking_failed;
          __Pyx_GOTREF(__pyx_t_7);
          if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 552, __pyx_L1_error)
          __pyx_t_9 = NULL;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          goto __pyx_L9_unpacking_done;
          __pyx_L8_unpacking_failed:;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __pyx_t_9 = NULL;
          if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
          __PYX_ERR(0, 552, __pyx_L1_error)
          __pyx_L9_unpacking_done:;
        }
        __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_6);
        __pyx_t_6 = 0;
        __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
        __pyx_t_7 = 0;

        /* "ddtrace/internal/_encoding.pyx":553
 *             if ret == 0:
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)             # <<<<<<<<<<<<<<
 *                     if ret != 0: break
 *                     ret = pack_number(&self.pk, v)
 */
        __pyx_t_10 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_v_k); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 553, __pyx_L1_error)
        __pyx_v_ret = __pyx_t_10;

        /* "ddtrace/internal/_encoding.pyx":554
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break             # <<<<<<<<<<<<<<
 *                     ret = pack_number(&self.pk, v)
 *                     if ret != 0: break
 */
        __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
        if (__pyx_t_1) {
          goto __pyx_L7_break;
        }

        /* "ddtrace/internal/_encoding.pyx":555
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 *                     ret = pack_number(&self.pk, v)             # <<<<<<<<<<<<<<
 *                     if ret != 0: break
 *             return ret
 */
        __pyx_t_10 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_v_v); if (unlikely(__pyx_t_10 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 555, __pyx_L1_error)
        __pyx_v_ret = __pyx_t_10;

        /* "ddtrace/internal/_encoding.pyx":556
 *                     if ret != 0: break
 *                     ret = pack_number(&self.pk, v)
 *                     if ret != 0: break             # <<<<<<<<<<<<<<
 *             return ret
 * 
 */
        __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
        if (__pyx_t_1) {
          goto __pyx_L7_break;
        }

        /* "ddtrace/internal/_encoding.pyx":552
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:
 *                 for k, v in d.items():             # <<<<<<<<<<<<<<
 *                     ret = pack_text(&self.pk, k)
 *                     if ret != 0: break
 */
      }
      __pyx_L7_break:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "ddtrace/internal/_encoding.pyx":551
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 *             if ret == 0:             # <<<<<<<<<<<<<<
 *                 for k, v in d.items():
 *                     ret = pack_text(&self.pk, k)
 */
    }

    /* "ddtrace/internal/_encoding.pyx":557
 *                     ret = pack_number(&self.pk, v)
 *                     if ret != 0: break
 *             return ret             # <<<<<<<<<<<<<<
 * 
 *         raise TypeError("Unhandled metrics type: %r" % type(metrics))
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;

    /* "ddtrace/internal/_encoding.pyx":544
 *         cdef dict d
 * 
 *         if PyDict_CheckExact(metrics):             # <<<<<<<<<<<<<<
 *             d = <dict> metrics
 *             L = len(d)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":559
 *             return ret
 * 
 *         raise TypeError("Unhandled metrics type: %r" % type(metrics))             # <<<<<<<<<<<<<<
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:
 */
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7cpython_4type_type), __pyx_v_metrics); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_Unhandled_metrics_type_r, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_TypeError, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_Raise(__pyx_t_4, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __PYX_ERR(0, 559, __pyx_L1_error)

  /* "ddtrace/internal/_encoding.pyx":539
 *         raise TypeError("Unhandled meta type: %r" % type(meta))
 * 
 *     cdef inline int _pack_metrics(self, object metrics) except? -1:             # <<<<<<<<<<<<<<
 *         cdef Py_ssize_t L
 *         cdef int ret
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03._pack_metrics", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":561
 *         raise TypeError("Unhandled metrics type: %r" % type(metrics))
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         cdef Py_ssize_t L
 */

static int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_pack_span(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, PyObject *__pyx_v_span, void *__pyx_v_dd_origin) {
  int __pyx_v_ret;
  Py_ssize_t __pyx_v_L;
  int __pyx_v_has_span_type;
  int __pyx_v_has_meta;
  int __pyx_v_has_metrics;
  int __pyx_v_has_error;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_span", 0);

  /* "ddtrace/internal/_encoding.pyx":568
 *         cdef int has_metrics
 * 
 *         has_error = <bint> (span.error != 0)             # <<<<<<<<<<<<<<
 *         has_span_type = <bint> (span.span_type is not None)
 *         has_meta = <bint> (len(span._meta) > 0 or dd_origin is not NULL)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_NeObjC(__pyx_t_1, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_has_error = __pyx_t_3;

  /* "ddtrace/internal/_encoding.pyx":569
 * 
 *         has_error = <bint> (span.error != 0)
 *         has_span_type = <bint> (span.span_type is not None)             # <<<<<<<<<<<<<<
 *         has_meta = <bint> (len(span._meta) > 0 or dd_origin is not NULL)
 *         has_metrics = <bint> (len(span._metrics) > 0)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_span_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = (__pyx_t_2 != Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_has_span_type = (__pyx_t_3 != 0);

  /* "ddtrace/internal/_encoding.pyx":570
 *         has_error = <bint> (span.error != 0)
 *         has_span_type = <bint> (span.span_type is not None)
 *         has_meta = <bint> (len(span._meta) > 0 or dd_origin is not NULL)             # <<<<<<<<<<<<<<
 *         has_metrics = <bint> (len(span._metrics) > 0)
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_meta); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 570, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 570, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = ((__pyx_t_5 > 0) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_4 = __pyx_t_3;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_dd_origin != NULL) != 0);
  __pyx_t_4 = __pyx_t_3;
  __pyx_L3_bool_binop_done:;
  __pyx_v_has_meta = __pyx_t_4;

  /* "ddtrace/internal/_encoding.pyx":571
 *         has_span_type = <bint> (span.span_type is not None)
 *         has_meta = <bint> (len(span._meta) > 0 or dd_origin is not NULL)
 *         has_metrics = <bint> (len(span._metrics) > 0)             # <<<<<<<<<<<<<<
 * 
 *         L = 8 + has_span_type + has_meta + has_metrics + has_error
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_metrics); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 571, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_has_metrics = ((__pyx_t_5 > 0) != 0);

  /* "ddtrace/internal/_encoding.pyx":573
 *         has_metrics = <bint> (len(span._metrics) > 0)
 * 
 *         L = 8 + has_span_type + has_meta + has_metrics + has_error             # <<<<<<<<<<<<<<
 * 
 *         ret = msgpack_pack_map(&self.pk, L)
 */
  __pyx_v_L = ((((8 + __pyx_v_has_span_type) + __pyx_v_has_meta) + __pyx_v_has_metrics) + __pyx_v_has_error);

  /* "ddtrace/internal/_encoding.pyx":575
 *         L = 8 + has_span_type + has_meta + has_metrics + has_error
 * 
 *         ret = msgpack_pack_map(&self.pk, L)             # <<<<<<<<<<<<<<
 * 
 *         if ret == 0:
 */
  __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->__pyx_base.pk), __pyx_v_L);

  /* "ddtrace/internal/_encoding.pyx":577
 *         ret = msgpack_pack_map(&self.pk, L)
 * 
 *         if ret == 0:             # <<<<<<<<<<<<<<
 *             ret = pack_bytes(&self.pk, <char *> b"trace_id", 8)
 *             if ret != 0: return ret
 */
  __pyx_t_3 = ((__pyx_v_ret == 0) != 0);
  if (__pyx_t_3) {

    /* "ddtrace/internal/_encoding.pyx":578
 * 
 *         if ret == 0:
 *             ret = pack_bytes(&self.pk, <char *> b"trace_id", 8)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.trace_id)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"trace_id")), 8);

    /* "ddtrace/internal/_encoding.pyx":579
 *         if ret == 0:
 *             ret = pack_bytes(&self.pk, <char *> b"trace_id", 8)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_number(&self.pk, span.trace_id)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":580
 *             ret = pack_bytes(&self.pk, <char *> b"trace_id", 8)
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.trace_id)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_trace_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":581
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.trace_id)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"parent_id", 9)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":583
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"parent_id", 9)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.parent_id)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"parent_id")), 9);

    /* "ddtrace/internal/_encoding.pyx":584
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"parent_id", 9)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_number(&self.pk, span.parent_id)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":585
 *             ret = pack_bytes(&self.pk, <char *> b"parent_id", 9)
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.parent_id)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_parent_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 585, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 585, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":586
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.parent_id)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"span_id", 7)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":588
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"span_id", 7)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.span_id)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"span_id")), 7);

    /* "ddtrace/internal/_encoding.pyx":589
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"span_id", 7)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_number(&self.pk, span.span_id)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":590
 *             ret = pack_bytes(&self.pk, <char *> b"span_id", 7)
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.span_id)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_span_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":591
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.span_id)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"service", 7)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":593
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"service", 7)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.service)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"service")), 7);

    /* "ddtrace/internal/_encoding.pyx":594
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"service", 7)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_text(&self.pk, span.service)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":595
 *             ret = pack_bytes(&self.pk, <char *> b"service", 7)
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.service)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_service); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":596
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.service)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"resource", 8)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":598
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"resource", 8)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.resource)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"resource")), 8);

    /* "ddtrace/internal/_encoding.pyx":599
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"resource", 8)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_text(&self.pk, span.resource)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":600
 *             ret = pack_bytes(&self.pk, <char *> b"resource", 8)
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.resource)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_resource); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 600, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":601
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.resource)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"name", 4)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":603
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"name", 4)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.name)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"name")), 4);

    /* "ddtrace/internal/_encoding.pyx":604
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"name", 4)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_text(&self.pk, span.name)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":605
 *             ret = pack_bytes(&self.pk, <char *> b"name", 4)
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.name)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 605, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":606
 *             if ret != 0: return ret
 *             ret = pack_text(&self.pk, span.name)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"start", 5)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":608
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"start", 5)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.start_ns)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"start")), 5);

    /* "ddtrace/internal/_encoding.pyx":609
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"start", 5)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_number(&self.pk, span.start_ns)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":610
 *             ret = pack_bytes(&self.pk, <char *> b"start", 5)
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.start_ns)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_start_ns); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 610, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 610, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":611
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.start_ns)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"duration", 8)
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":613
 *             if ret != 0: return ret
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"duration", 8)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.duration_ns)
 */
    __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"duration")), 8);

    /* "ddtrace/internal/_encoding.pyx":614
 * 
 *             ret = pack_bytes(&self.pk, <char *> b"duration", 8)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = pack_number(&self.pk, span.duration_ns)
 *             if ret != 0: return ret
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":615
 *             ret = pack_bytes(&self.pk, <char *> b"duration", 8)
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.duration_ns)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_duration_ns); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 615, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 615, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_ret = __pyx_t_4;

    /* "ddtrace/internal/_encoding.pyx":616
 *             if ret != 0: return ret
 *             ret = pack_number(&self.pk, span.duration_ns)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             if has_error:
 */
    __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_3) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":618
 *             if ret != 0: return ret
 * 
 *             if has_error:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"error", 5)
 *                 if ret != 0: return ret
 */
    __pyx_t_3 = (__pyx_v_has_error != 0);
    if (__pyx_t_3) {

      /* "ddtrace/internal/_encoding.pyx":619
 * 
 *             if has_error:
 *                 ret = pack_bytes(&self.pk, <char *> b"error", 5)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = msgpack_pack_long(&self.pk, <long> 1)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"error")), 5);

      /* "ddtrace/internal/_encoding.pyx":620
 *             if has_error:
 *                 ret = pack_bytes(&self.pk, <char *> b"error", 5)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_long(&self.pk, <long> 1)
 *                 if ret != 0: return ret
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":621
 *                 ret = pack_bytes(&self.pk, <char *> b"error", 5)
 *                 if ret != 0: return ret
 *                 ret = msgpack_pack_long(&self.pk, <long> 1)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 * 
 */
      __pyx_v_ret = msgpack_pack_long((&__pyx_v_self->__pyx_base.pk), ((long)1));

      /* "ddtrace/internal/_encoding.pyx":622
 *                 if ret != 0: return ret
 *                 ret = msgpack_pack_long(&self.pk, <long> 1)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             if has_span_type:
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":618
 *             if ret != 0: return ret
 * 
 *             if has_error:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"error", 5)
 *                 if ret != 0: return ret
 */
    }

    /* "ddtrace/internal/_encoding.pyx":624
 *                 if ret != 0: return ret
 * 
 *             if has_span_type:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"type", 4)
 *                 if ret != 0: return ret
 */
    __pyx_t_3 = (__pyx_v_has_span_type != 0);
    if (__pyx_t_3) {

      /* "ddtrace/internal/_encoding.pyx":625
 * 
 *             if has_span_type:
 *                 ret = pack_bytes(&self.pk, <char *> b"type", 4)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = pack_text(&self.pk, span.span_type)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"type")), 4);

      /* "ddtrace/internal/_encoding.pyx":626
 *             if has_span_type:
 *                 ret = pack_bytes(&self.pk, <char *> b"type", 4)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = pack_text(&self.pk, span.span_type)
 *                 if ret != 0: return ret
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":627
 *                 ret = pack_bytes(&self.pk, <char *> b"type", 4)
 *                 if ret != 0: return ret
 *                 ret = pack_text(&self.pk, span.span_type)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_span_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_pack_text((&__pyx_v_self->__pyx_base.pk), __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_ret = __pyx_t_4;

      /* "ddtrace/internal/_encoding.pyx":628
 *                 if ret != 0: return ret
 *                 ret = pack_text(&self.pk, span.span_type)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             if has_meta:
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":624
 *                 if ret != 0: return ret
 * 
 *             if has_span_type:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"type", 4)
 *                 if ret != 0: return ret
 */
    }

    /* "ddtrace/internal/_encoding.pyx":630
 *                 if ret != 0: return ret
 * 
 *             if has_meta:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"meta", 4)
 *                 if ret != 0: return ret
 */
    __pyx_t_3 = (__pyx_v_has_meta != 0);
    if (__pyx_t_3) {

      /* "ddtrace/internal/_encoding.pyx":631
 * 
 *             if has_meta:
 *                 ret = pack_bytes(&self.pk, <char *> b"meta", 4)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = self._pack_meta(span._meta, <char *> dd_origin)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"meta")), 4);

      /* "ddtrace/internal/_encoding.pyx":632
 *             if has_meta:
 *                 ret = pack_bytes(&self.pk, <char *> b"meta", 4)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = self._pack_meta(span._meta, <char *> dd_origin)
 *                 if ret != 0: return ret
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":633
 *                 ret = pack_bytes(&self.pk, <char *> b"meta", 4)
 *                 if ret != 0: return ret
 *                 ret = self._pack_meta(span._meta, <char *> dd_origin)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_meta); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 633, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_meta(__pyx_v_self, __pyx_t_2, ((char *)__pyx_v_dd_origin)); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 633, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_ret = __pyx_t_4;

      /* "ddtrace/internal/_encoding.pyx":634
 *                 if ret != 0: return ret
 *                 ret = self._pack_meta(span._meta, <char *> dd_origin)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *             if has_metrics:
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":630
 *                 if ret != 0: return ret
 * 
 *             if has_meta:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"meta", 4)
 *                 if ret != 0: return ret
 */
    }

    /* "ddtrace/internal/_encoding.pyx":636
 *                 if ret != 0: return ret
 * 
 *             if has_metrics:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"metrics", 7)
 *                 if ret != 0: return ret
 */
    __pyx_t_3 = (__pyx_v_has_metrics != 0);
    if (__pyx_t_3) {

      /* "ddtrace/internal/_encoding.pyx":637
 * 
 *             if has_metrics:
 *                 ret = pack_bytes(&self.pk, <char *> b"metrics", 7)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = self._pack_metrics(span._metrics)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_pack_bytes((&__pyx_v_self->__pyx_base.pk), ((char *)((char *)"metrics")), 7);

      /* "ddtrace/internal/_encoding.pyx":638
 *             if has_metrics:
 *                 ret = pack_bytes(&self.pk, <char *> b"metrics", 7)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = self._pack_metrics(span._metrics)
 *                 if ret != 0: return ret
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":639
 *                 ret = pack_bytes(&self.pk, <char *> b"metrics", 7)
 *                 if ret != 0: return ret
 *                 ret = self._pack_metrics(span._metrics)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 * 
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_metrics); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_metrics(__pyx_v_self, __pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_ret = __pyx_t_4;

      /* "ddtrace/internal/_encoding.pyx":640
 *                 if ret != 0: return ret
 *                 ret = self._pack_metrics(span._metrics)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         return ret
 */
      __pyx_t_3 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_3) {
        __pyx_r = __pyx_v_ret;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":636
 *                 if ret != 0: return ret
 * 
 *             if has_metrics:             # <<<<<<<<<<<<<<
 *                 ret = pack_bytes(&self.pk, <char *> b"metrics", 7)
 *                 if ret != 0: return ret
 */
    }

    /* "ddtrace/internal/_encoding.pyx":577
 *         ret = msgpack_pack_map(&self.pk, L)
 * 
 *         if ret == 0:             # <<<<<<<<<<<<<<
 *             ret = pack_bytes(&self.pk, <char *> b"trace_id", 8)
 *             if ret != 0: return ret
 */
  }

  /* "ddtrace/internal/_encoding.pyx":642
 *                 if ret != 0: return ret
 * 
 *         return ret             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_ret;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":561
 *         raise TypeError("Unhandled metrics type: %r" % type(metrics))
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         cdef Py_ssize_t L
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.pack_span", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_2__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_2__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_4__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_4__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV03.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":648
 *     cdef MsgpackStringTable _st
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self._st = MsgpackStringTable(max_size)
 * 
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  size_t __pyx_v_max_size;
  CYTHON_UNUSED size_t __pyx_v_max_item_size;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_max_size,&__pyx_n_s_max_item_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_max_item_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, 1); __PYX_ERR(0, 648, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 648, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_max_size = __Pyx_PyInt_As_size_t(values[0]); if (unlikely((__pyx_v_max_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L3_error)
    __pyx_v_max_item_size = __Pyx_PyInt_As_size_t(values[1]); if (unlikely((__pyx_v_max_item_size == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 648, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 648, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05___cinit__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self), __pyx_v_max_size, __pyx_v_max_item_size);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, size_t __pyx_v_max_size, CYTHON_UNUSED size_t __pyx_v_max_item_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ddtrace/internal/_encoding.pyx":649
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):
 *         self._st = MsgpackStringTable(max_size)             # <<<<<<<<<<<<<<
 * 
 *     cpdef flush(self):
 */
  __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_max_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_st);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_st));
  __pyx_v_self->_st = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":648
 *     cdef MsgpackStringTable _st
 * 
 *     def __cinit__(self, size_t max_size, size_t max_item_size):             # <<<<<<<<<<<<<<
 *         self._st = MsgpackStringTable(max_size)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":651
 *         self._st = MsgpackStringTable(max_size)
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_3flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  long __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  char const *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  int __pyx_t_20;
  int __pyx_t_21;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_flush); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_3flush)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 651, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":652
 * 
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._st.append_raw(PyLong_FromLong(<long> self.get_buffer()), <Py_ssize_t> super(MsgpackEncoderV05, self).size)
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 652, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 652, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":653
 *     cpdef flush(self):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._st.append_raw(PyLong_FromLong(<long> self.get_buffer()), <Py_ssize_t> super(MsgpackEncoderV05, self).size)
 *                 return self._st.flush()
 */
          /*try:*/ {

            /* "ddtrace/internal/_encoding.pyx":654
 *         with self._lock:
 *             try:
 *                 self._st.append_raw(PyLong_FromLong(<long> self.get_buffer()), <Py_ssize_t> super(MsgpackEncoderV05, self).size)             # <<<<<<<<<<<<<<
 *                 return self._st.flush()
 *             finally:
 */
            __pyx_t_1 = PyLong_FromLong(((long)((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get_buffer(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_t_9 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_9 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_INCREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
            __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
            PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
            __Pyx_INCREF(((PyObject *)__pyx_v_self));
            __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
            PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_size); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __pyx_t_10 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_10 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
            __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->append_raw(__pyx_v_self->_st, __pyx_t_9, ((Py_ssize_t)__pyx_t_10)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 654, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

            /* "ddtrace/internal/_encoding.pyx":655
 *             try:
 *                 self._st.append_raw(PyLong_FromLong(<long> self.get_buffer()), <Py_ssize_t> super(MsgpackEncoderV05, self).size)
 *                 return self._st.flush()             # <<<<<<<<<<<<<<
 *             finally:
 *                 self._reset_buffer()
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->flush(__pyx_v_self->_st, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 655, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_1);
            __pyx_r = __pyx_t_1;
            __pyx_t_1 = 0;
            goto __pyx_L13_return;
          }

          /* "ddtrace/internal/_encoding.pyx":657
 *                 return self._st.flush()
 *             finally:
 *                 self._reset_buffer()             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
          /*finally:*/ {
            __pyx_L14_error:;
            /*exception exit:*/{
              __Pyx_PyThreadState_declare
              __Pyx_PyThreadState_assign
              __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              if (PY_MAJOR_VERSION >= 3) __Pyx_ExceptionSwap(&__pyx_t_17, &__pyx_t_18, &__pyx_t_19);
              if ((PY_MAJOR_VERSION < 3) || unlikely(__Pyx_GetException(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16) < 0)) __Pyx_ErrFetch(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_15);
              __Pyx_XGOTREF(__pyx_t_16);
              __Pyx_XGOTREF(__pyx_t_17);
              __Pyx_XGOTREF(__pyx_t_18);
              __Pyx_XGOTREF(__pyx_t_19);
              __pyx_t_11 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_13 = __pyx_filename;
              {
                __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._reset_buffer(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 657, __pyx_L17_error)
                __Pyx_GOTREF(__pyx_t_1);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              }
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_XGIVEREF(__pyx_t_18);
                __Pyx_XGIVEREF(__pyx_t_19);
                __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
              }
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_XGIVEREF(__pyx_t_15);
              __Pyx_XGIVEREF(__pyx_t_16);
              __Pyx_ErrRestore(__pyx_t_14, __pyx_t_15, __pyx_t_16);
              __pyx_t_14 = 0; __pyx_t_15 = 0; __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
              __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_13;
              goto __pyx_L7_error;
              __pyx_L17_error:;
              if (PY_MAJOR_VERSION >= 3) {
                __Pyx_XGIVEREF(__pyx_t_17);
                __Pyx_XGIVEREF(__pyx_t_18);
                __Pyx_XGIVEREF(__pyx_t_19);
                __Pyx_ExceptionReset(__pyx_t_17, __pyx_t_18, __pyx_t_19);
              }
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
              __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
              __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0;
              goto __pyx_L7_error;
            }
            __pyx_L13_return: {
              __pyx_t_19 = __pyx_r;
              __pyx_r = 0;
              __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._reset_buffer(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 657, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
              __pyx_r = __pyx_t_19;
              __pyx_t_19 = 0;
              goto __pyx_L11_try_return;
            }
          }

          /* "ddtrace/internal/_encoding.pyx":652
 * 
 *     cpdef flush(self):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._st.append_raw(PyLong_FromLong(<long> self.get_buffer()), <Py_ssize_t> super(MsgpackEncoderV05, self).size)
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 652, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 652, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_19 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_19)) __PYX_ERR(0, 652, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_19);
          __pyx_t_20 = __Pyx_PyObject_IsTrue(__pyx_t_19);
          __Pyx_DECREF(__pyx_t_19); __pyx_t_19 = 0;
          if (__pyx_t_20 < 0) __PYX_ERR(0, 652, __pyx_L9_except_error)
          __pyx_t_21 = ((!(__pyx_t_20 != 0)) != 0);
          if (__pyx_t_21) {
            __Pyx_GIVEREF(__pyx_t_1);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_3);
            __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
            __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
            __PYX_ERR(0, 652, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 652, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_8 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_5) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 652, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L21;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L21:;
  }

  /* "ddtrace/internal/_encoding.pyx":651
 *         self._st = MsgpackStringTable(max_size)
 * 
 *     cpdef flush(self):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_3flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_3flush(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("flush (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_2flush(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_2flush(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("flush", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_flush(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.flush", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":660
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size___get__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size___get__(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "ddtrace/internal/_encoding.pyx":662
 *     def size(self):
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self._st.size + super(MsgpackEncoderV05, self).size
 * 
 */
  /*with:*/ {
    __pyx_t_1 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 662, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":663
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 *             return self._st.size + super(MsgpackEncoderV05, self).size             # <<<<<<<<<<<<<<
 * 
 *     cpdef put(self, list trace):
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->_st), __pyx_n_s_size); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 663, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_INCREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
          __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
          PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
          __Pyx_INCREF(((PyObject *)__pyx_v_self));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
          PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
          __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 663, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_size); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_t_4 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 663, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __pyx_r = __pyx_t_4;
          __pyx_t_4 = 0;
          goto __pyx_L11_try_return;

          /* "ddtrace/internal/_encoding.pyx":662
 *     def size(self):
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             return self._st.size + super(MsgpackEncoderV05, self).size
 * 
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 662, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_8 = PyTuple_Pack(3, __pyx_t_4, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 662, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_8, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 662, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_9);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (__pyx_t_10 < 0) __PYX_ERR(0, 662, __pyx_L9_except_error)
          __pyx_t_11 = ((!(__pyx_t_10 != 0)) != 0);
          if (__pyx_t_11) {
            __Pyx_GIVEREF(__pyx_t_4);
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_XGIVEREF(__pyx_t_2);
            __Pyx_ErrRestoreWithState(__pyx_t_4, __pyx_t_3, __pyx_t_2);
            __pyx_t_4 = 0; __pyx_t_3 = 0; __pyx_t_2 = 0; 
            __PYX_ERR(0, 662, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
        goto __pyx_L4_return;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_1) {
          __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 662, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L4_return: {
        __pyx_t_7 = __pyx_r;
        __pyx_r = 0;
        if (__pyx_t_1) {
          __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 662, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __pyx_r = __pyx_t_7;
        __pyx_t_7 = 0;
        goto __pyx_L0;
      }
      __pyx_L6:;
    }
    goto __pyx_L16;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    goto __pyx_L1_error;
    __pyx_L16:;
  }

  /* "ddtrace/internal/_encoding.pyx":660
 * 
 *     @property
 *     def size(self):             # <<<<<<<<<<<<<<
 *         """Return the size in bytes of the encoder buffer."""
 *         with self._lock:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.size.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":665
 *             return self._st.size + super(MsgpackEncoderV05, self).size
 * 
 *     cpdef put(self, list trace):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_trace, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_t_13;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_put); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 665, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_5put)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_trace) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_trace);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":666
 * 
 *     cpdef put(self, list trace):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._st.savepoint()
 */
  /*with:*/ {
    __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_exit); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 666, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->__pyx_base.__pyx_base._lock, __pyx_n_s_enter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 666, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 666, __pyx_L3_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    /*try:*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":667
 *     cpdef put(self, list trace):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._st.savepoint()
 *                 super(MsgpackEncoderV05, self).put(trace)
 */
          {
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
            __Pyx_XGOTREF(__pyx_t_9);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_11);
            /*try:*/ {

              /* "ddtrace/internal/_encoding.pyx":668
 *         with self._lock:
 *             try:
 *                 self._st.savepoint()             # <<<<<<<<<<<<<<
 *                 super(MsgpackEncoderV05, self).put(trace)
 *             except Exception:
 */
              __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->savepoint(__pyx_v_self->_st); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 668, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "ddtrace/internal/_encoding.pyx":669
 *             try:
 *                 self._st.savepoint()
 *                 super(MsgpackEncoderV05, self).put(trace)             # <<<<<<<<<<<<<<
 *             except Exception:
 *                 self._st.rollback()
 */
              __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_INCREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
              __Pyx_GIVEREF(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
              PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05));
              __Pyx_INCREF(((PyObject *)__pyx_v_self));
              __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
              PyTuple_SET_ITEM(__pyx_t_2, 1, ((PyObject *)__pyx_v_self));
              __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 669, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_put); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 669, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
              __pyx_t_3 = NULL;
              if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
                __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
                if (likely(__pyx_t_3)) {
                  PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
                  __Pyx_INCREF(__pyx_t_3);
                  __Pyx_INCREF(function);
                  __Pyx_DECREF_SET(__pyx_t_2, function);
                }
              }
              __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_trace) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_trace);
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 669, __pyx_L13_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

              /* "ddtrace/internal/_encoding.pyx":667
 *     cpdef put(self, list trace):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._st.savepoint()
 *                 super(MsgpackEncoderV05, self).put(trace)
 */
            }
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            goto __pyx_L18_try_end;
            __pyx_L13_error:;
            __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

            /* "ddtrace/internal/_encoding.pyx":670
 *                 self._st.savepoint()
 *                 super(MsgpackEncoderV05, self).put(trace)
 *             except Exception:             # <<<<<<<<<<<<<<
 *                 self._st.rollback()
 *                 raise
 */
            __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
            if (__pyx_t_12) {
              __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
              if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 670, __pyx_L15_except_error)
              __Pyx_GOTREF(__pyx_t_1);
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_GOTREF(__pyx_t_3);

              /* "ddtrace/internal/_encoding.pyx":671
 *                 super(MsgpackEncoderV05, self).put(trace)
 *             except Exception:
 *                 self._st.rollback()             # <<<<<<<<<<<<<<
 *                 raise
 * 
 */
              __pyx_t_4 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->rollback(__pyx_v_self->_st); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 671, __pyx_L15_except_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

              /* "ddtrace/internal/_encoding.pyx":672
 *             except Exception:
 *                 self._st.rollback()
 *                 raise             # <<<<<<<<<<<<<<
 * 
 *     cdef inline int _pack_string(self, object string):
 */
              __Pyx_GIVEREF(__pyx_t_1);
              __Pyx_GIVEREF(__pyx_t_2);
              __Pyx_XGIVEREF(__pyx_t_3);
              __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
              __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
              __PYX_ERR(0, 672, __pyx_L15_except_error)
            }
            goto __pyx_L15_except_error;
            __pyx_L15_except_error:;

            /* "ddtrace/internal/_encoding.pyx":667
 *     cpdef put(self, list trace):
 *         with self._lock:
 *             try:             # <<<<<<<<<<<<<<
 *                 self._st.savepoint()
 *                 super(MsgpackEncoderV05, self).put(trace)
 */
            __Pyx_XGIVEREF(__pyx_t_9);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_11);
            __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
            goto __pyx_L7_error;
            __pyx_L18_try_end:;
          }

          /* "ddtrace/internal/_encoding.pyx":666
 * 
 *     cpdef put(self, list trace):
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             try:
 *                 self._st.savepoint()
 */
        }
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L12_try_end;
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        /*except:*/ {
          __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_2, &__pyx_t_1) < 0) __PYX_ERR(0, 666, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_GOTREF(__pyx_t_1);
          __pyx_t_4 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 666, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 666, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_13 = __Pyx_PyObject_IsTrue(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (__pyx_t_13 < 0) __PYX_ERR(0, 666, __pyx_L9_except_error)
          __pyx_t_14 = ((!(__pyx_t_13 != 0)) != 0);
          if (__pyx_t_14) {
            __Pyx_GIVEREF(__pyx_t_3);
            __Pyx_GIVEREF(__pyx_t_2);
            __Pyx_XGIVEREF(__pyx_t_1);
            __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_2, __pyx_t_1);
            __pyx_t_3 = 0; __pyx_t_2 = 0; __pyx_t_1 = 0; 
            __PYX_ERR(0, 666, __pyx_L9_except_error)
          }
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          goto __pyx_L8_exception_handled;
        }
        __pyx_L9_except_error:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        goto __pyx_L1_error;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_6);
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
        __pyx_L12_try_end:;
      }
    }
    /*finally:*/ {
      /*normal exit:*/{
        if (__pyx_t_5) {
          __pyx_t_8 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_tuple__6, NULL);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 666, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        goto __pyx_L6;
      }
      __pyx_L6:;
    }
    goto __pyx_L24;
    __pyx_L3_error:;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L1_error;
    __pyx_L24:;
  }

  /* "ddtrace/internal/_encoding.pyx":665
 *             return self._st.size + super(MsgpackEncoderV05, self).size
 * 
 *     cpdef put(self, list trace):             # <<<<<<<<<<<<<<
 *         with self._lock:
 *             try:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_5put(PyObject *__pyx_v_self, PyObject *__pyx_v_trace) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_trace), (&PyList_Type), 1, "trace", 1))) __PYX_ERR(0, 665, __pyx_L1_error)
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4put(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self), ((PyObject*)__pyx_v_trace));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4put(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_trace) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_put(__pyx_v_self, __pyx_v_trace, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":674
 *                 raise
 * 
 *     cdef inline int _pack_string(self, object string):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_uint32(&self.pk, self._st._index(string))
 * 
 */

static CYTHON_INLINE int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_string) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_pack_string", 0);

  /* "ddtrace/internal/_encoding.pyx":675
 * 
 *     cdef inline int _pack_string(self, object string):
 *         return msgpack_pack_uint32(&self.pk, self._st._index(string))             # <<<<<<<<<<<<<<
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 */
  __pyx_r = msgpack_pack_uint32((&__pyx_v_self->__pyx_base.pk), ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->__pyx_base._index(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->_st), __pyx_v_string));
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":674
 *                 raise
 * 
 *     cdef inline int _pack_string(self, object string):             # <<<<<<<<<<<<<<
 *         return msgpack_pack_uint32(&self.pk, self._st._index(string))
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":677
 *         return msgpack_pack_uint32(&self.pk, self._st._index(string))
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         return <void *> PyLong_AsLong(self._st._index(dd_origin))
 * 
 */

static void *__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_get_dd_origin_ref(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_dd_origin) {
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  long __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_dd_origin_ref", 0);

  /* "ddtrace/internal/_encoding.pyx":678
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):
 *         return <void *> PyLong_AsLong(self._st._index(dd_origin))             # <<<<<<<<<<<<<<
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:
 */
  __pyx_t_1 = __Pyx_PyInt_From_uint32_t(((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable *)__pyx_v_self->_st->__pyx_base.__pyx_vtab)->__pyx_base._index(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v_self->_st), __pyx_v_dd_origin)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 678, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyLong_AsLong(__pyx_t_1); if (unlikely(__pyx_t_2 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 678, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = ((void *)__pyx_t_2);
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":677
 *         return msgpack_pack_uint32(&self.pk, self._st._index(string))
 * 
 *     cdef void * get_dd_origin_ref(self, str dd_origin):             # <<<<<<<<<<<<<<
 *         return <void *> PyLong_AsLong(self._st._index(dd_origin))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("ddtrace.internal._encoding.MsgpackEncoderV05.get_dd_origin_ref", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":680
 *         return <void *> PyLong_AsLong(self._st._index(dd_origin))
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 * 
 */

static int __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_pack_span(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, PyObject *__pyx_v_span, void *__pyx_v_dd_origin) {
  int __pyx_v_ret;
  PyObject *__pyx_v__ = NULL;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  uint64_t __pyx_t_3;
  uint64_t __pyx_t_4;
  int64_t __pyx_t_5;
  int64_t __pyx_t_6;
  int32_t __pyx_t_7;
  int32_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *(*__pyx_t_15)(PyObject *);
  int __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack_span", 0);

  /* "ddtrace/internal/_encoding.pyx":683
 *         cdef int ret
 * 
 *         ret = msgpack_pack_array(&self.pk, 12)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_v_ret = msgpack_pack_array((&__pyx_v_self->__pyx_base.pk), 12);

  /* "ddtrace/internal/_encoding.pyx":684
 * 
 *         ret = msgpack_pack_array(&self.pk, 12)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         ret = self._pack_string(span.service)
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":686
 *         if ret != 0: return ret
 * 
 *         ret = self._pack_string(span.service)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.name)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_service); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":687
 * 
 *         ret = self._pack_string(span.service)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 *         ret = self._pack_string(span.name)
 *         if ret != 0: return ret
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":688
 *         ret = self._pack_string(span.service)
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.name)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.resource)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 688, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":689
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.name)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 *         ret = self._pack_string(span.resource)
 *         if ret != 0: return ret
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":690
 *         ret = self._pack_string(span.name)
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.resource)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_resource); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 690, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":691
 *         if ret != 0: return ret
 *         ret = self._pack_string(span.resource)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.trace_id
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":693
 *         if ret != 0: return ret
 * 
 *         _ = span.trace_id             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_trace_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 693, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__ = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":694
 * 
 *         _ = span.trace_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_4 = __Pyx_PyInt_As_uint64_t(__pyx_v__); if (unlikely((__pyx_t_4 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 694, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_4;
  } else {
    __pyx_t_3 = 0;
  }
  __pyx_v_ret = msgpack_pack_uint64((&__pyx_v_self->__pyx_base.pk), __pyx_t_3);

  /* "ddtrace/internal/_encoding.pyx":695
 *         _ = span.trace_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.span_id
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":697
 *         if ret != 0: return ret
 * 
 *         _ = span.span_id             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_span_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v__, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":698
 * 
 *         _ = span.span_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_4 = __Pyx_PyInt_As_uint64_t(__pyx_v__); if (unlikely((__pyx_t_4 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 698, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_4;
  } else {
    __pyx_t_3 = 0;
  }
  __pyx_v_ret = msgpack_pack_uint64((&__pyx_v_self->__pyx_base.pk), __pyx_t_3);

  /* "ddtrace/internal/_encoding.pyx":699
 *         _ = span.span_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.parent_id
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":701
 *         if ret != 0: return ret
 * 
 *         _ = span.parent_id             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_parent_id); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v__, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":702
 * 
 *         _ = span.parent_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_4 = __Pyx_PyInt_As_uint64_t(__pyx_v__); if (unlikely((__pyx_t_4 == ((uint64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 702, __pyx_L1_error)
    __pyx_t_3 = __pyx_t_4;
  } else {
    __pyx_t_3 = 0;
  }
  __pyx_v_ret = msgpack_pack_uint64((&__pyx_v_self->__pyx_base.pk), __pyx_t_3);

  /* "ddtrace/internal/_encoding.pyx":703
 *         _ = span.parent_id
 *         ret = msgpack_pack_uint64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.start_ns
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":705
 *         if ret != 0: return ret
 * 
 *         _ = span.start_ns             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_start_ns); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 705, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v__, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":706
 * 
 *         _ = span.start_ns
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_6 = __Pyx_PyInt_As_int64_t(__pyx_v__); if (unlikely((__pyx_t_6 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 706, __pyx_L1_error)
    __pyx_t_5 = __pyx_t_6;
  } else {
    __pyx_t_5 = 0;
  }
  __pyx_v_ret = msgpack_pack_int64((&__pyx_v_self->__pyx_base.pk), __pyx_t_5);

  /* "ddtrace/internal/_encoding.pyx":707
 *         _ = span.start_ns
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.duration_ns
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":709
 *         if ret != 0: return ret
 * 
 *         _ = span.duration_ns             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_duration_ns); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v__, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":710
 * 
 *         _ = span.duration_ns
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_6 = __Pyx_PyInt_As_int64_t(__pyx_v__); if (unlikely((__pyx_t_6 == ((int64_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 710, __pyx_L1_error)
    __pyx_t_5 = __pyx_t_6;
  } else {
    __pyx_t_5 = 0;
  }
  __pyx_v_ret = msgpack_pack_int64((&__pyx_v_self->__pyx_base.pk), __pyx_t_5);

  /* "ddtrace/internal/_encoding.pyx":711
 *         _ = span.duration_ns
 *         ret = msgpack_pack_int64(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         _ = span.error
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":713
 *         if ret != 0: return ret
 * 
 *         _ = span.error             # <<<<<<<<<<<<<<
 *         ret = msgpack_pack_int32(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 713, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v__, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":714
 * 
 *         _ = span.error
 *         ret = msgpack_pack_int32(&self.pk, _ if _ is not None else 0)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_1 = (__pyx_v__ != Py_None);
  if ((__pyx_t_1 != 0)) {
    __pyx_t_8 = __Pyx_PyInt_As_int32_t(__pyx_v__); if (unlikely((__pyx_t_8 == ((int32_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 714, __pyx_L1_error)
    __pyx_t_7 = __pyx_t_8;
  } else {
    __pyx_t_7 = 0;
  }
  __pyx_v_ret = msgpack_pack_int32((&__pyx_v_self->__pyx_base.pk), __pyx_t_7);

  /* "ddtrace/internal/_encoding.pyx":715
 *         _ = span.error
 *         ret = msgpack_pack_int32(&self.pk, _ if _ is not None else 0)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._meta) + (dd_origin is not NULL))
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":717
 *         if ret != 0: return ret
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._meta) + (dd_origin is not NULL))             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 *         if span._meta:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_meta); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 717, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->__pyx_base.pk), (__pyx_t_9 + (__pyx_v_dd_origin != NULL)));

  /* "ddtrace/internal/_encoding.pyx":718
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._meta) + (dd_origin is not NULL))
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 *         if span._meta:
 *             for k, v in span._meta.items():
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":719
 *         ret = msgpack_pack_map(&self.pk, len(span._meta) + (dd_origin is not NULL))
 *         if ret != 0: return ret
 *         if span._meta:             # <<<<<<<<<<<<<<
 *             for k, v in span._meta.items():
 *                 ret = self._pack_string(k)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_meta); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 719, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":720
 *         if ret != 0: return ret
 *         if span._meta:
 *             for k, v in span._meta.items():             # <<<<<<<<<<<<<<
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_meta); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_items); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_10) : __Pyx_PyObject_CallNoArg(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (likely(PyList_CheckExact(__pyx_t_2)) || PyTuple_CheckExact(__pyx_t_2)) {
      __pyx_t_11 = __pyx_t_2; __Pyx_INCREF(__pyx_t_11); __pyx_t_9 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_11 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 720, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_12 = Py_TYPE(__pyx_t_11)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 720, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_11))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_11)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 720, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_11, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_11)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_11, __pyx_t_9); __Pyx_INCREF(__pyx_t_2); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 720, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_11, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 720, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_12(__pyx_t_11);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 720, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 720, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_13 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_13 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_13);
        #else
        __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_13 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_14 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 720, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_15 = Py_TYPE(__pyx_t_14)->tp_iternext;
        index = 0; __pyx_t_10 = __pyx_t_15(__pyx_t_14); if (unlikely(!__pyx_t_10)) goto __pyx_L17_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        index = 1; __pyx_t_13 = __pyx_t_15(__pyx_t_14); if (unlikely(!__pyx_t_13)) goto __pyx_L17_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_13);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_14), 2) < 0) __PYX_ERR(0, 720, __pyx_L1_error)
        __pyx_t_15 = NULL;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        goto __pyx_L18_unpacking_done;
        __pyx_L17_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_15 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 720, __pyx_L1_error)
        __pyx_L18_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_10);
      __pyx_t_10 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_13);
      __pyx_t_13 = 0;

      /* "ddtrace/internal/_encoding.pyx":721
 *         if span._meta:
 *             for k, v in span._meta.items():
 *                 ret = self._pack_string(k)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = self._pack_string(v)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_v_k);

      /* "ddtrace/internal/_encoding.pyx":722
 *             for k, v in span._meta.items():
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = self._pack_string(v)
 *                 if ret != 0: return ret
 */
      __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_1) {
        __pyx_r = __pyx_v_ret;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":723
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 *                 ret = self._pack_string(v)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *         if dd_origin is not NULL:
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_v_v);

      /* "ddtrace/internal/_encoding.pyx":724
 *                 if ret != 0: return ret
 *                 ret = self._pack_string(v)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *         if dd_origin is not NULL:
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)
 */
      __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_1) {
        __pyx_r = __pyx_v_ret;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":720
 *         if ret != 0: return ret
 *         if span._meta:
 *             for k, v in span._meta.items():             # <<<<<<<<<<<<<<
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 */
    }
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "ddtrace/internal/_encoding.pyx":719
 *         ret = msgpack_pack_map(&self.pk, len(span._meta) + (dd_origin is not NULL))
 *         if ret != 0: return ret
 *         if span._meta:             # <<<<<<<<<<<<<<
 *             for k, v in span._meta.items():
 *                 ret = self._pack_string(k)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":725
 *                 ret = self._pack_string(v)
 *                 if ret != 0: return ret
 *         if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)
 *             if ret != 0: return ret
 */
  __pyx_t_1 = ((__pyx_v_dd_origin != NULL) != 0);
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":726
 *                 if ret != 0: return ret
 *         if dd_origin is not NULL:
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> dd_origin)
 */
    __pyx_v_ret = msgpack_pack_uint32((&__pyx_v_self->__pyx_base.pk), ((uint32_t)1));

    /* "ddtrace/internal/_encoding.pyx":727
 *         if dd_origin is not NULL:
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> dd_origin)
 *             if ret != 0: return ret
 */
    __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_1) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":728
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)
 *             if ret != 0: return ret
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> dd_origin)             # <<<<<<<<<<<<<<
 *             if ret != 0: return ret
 * 
 */
    __pyx_v_ret = msgpack_pack_uint32((&__pyx_v_self->__pyx_base.pk), ((uint32_t)__pyx_v_dd_origin));

    /* "ddtrace/internal/_encoding.pyx":729
 *             if ret != 0: return ret
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> dd_origin)
 *             if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._metrics))
 */
    __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
    if (__pyx_t_1) {
      __pyx_r = __pyx_v_ret;
      goto __pyx_L0;
    }

    /* "ddtrace/internal/_encoding.pyx":725
 *                 ret = self._pack_string(v)
 *                 if ret != 0: return ret
 *         if dd_origin is not NULL:             # <<<<<<<<<<<<<<
 *             ret = msgpack_pack_uint32(&self.pk, <stdint.uint32_t> 1)
 *             if ret != 0: return ret
 */
  }

  /* "ddtrace/internal/_encoding.pyx":731
 *             if ret != 0: return ret
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._metrics))             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 *         if span._metrics:
 */
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_metrics); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_9 = PyObject_Length(__pyx_t_11); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->__pyx_base.pk), __pyx_t_9);

  /* "ddtrace/internal/_encoding.pyx":732
 * 
 *         ret = msgpack_pack_map(&self.pk, len(span._metrics))
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 *         if span._metrics:
 *             for k, v in span._metrics.items():
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":733
 *         ret = msgpack_pack_map(&self.pk, len(span._metrics))
 *         if ret != 0: return ret
 *         if span._metrics:             # <<<<<<<<<<<<<<
 *             for k, v in span._metrics.items():
 *                 ret = self._pack_string(k)
 */
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_metrics); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (__pyx_t_1) {

    /* "ddtrace/internal/_encoding.pyx":734
 *         if ret != 0: return ret
 *         if span._metrics:
 *             for k, v in span._metrics.items():             # <<<<<<<<<<<<<<
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_metrics); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_items); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_13))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_13);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_13, function);
      }
    }
    __pyx_t_11 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_13);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 734, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (likely(PyList_CheckExact(__pyx_t_11)) || PyTuple_CheckExact(__pyx_t_11)) {
      __pyx_t_13 = __pyx_t_11; __Pyx_INCREF(__pyx_t_13); __pyx_t_9 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_9 = -1; __pyx_t_13 = PyObject_GetIter(__pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 734, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __pyx_t_12 = Py_TYPE(__pyx_t_13)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 734, __pyx_L1_error)
    }
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_13))) {
          if (__pyx_t_9 >= PyList_GET_SIZE(__pyx_t_13)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_11 = PyList_GET_ITEM(__pyx_t_13, __pyx_t_9); __Pyx_INCREF(__pyx_t_11); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 734, __pyx_L1_error)
          #else
          __pyx_t_11 = PySequence_ITEM(__pyx_t_13, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          #endif
        } else {
          if (__pyx_t_9 >= PyTuple_GET_SIZE(__pyx_t_13)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_11 = PyTuple_GET_ITEM(__pyx_t_13, __pyx_t_9); __Pyx_INCREF(__pyx_t_11); __pyx_t_9++; if (unlikely(0 < 0)) __PYX_ERR(0, 734, __pyx_L1_error)
          #else
          __pyx_t_11 = PySequence_ITEM(__pyx_t_13, __pyx_t_9); __pyx_t_9++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          #endif
        }
      } else {
        __pyx_t_11 = __pyx_t_12(__pyx_t_13);
        if (unlikely(!__pyx_t_11)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 734, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_11);
      }
      if ((likely(PyTuple_CheckExact(__pyx_t_11))) || (PyList_CheckExact(__pyx_t_11))) {
        PyObject* sequence = __pyx_t_11;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 734, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_10);
        #else
        __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        #endif
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_14 = PyObject_GetIter(__pyx_t_11); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 734, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_15 = Py_TYPE(__pyx_t_14)->tp_iternext;
        index = 0; __pyx_t_2 = __pyx_t_15(__pyx_t_14); if (unlikely(!__pyx_t_2)) goto __pyx_L28_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_2);
        index = 1; __pyx_t_10 = __pyx_t_15(__pyx_t_14); if (unlikely(!__pyx_t_10)) goto __pyx_L28_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_10);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_14), 2) < 0) __PYX_ERR(0, 734, __pyx_L1_error)
        __pyx_t_15 = NULL;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        goto __pyx_L29_unpacking_done;
        __pyx_L28_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_15 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 734, __pyx_L1_error)
        __pyx_L29_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_2);
      __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_10);
      __pyx_t_10 = 0;

      /* "ddtrace/internal/_encoding.pyx":735
 *         if span._metrics:
 *             for k, v in span._metrics.items():
 *                 ret = self._pack_string(k)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 *                 ret = pack_number(&self.pk, v)
 */
      __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_v_k);

      /* "ddtrace/internal/_encoding.pyx":736
 *             for k, v in span._metrics.items():
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 *                 ret = pack_number(&self.pk, v)
 *                 if ret != 0: return ret
 */
      __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_1) {
        __pyx_r = __pyx_v_ret;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":737
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 *                 ret = pack_number(&self.pk, v)             # <<<<<<<<<<<<<<
 *                 if ret != 0: return ret
 * 
 */
      __pyx_t_16 = __pyx_f_7ddtrace_8internal_9_encoding_pack_number((&__pyx_v_self->__pyx_base.pk), __pyx_v_v); if (unlikely(__pyx_t_16 == ((int)-1) && PyErr_Occurred())) __PYX_ERR(0, 737, __pyx_L1_error)
      __pyx_v_ret = __pyx_t_16;

      /* "ddtrace/internal/_encoding.pyx":738
 *                 if ret != 0: return ret
 *                 ret = pack_number(&self.pk, v)
 *                 if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         ret = self._pack_string(span.span_type)
 */
      __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
      if (__pyx_t_1) {
        __pyx_r = __pyx_v_ret;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        goto __pyx_L0;
      }

      /* "ddtrace/internal/_encoding.pyx":734
 *         if ret != 0: return ret
 *         if span._metrics:
 *             for k, v in span._metrics.items():             # <<<<<<<<<<<<<<
 *                 ret = self._pack_string(k)
 *                 if ret != 0: return ret
 */
    }
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

    /* "ddtrace/internal/_encoding.pyx":733
 *         ret = msgpack_pack_map(&self.pk, len(span._metrics))
 *         if ret != 0: return ret
 *         if span._metrics:             # <<<<<<<<<<<<<<
 *             for k, v in span._metrics.items():
 *                 ret = self._pack_string(k)
 */
  }

  /* "ddtrace/internal/_encoding.pyx":740
 *                 if ret != 0: return ret
 * 
 *         ret = self._pack_string(span.span_type)             # <<<<<<<<<<<<<<
 *         if ret != 0: return ret
 * 
 */
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_span, __pyx_n_s_span_type); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 740, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_v_ret = __pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string(__pyx_v_self, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "ddtrace/internal/_encoding.pyx":741
 * 
 *         ret = self._pack_string(span.span_type)
 *         if ret != 0: return ret             # <<<<<<<<<<<<<<
 * 
 *         return 0
 */
  __pyx_t_1 = ((__pyx_v_ret != 0) != 0);
  if (__pyx_t_1) {
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
  }

  /* "ddtrace/internal/_encoding.pyx":743
 *         if ret != 0: return ret
 * 
 *         return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":680
 *         return <void *> PyLong_AsLong(self._st._index(dd_origin))
 * 
 *     cdef int pack_span(self, object span, void *dd_origin) except? -1:             # <<<<<<<<<<<<<<
 *         cdef int ret
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.pack_span", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_6__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_6__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_8__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_8__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.MsgpackEncoderV05.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":766
 *     cdef const char *unicode_errors
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(PyTuple_GET_SIZE(__pyx_args) > 0)) {
    __Pyx_RaiseArgtupleInvalid("__cinit__", 1, 0, 0, PyTuple_GET_SIZE(__pyx_args)); return -1;}
  if (unlikely(__pyx_kwds) && unlikely(PyDict_Size(__pyx_kwds) > 0) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 0))) return -1;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer___cinit__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_6Packer___cinit__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self) {
  int __pyx_v_buf_size;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "ddtrace/internal/_encoding.pyx":767
 * 
 *     def __cinit__(self):
 *         cdef int buf_size = 1024*1024             # <<<<<<<<<<<<<<
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:
 */
  __pyx_v_buf_size = 0x100000;

  /* "ddtrace/internal/_encoding.pyx":768
 *     def __cinit__(self):
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)             # <<<<<<<<<<<<<<
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")
 */
  __pyx_v_self->pk.buf = ((char *)PyMem_Malloc(__pyx_v_buf_size));

  /* "ddtrace/internal/_encoding.pyx":769
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.pk.buf_size = buf_size
 */
  __pyx_t_1 = ((__pyx_v_self->pk.buf == NULL) != 0);
  if (unlikely(__pyx_t_1)) {

    /* "ddtrace/internal/_encoding.pyx":770
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")             # <<<<<<<<<<<<<<
 *         self.pk.buf_size = buf_size
 *         self.pk.length = 0
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 770, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 770, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":769
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 *         if self.pk.buf == NULL:             # <<<<<<<<<<<<<<
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.pk.buf_size = buf_size
 */
  }

  /* "ddtrace/internal/_encoding.pyx":771
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.pk.buf_size = buf_size             # <<<<<<<<<<<<<<
 *         self.pk.length = 0
 * 
 */
  __pyx_v_self->pk.buf_size = __pyx_v_buf_size;

  /* "ddtrace/internal/_encoding.pyx":772
 *             raise MemoryError("Unable to allocate internal buffer.")
 *         self.pk.buf_size = buf_size
 *         self.pk.length = 0             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, default=None):
 */
  __pyx_v_self->pk.length = 0;

  /* "ddtrace/internal/_encoding.pyx":766
 *     cdef const char *unicode_errors
 * 
 *     def __cinit__(self):             # <<<<<<<<<<<<<<
 *         cdef int buf_size = 1024*1024
 *         self.pk.buf = <char*> PyMem_Malloc(buf_size)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":774
 *         self.pk.length = 0
 * 
 *     def __init__(self, default=None):             # <<<<<<<<<<<<<<
 *         if default is not None:
 *             if not PyCallable_Check(default):
 */

/* Python wrapper */
static int __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_default = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_default,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_default);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 774, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_default = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 774, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_2__init__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self), __pyx_v_default);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_2__init__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_default) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "ddtrace/internal/_encoding.pyx":775
 * 
 *     def __init__(self, default=None):
 *         if default is not None:             # <<<<<<<<<<<<<<
 *             if not PyCallable_Check(default):
 *                 raise TypeError("default must be a callable.")
 */
  __pyx_t_1 = (__pyx_v_default != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "ddtrace/internal/_encoding.pyx":776
 *     def __init__(self, default=None):
 *         if default is not None:
 *             if not PyCallable_Check(default):             # <<<<<<<<<<<<<<
 *                 raise TypeError("default must be a callable.")
 *         self._default = default
 */
    __pyx_t_2 = ((!(PyCallable_Check(__pyx_v_default) != 0)) != 0);
    if (unlikely(__pyx_t_2)) {

      /* "ddtrace/internal/_encoding.pyx":777
 *         if default is not None:
 *             if not PyCallable_Check(default):
 *                 raise TypeError("default must be a callable.")             # <<<<<<<<<<<<<<
 *         self._default = default
 * 
 */
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 777, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_Raise(__pyx_t_3, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __PYX_ERR(0, 777, __pyx_L1_error)

      /* "ddtrace/internal/_encoding.pyx":776
 *     def __init__(self, default=None):
 *         if default is not None:
 *             if not PyCallable_Check(default):             # <<<<<<<<<<<<<<
 *                 raise TypeError("default must be a callable.")
 *         self._default = default
 */
    }

    /* "ddtrace/internal/_encoding.pyx":775
 * 
 *     def __init__(self, default=None):
 *         if default is not None:             # <<<<<<<<<<<<<<
 *             if not PyCallable_Check(default):
 *                 raise TypeError("default must be a callable.")
 */
  }

  /* "ddtrace/internal/_encoding.pyx":778
 *             if not PyCallable_Check(default):
 *                 raise TypeError("default must be a callable.")
 *         self._default = default             # <<<<<<<<<<<<<<
 * 
 *         if PY_MAJOR_VERSION < 3:
 */
  __Pyx_INCREF(__pyx_v_default);
  __Pyx_GIVEREF(__pyx_v_default);
  __Pyx_GOTREF(__pyx_v_self->_default);
  __Pyx_DECREF(__pyx_v_self->_default);
  __pyx_v_self->_default = __pyx_v_default;

  /* "ddtrace/internal/_encoding.pyx":783
 *             self.encoding = "utf-8"
 *         else:
 *             self.encoding = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_v_self->encoding = NULL;

  /* "ddtrace/internal/_encoding.pyx":774
 *         self.pk.length = 0
 * 
 *     def __init__(self, default=None):             # <<<<<<<<<<<<<<
 *         if default is not None:
 *             if not PyCallable_Check(default):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":785
 *             self.encoding = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

/* Python wrapper */
static void __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_4__dealloc__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_4__dealloc__(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "ddtrace/internal/_encoding.pyx":786
 * 
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)             # <<<<<<<<<<<<<<
 *         self.pk.buf = NULL
 * 
 */
  PyMem_Free(__pyx_v_self->pk.buf);

  /* "ddtrace/internal/_encoding.pyx":787
 *     def __dealloc__(self):
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     cdef int _pack(self, object o) except -1:
 */
  __pyx_v_self->pk.buf = NULL;

  /* "ddtrace/internal/_encoding.pyx":785
 *             self.encoding = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         PyMem_Free(self.pk.buf)
 *         self.pk.buf = NULL
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "ddtrace/internal/_encoding.pyx":789
 *         self.pk.buf = NULL
 * 
 *     cdef int _pack(self, object o) except -1:             # <<<<<<<<<<<<<<
 *         cdef long long llval
 *         cdef unsigned long long ullval
 */

static int __pyx_f_7ddtrace_8internal_9_encoding_6Packer__pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_o) {
  PY_LONG_LONG __pyx_v_llval;
  unsigned PY_LONG_LONG __pyx_v_ullval;
  long __pyx_v_longval;
  double __pyx_v_dval;
  char *__pyx_v_rawval;
  int __pyx_v_ret;
  PyObject *__pyx_v_d = 0;
  Py_ssize_t __pyx_v_L;
  int __pyx_v_default_used;
  CYTHON_UNUSED PyObject *__pyx_v_oe = NULL;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  unsigned PY_LONG_LONG __pyx_t_7;
  PY_LONG_LONG __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  long __pyx_t_16;
  double __pyx_t_17;
  Py_ssize_t __pyx_t_18;
  PyObject *__pyx_t_19;
  char *__pyx_t_20;
  PyObject *(*__pyx_t_21)(PyObject *);
  PyObject *(*__pyx_t_22)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_pack", 0);
  __Pyx_INCREF(__pyx_v_o);

  /* "ddtrace/internal/_encoding.pyx":799
 *         cdef dict d
 *         cdef Py_ssize_t L
 *         cdef int default_used = 0             # <<<<<<<<<<<<<<
 *         cdef Py_buffer view
 *         cdef long i
 */
  __pyx_v_default_used = 0;

  /* "ddtrace/internal/_encoding.pyx":803
 *         cdef long i
 * 
 *         while True:             # <<<<<<<<<<<<<<
 *             if o is None:
 *                 ret = msgpack_pack_nil(&self.pk)
 */
  while (1) {

    /* "ddtrace/internal/_encoding.pyx":804
 * 
 *         while True:
 *             if o is None:             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_nil(&self.pk)
 *             elif PyLong_CheckExact(o):
 */
    __pyx_t_1 = (__pyx_v_o == Py_None);
    __pyx_t_2 = (__pyx_t_1 != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":805
 *         while True:
 *             if o is None:
 *                 ret = msgpack_pack_nil(&self.pk)             # <<<<<<<<<<<<<<
 *             elif PyLong_CheckExact(o):
 *                 # PyInt_Check(long) is True for Python 3.
 */
      __pyx_v_ret = msgpack_pack_nil((&__pyx_v_self->pk));

      /* "ddtrace/internal/_encoding.pyx":804
 * 
 *         while True:
 *             if o is None:             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_nil(&self.pk)
 *             elif PyLong_CheckExact(o):
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":806
 *             if o is None:
 *                 ret = msgpack_pack_nil(&self.pk)
 *             elif PyLong_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 # PyInt_Check(long) is True for Python 3.
 *                 # So we should test long before int.
 */
    __pyx_t_2 = (PyLong_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":809
 *                 # PyInt_Check(long) is True for Python 3.
 *                 # So we should test long before int.
 *                 try:             # <<<<<<<<<<<<<<
 *                     if o > 0:
 *                         ullval = o
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_5);
        /*try:*/ {

          /* "ddtrace/internal/_encoding.pyx":810
 *                 # So we should test long before int.
 *                 try:
 *                     if o > 0:             # <<<<<<<<<<<<<<
 *                         ullval = o
 *                         ret = msgpack_pack_unsigned_long_long(&self.pk, ullval)
 */
          __pyx_t_6 = PyObject_RichCompare(__pyx_v_o, __pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 810, __pyx_L6_error)
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 810, __pyx_L6_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (__pyx_t_2) {

            /* "ddtrace/internal/_encoding.pyx":811
 *                 try:
 *                     if o > 0:
 *                         ullval = o             # <<<<<<<<<<<<<<
 *                         ret = msgpack_pack_unsigned_long_long(&self.pk, ullval)
 *                     else:
 */
            __pyx_t_7 = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(__pyx_v_o); if (unlikely((__pyx_t_7 == (unsigned PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 811, __pyx_L6_error)
            __pyx_v_ullval = __pyx_t_7;

            /* "ddtrace/internal/_encoding.pyx":812
 *                     if o > 0:
 *                         ullval = o
 *                         ret = msgpack_pack_unsigned_long_long(&self.pk, ullval)             # <<<<<<<<<<<<<<
 *                     else:
 *                         llval = o
 */
            __pyx_v_ret = msgpack_pack_unsigned_long_long((&__pyx_v_self->pk), __pyx_v_ullval);

            /* "ddtrace/internal/_encoding.pyx":810
 *                 # So we should test long before int.
 *                 try:
 *                     if o > 0:             # <<<<<<<<<<<<<<
 *                         ullval = o
 *                         ret = msgpack_pack_unsigned_long_long(&self.pk, ullval)
 */
            goto __pyx_L14;
          }

          /* "ddtrace/internal/_encoding.pyx":814
 *                         ret = msgpack_pack_unsigned_long_long(&self.pk, ullval)
 *                     else:
 *                         llval = o             # <<<<<<<<<<<<<<
 *                         ret = msgpack_pack_long_long(&self.pk, llval)
 *                 except OverflowError as oe:
 */
          /*else*/ {
            __pyx_t_8 = __Pyx_PyInt_As_PY_LONG_LONG(__pyx_v_o); if (unlikely((__pyx_t_8 == (PY_LONG_LONG)-1) && PyErr_Occurred())) __PYX_ERR(0, 814, __pyx_L6_error)
            __pyx_v_llval = __pyx_t_8;

            /* "ddtrace/internal/_encoding.pyx":815
 *                     else:
 *                         llval = o
 *                         ret = msgpack_pack_long_long(&self.pk, llval)             # <<<<<<<<<<<<<<
 *                 except OverflowError as oe:
 *                     if not default_used and self._default is not None:
 */
            __pyx_v_ret = msgpack_pack_long_long((&__pyx_v_self->pk), __pyx_v_llval);
          }
          __pyx_L14:;

          /* "ddtrace/internal/_encoding.pyx":809
 *                 # PyInt_Check(long) is True for Python 3.
 *                 # So we should test long before int.
 *                 try:             # <<<<<<<<<<<<<<
 *                     if o > 0:
 *                         ullval = o
 */
        }
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        goto __pyx_L13_try_end;
        __pyx_L6_error:;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "ddtrace/internal/_encoding.pyx":816
 *                         llval = o
 *                         ret = msgpack_pack_long_long(&self.pk, llval)
 *                 except OverflowError as oe:             # <<<<<<<<<<<<<<
 *                     if not default_used and self._default is not None:
 *                         o = self._default(o)
 */
        __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_OverflowError);
        if (__pyx_t_9) {
          __Pyx_AddTraceback("ddtrace.internal._encoding.Packer._pack", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_10, &__pyx_t_11) < 0) __PYX_ERR(0, 816, __pyx_L8_except_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_10);
          __Pyx_XDECREF_SET(__pyx_v_oe, __pyx_t_10);

          /* "ddtrace/internal/_encoding.pyx":817
 *                         ret = msgpack_pack_long_long(&self.pk, llval)
 *                 except OverflowError as oe:
 *                     if not default_used and self._default is not None:             # <<<<<<<<<<<<<<
 *                         o = self._default(o)
 *                         default_used = True
 */
          __pyx_t_1 = ((!(__pyx_v_default_used != 0)) != 0);
          if (__pyx_t_1) {
          } else {
            __pyx_t_2 = __pyx_t_1;
            goto __pyx_L18_bool_binop_done;
          }
          __pyx_t_1 = (__pyx_v_self->_default != Py_None);
          __pyx_t_12 = (__pyx_t_1 != 0);
          __pyx_t_2 = __pyx_t_12;
          __pyx_L18_bool_binop_done:;
          if (likely(__pyx_t_2)) {

            /* "ddtrace/internal/_encoding.pyx":818
 *                 except OverflowError as oe:
 *                     if not default_used and self._default is not None:
 *                         o = self._default(o)             # <<<<<<<<<<<<<<
 *                         default_used = True
 *                         continue
 */
            __Pyx_INCREF(__pyx_v_self->_default);
            __pyx_t_14 = __pyx_v_self->_default; __pyx_t_15 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_15 = PyMethod_GET_SELF(__pyx_t_14);
              if (likely(__pyx_t_15)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
                __Pyx_INCREF(__pyx_t_15);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_14, function);
              }
            }
            __pyx_t_13 = (__pyx_t_15) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_15, __pyx_v_o) : __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_v_o);
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 818, __pyx_L8_except_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF_SET(__pyx_v_o, __pyx_t_13);
            __pyx_t_13 = 0;

            /* "ddtrace/internal/_encoding.pyx":819
 *                     if not default_used and self._default is not None:
 *                         o = self._default(o)
 *                         default_used = True             # <<<<<<<<<<<<<<
 *                         continue
 *                     else:
 */
            __pyx_v_default_used = 1;

            /* "ddtrace/internal/_encoding.pyx":820
 *                         o = self._default(o)
 *                         default_used = True
 *                         continue             # <<<<<<<<<<<<<<
 *                     else:
 *                         raise OverflowError("Integer value out of range")
 */
            goto __pyx_L16_except_continue;

            /* "ddtrace/internal/_encoding.pyx":817
 *                         ret = msgpack_pack_long_long(&self.pk, llval)
 *                 except OverflowError as oe:
 *                     if not default_used and self._default is not None:             # <<<<<<<<<<<<<<
 *                         o = self._default(o)
 *                         default_used = True
 */
          }

          /* "ddtrace/internal/_encoding.pyx":822
 *                         continue
 *                     else:
 *                         raise OverflowError("Integer value out of range")             # <<<<<<<<<<<<<<
 *             elif PyInt_CheckExact(o):
 *                 longval = o
 */
          /*else*/ {
            __pyx_t_13 = __Pyx_PyObject_Call(__pyx_builtin_OverflowError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 822, __pyx_L8_except_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_Raise(__pyx_t_13, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            __PYX_ERR(0, 822, __pyx_L8_except_error)
          }
          __pyx_L16_except_continue:;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          goto __pyx_L12_try_continue;
        }
        goto __pyx_L8_except_error;
        __pyx_L8_except_error:;

        /* "ddtrace/internal/_encoding.pyx":809
 *                 # PyInt_Check(long) is True for Python 3.
 *                 # So we should test long before int.
 *                 try:             # <<<<<<<<<<<<<<
 *                     if o > 0:
 *                         ullval = o
 */
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        goto __pyx_L1_error;
        __pyx_L12_try_continue:;
        __Pyx_XGIVEREF(__pyx_t_3);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_XGIVEREF(__pyx_t_5);
        __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
        goto __pyx_L3_continue;
        __pyx_L13_try_end:;
      }

      /* "ddtrace/internal/_encoding.pyx":806
 *             if o is None:
 *                 ret = msgpack_pack_nil(&self.pk)
 *             elif PyLong_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 # PyInt_Check(long) is True for Python 3.
 *                 # So we should test long before int.
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":823
 *                     else:
 *                         raise OverflowError("Integer value out of range")
 *             elif PyInt_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 longval = o
 *                 ret = msgpack_pack_long(&self.pk, longval)
 */
    __pyx_t_2 = (PyInt_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":824
 *                         raise OverflowError("Integer value out of range")
 *             elif PyInt_CheckExact(o):
 *                 longval = o             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_long(&self.pk, longval)
 *             elif PyFloat_CheckExact(o):
 */
      __pyx_t_16 = __Pyx_PyInt_As_long(__pyx_v_o); if (unlikely((__pyx_t_16 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 824, __pyx_L1_error)
      __pyx_v_longval = __pyx_t_16;

      /* "ddtrace/internal/_encoding.pyx":825
 *             elif PyInt_CheckExact(o):
 *                 longval = o
 *                 ret = msgpack_pack_long(&self.pk, longval)             # <<<<<<<<<<<<<<
 *             elif PyFloat_CheckExact(o):
 *                 dval = o
 */
      __pyx_v_ret = msgpack_pack_long((&__pyx_v_self->pk), __pyx_v_longval);

      /* "ddtrace/internal/_encoding.pyx":823
 *                     else:
 *                         raise OverflowError("Integer value out of range")
 *             elif PyInt_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 longval = o
 *                 ret = msgpack_pack_long(&self.pk, longval)
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":826
 *                 longval = o
 *                 ret = msgpack_pack_long(&self.pk, longval)
 *             elif PyFloat_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 dval = o
 *                 ret = msgpack_pack_double(&self.pk, dval)
 */
    __pyx_t_2 = (PyFloat_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":827
 *                 ret = msgpack_pack_long(&self.pk, longval)
 *             elif PyFloat_CheckExact(o):
 *                 dval = o             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_double(&self.pk, dval)
 *             elif PyBytesLike_CheckExact(o):
 */
      __pyx_t_17 = __pyx_PyFloat_AsDouble(__pyx_v_o); if (unlikely((__pyx_t_17 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 827, __pyx_L1_error)
      __pyx_v_dval = __pyx_t_17;

      /* "ddtrace/internal/_encoding.pyx":828
 *             elif PyFloat_CheckExact(o):
 *                 dval = o
 *                 ret = msgpack_pack_double(&self.pk, dval)             # <<<<<<<<<<<<<<
 *             elif PyBytesLike_CheckExact(o):
 *                 L = len(o)
 */
      __pyx_v_ret = msgpack_pack_double((&__pyx_v_self->pk), __pyx_v_dval);

      /* "ddtrace/internal/_encoding.pyx":826
 *                 longval = o
 *                 ret = msgpack_pack_long(&self.pk, longval)
 *             elif PyFloat_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 dval = o
 *                 ret = msgpack_pack_double(&self.pk, dval)
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":829
 *                 dval = o
 *                 ret = msgpack_pack_double(&self.pk, dval)
 *             elif PyBytesLike_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 L = len(o)
 *                 if L > ITEM_LIMIT:
 */
    __pyx_t_2 = (__pyx_f_7ddtrace_8internal_9_encoding_PyBytesLike_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":830
 *                 ret = msgpack_pack_double(&self.pk, dval)
 *             elif PyBytesLike_CheckExact(o):
 *                 L = len(o)             # <<<<<<<<<<<<<<
 *                 if L > ITEM_LIMIT:
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)
 */
      __pyx_t_18 = PyObject_Length(__pyx_v_o); if (unlikely(__pyx_t_18 == ((Py_ssize_t)-1))) __PYX_ERR(0, 830, __pyx_L1_error)
      __pyx_v_L = __pyx_t_18;

      /* "ddtrace/internal/_encoding.pyx":831
 *             elif PyBytesLike_CheckExact(o):
 *                 L = len(o)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)
 *                 rawval = o
 */
      __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":832
 *                 L = len(o)
 *                 if L > ITEM_LIMIT:
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)             # <<<<<<<<<<<<<<
 *                 rawval = o
 *                 ret = msgpack_pack_bin(&self.pk, L)
 */
        __pyx_t_19 = PyErr_Format(__pyx_builtin_ValueError, ((char *)"%.200s object is too large"), Py_TYPE(__pyx_v_o)->tp_name); if (unlikely(__pyx_t_19 == ((PyObject *)NULL))) __PYX_ERR(0, 832, __pyx_L1_error)

        /* "ddtrace/internal/_encoding.pyx":831
 *             elif PyBytesLike_CheckExact(o):
 *                 L = len(o)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)
 *                 rawval = o
 */
      }

      /* "ddtrace/internal/_encoding.pyx":833
 *                 if L > ITEM_LIMIT:
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)
 *                 rawval = o             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_bin(&self.pk, L)
 *                 if ret == 0:
 */
      __pyx_t_20 = __Pyx_PyObject_AsWritableString(__pyx_v_o); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 833, __pyx_L1_error)
      __pyx_v_rawval = __pyx_t_20;

      /* "ddtrace/internal/_encoding.pyx":834
 *                     PyErr_Format(ValueError, b"%.200s object is too large", Py_TYPE(o).tp_name)
 *                 rawval = o
 *                 ret = msgpack_pack_bin(&self.pk, L)             # <<<<<<<<<<<<<<
 *                 if ret == 0:
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 */
      __pyx_v_ret = msgpack_pack_bin((&__pyx_v_self->pk), __pyx_v_L);

      /* "ddtrace/internal/_encoding.pyx":835
 *                 rawval = o
 *                 ret = msgpack_pack_bin(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):
 */
      __pyx_t_2 = ((__pyx_v_ret == 0) != 0);
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":836
 *                 ret = msgpack_pack_bin(&self.pk, L)
 *                 if ret == 0:
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)             # <<<<<<<<<<<<<<
 *             elif PyUnicode_CheckExact(o):
 *                 if self.encoding == NULL:
 */
        __pyx_v_ret = msgpack_pack_raw_body((&__pyx_v_self->pk), __pyx_v_rawval, __pyx_v_L);

        /* "ddtrace/internal/_encoding.pyx":835
 *                 rawval = o
 *                 ret = msgpack_pack_bin(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):
 */
      }

      /* "ddtrace/internal/_encoding.pyx":829
 *                 dval = o
 *                 ret = msgpack_pack_double(&self.pk, dval)
 *             elif PyBytesLike_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 L = len(o)
 *                 if L > ITEM_LIMIT:
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":837
 *                 if ret == 0:
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 if self.encoding == NULL:
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 */
    __pyx_t_2 = (PyUnicode_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":838
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):
 *                 if self.encoding == NULL:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 *                     if ret == -2:
 */
      __pyx_t_2 = ((__pyx_v_self->encoding == NULL) != 0);
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":839
 *             elif PyUnicode_CheckExact(o):
 *                 if self.encoding == NULL:
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)             # <<<<<<<<<<<<<<
 *                     if ret == -2:
 *                         raise ValueError("unicode string is too large")
 */
        __pyx_v_ret = msgpack_pack_unicode((&__pyx_v_self->pk), __pyx_v_o, __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT);

        /* "ddtrace/internal/_encoding.pyx":840
 *                 if self.encoding == NULL:
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 *                     if ret == -2:             # <<<<<<<<<<<<<<
 *                         raise ValueError("unicode string is too large")
 *                 else:
 */
        __pyx_t_2 = ((__pyx_v_ret == -2L) != 0);
        if (unlikely(__pyx_t_2)) {

          /* "ddtrace/internal/_encoding.pyx":841
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 *                     if ret == -2:
 *                         raise ValueError("unicode string is too large")             # <<<<<<<<<<<<<<
 *                 else:
 *                     o = PyUnicode_AsEncodedString(o, self.encoding, self.unicode_errors)
 */
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 841, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_Raise(__pyx_t_11, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __PYX_ERR(0, 841, __pyx_L1_error)

          /* "ddtrace/internal/_encoding.pyx":840
 *                 if self.encoding == NULL:
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 *                     if ret == -2:             # <<<<<<<<<<<<<<
 *                         raise ValueError("unicode string is too large")
 *                 else:
 */
        }

        /* "ddtrace/internal/_encoding.pyx":838
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):
 *                 if self.encoding == NULL:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 *                     if ret == -2:
 */
        goto __pyx_L22;
      }

      /* "ddtrace/internal/_encoding.pyx":843
 *                         raise ValueError("unicode string is too large")
 *                 else:
 *                     o = PyUnicode_AsEncodedString(o, self.encoding, self.unicode_errors)             # <<<<<<<<<<<<<<
 *                     L = len(o)
 *                     if L > ITEM_LIMIT:
 */
      /*else*/ {
        __pyx_t_11 = PyUnicode_AsEncodedString(__pyx_v_o, __pyx_v_self->encoding, __pyx_v_self->unicode_errors); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 843, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF_SET(__pyx_v_o, __pyx_t_11);
        __pyx_t_11 = 0;

        /* "ddtrace/internal/_encoding.pyx":844
 *                 else:
 *                     o = PyUnicode_AsEncodedString(o, self.encoding, self.unicode_errors)
 *                     L = len(o)             # <<<<<<<<<<<<<<
 *                     if L > ITEM_LIMIT:
 *                         raise ValueError("unicode string is too large")
 */
        __pyx_t_18 = PyObject_Length(__pyx_v_o); if (unlikely(__pyx_t_18 == ((Py_ssize_t)-1))) __PYX_ERR(0, 844, __pyx_L1_error)
        __pyx_v_L = __pyx_t_18;

        /* "ddtrace/internal/_encoding.pyx":845
 *                     o = PyUnicode_AsEncodedString(o, self.encoding, self.unicode_errors)
 *                     L = len(o)
 *                     if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                         raise ValueError("unicode string is too large")
 *                     ret = msgpack_pack_raw(&self.pk, L)
 */
        __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
        if (unlikely(__pyx_t_2)) {

          /* "ddtrace/internal/_encoding.pyx":846
 *                     L = len(o)
 *                     if L > ITEM_LIMIT:
 *                         raise ValueError("unicode string is too large")             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_raw(&self.pk, L)
 *                     if ret == 0:
 */
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 846, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_Raise(__pyx_t_11, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          __PYX_ERR(0, 846, __pyx_L1_error)

          /* "ddtrace/internal/_encoding.pyx":845
 *                     o = PyUnicode_AsEncodedString(o, self.encoding, self.unicode_errors)
 *                     L = len(o)
 *                     if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                         raise ValueError("unicode string is too large")
 *                     ret = msgpack_pack_raw(&self.pk, L)
 */
        }

        /* "ddtrace/internal/_encoding.pyx":847
 *                     if L > ITEM_LIMIT:
 *                         raise ValueError("unicode string is too large")
 *                     ret = msgpack_pack_raw(&self.pk, L)             # <<<<<<<<<<<<<<
 *                     if ret == 0:
 *                         rawval = o
 */
        __pyx_v_ret = msgpack_pack_raw((&__pyx_v_self->pk), __pyx_v_L);

        /* "ddtrace/internal/_encoding.pyx":848
 *                         raise ValueError("unicode string is too large")
 *                     ret = msgpack_pack_raw(&self.pk, L)
 *                     if ret == 0:             # <<<<<<<<<<<<<<
 *                         rawval = o
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 */
        __pyx_t_2 = ((__pyx_v_ret == 0) != 0);
        if (__pyx_t_2) {

          /* "ddtrace/internal/_encoding.pyx":849
 *                     ret = msgpack_pack_raw(&self.pk, L)
 *                     if ret == 0:
 *                         rawval = o             # <<<<<<<<<<<<<<
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyDict_CheckExact(o):
 */
          __pyx_t_20 = __Pyx_PyObject_AsWritableString(__pyx_v_o); if (unlikely((!__pyx_t_20) && PyErr_Occurred())) __PYX_ERR(0, 849, __pyx_L1_error)
          __pyx_v_rawval = __pyx_t_20;

          /* "ddtrace/internal/_encoding.pyx":850
 *                     if ret == 0:
 *                         rawval = o
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)             # <<<<<<<<<<<<<<
 *             elif PyDict_CheckExact(o):
 *                 d = <dict>o
 */
          __pyx_v_ret = msgpack_pack_raw_body((&__pyx_v_self->pk), __pyx_v_rawval, __pyx_v_L);

          /* "ddtrace/internal/_encoding.pyx":848
 *                         raise ValueError("unicode string is too large")
 *                     ret = msgpack_pack_raw(&self.pk, L)
 *                     if ret == 0:             # <<<<<<<<<<<<<<
 *                         rawval = o
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 */
        }
      }
      __pyx_L22:;

      /* "ddtrace/internal/_encoding.pyx":837
 *                 if ret == 0:
 *                     ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyUnicode_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 if self.encoding == NULL:
 *                     ret = msgpack_pack_unicode(&self.pk, o, ITEM_LIMIT)
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":851
 *                         rawval = o
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyDict_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 d = <dict>o
 *                 L = len(d)
 */
    __pyx_t_2 = (PyDict_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":852
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyDict_CheckExact(o):
 *                 d = <dict>o             # <<<<<<<<<<<<<<
 *                 L = len(d)
 *                 if L > ITEM_LIMIT:
 */
      __pyx_t_11 = __pyx_v_o;
      __Pyx_INCREF(__pyx_t_11);
      __pyx_v_d = ((PyObject*)__pyx_t_11);
      __pyx_t_11 = 0;

      /* "ddtrace/internal/_encoding.pyx":853
 *             elif PyDict_CheckExact(o):
 *                 d = <dict>o
 *                 L = len(d)             # <<<<<<<<<<<<<<
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("dict is too large")
 */
      if (unlikely(__pyx_v_d == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 853, __pyx_L1_error)
      }
      __pyx_t_18 = PyDict_Size(__pyx_v_d); if (unlikely(__pyx_t_18 == ((Py_ssize_t)-1))) __PYX_ERR(0, 853, __pyx_L1_error)
      __pyx_v_L = __pyx_t_18;

      /* "ddtrace/internal/_encoding.pyx":854
 *                 d = <dict>o
 *                 L = len(d)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     raise ValueError("dict is too large")
 *                 ret = msgpack_pack_map(&self.pk, L)
 */
      __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
      if (unlikely(__pyx_t_2)) {

        /* "ddtrace/internal/_encoding.pyx":855
 *                 L = len(d)
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("dict is too large")             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_map(&self.pk, L)
 *                 if ret == 0:
 */
        __pyx_t_11 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 855, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_Raise(__pyx_t_11, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __PYX_ERR(0, 855, __pyx_L1_error)

        /* "ddtrace/internal/_encoding.pyx":854
 *                 d = <dict>o
 *                 L = len(d)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     raise ValueError("dict is too large")
 *                 ret = msgpack_pack_map(&self.pk, L)
 */
      }

      /* "ddtrace/internal/_encoding.pyx":856
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("dict is too large")
 *                 ret = msgpack_pack_map(&self.pk, L)             # <<<<<<<<<<<<<<
 *                 if ret == 0:
 *                     for k, v in d.items():
 */
      __pyx_v_ret = msgpack_pack_map((&__pyx_v_self->pk), __pyx_v_L);

      /* "ddtrace/internal/_encoding.pyx":857
 *                     raise ValueError("dict is too large")
 *                 ret = msgpack_pack_map(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     for k, v in d.items():
 *                        ret = self._pack(k)
 */
      __pyx_t_2 = ((__pyx_v_ret == 0) != 0);
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":858
 *                 ret = msgpack_pack_map(&self.pk, L)
 *                 if ret == 0:
 *                     for k, v in d.items():             # <<<<<<<<<<<<<<
 *                        ret = self._pack(k)
 *                        if ret != 0: break
 */
        if (unlikely(__pyx_v_d == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
          __PYX_ERR(0, 858, __pyx_L1_error)
        }
        __pyx_t_11 = __Pyx_PyDict_Items(__pyx_v_d); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 858, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (likely(PyList_CheckExact(__pyx_t_11)) || PyTuple_CheckExact(__pyx_t_11)) {
          __pyx_t_10 = __pyx_t_11; __Pyx_INCREF(__pyx_t_10); __pyx_t_18 = 0;
          __pyx_t_21 = NULL;
        } else {
          __pyx_t_18 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 858, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_21 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 858, __pyx_L1_error)
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        for (;;) {
          if (likely(!__pyx_t_21)) {
            if (likely(PyList_CheckExact(__pyx_t_10))) {
              if (__pyx_t_18 >= PyList_GET_SIZE(__pyx_t_10)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_11 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_18); __Pyx_INCREF(__pyx_t_11); __pyx_t_18++; if (unlikely(0 < 0)) __PYX_ERR(0, 858, __pyx_L1_error)
              #else
              __pyx_t_11 = PySequence_ITEM(__pyx_t_10, __pyx_t_18); __pyx_t_18++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 858, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              #endif
            } else {
              if (__pyx_t_18 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_11 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_18); __Pyx_INCREF(__pyx_t_11); __pyx_t_18++; if (unlikely(0 < 0)) __PYX_ERR(0, 858, __pyx_L1_error)
              #else
              __pyx_t_11 = PySequence_ITEM(__pyx_t_10, __pyx_t_18); __pyx_t_18++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 858, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              #endif
            }
          } else {
            __pyx_t_11 = __pyx_t_21(__pyx_t_10);
            if (unlikely(!__pyx_t_11)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 858, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_11);
          }
          if ((likely(PyTuple_CheckExact(__pyx_t_11))) || (PyList_CheckExact(__pyx_t_11))) {
            PyObject* sequence = __pyx_t_11;
            Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
            if (unlikely(size != 2)) {
              if (size > 2) __Pyx_RaiseTooManyValuesError(2);
              else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
              __PYX_ERR(0, 858, __pyx_L1_error)
            }
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            if (likely(PyTuple_CheckExact(sequence))) {
              __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0); 
              __pyx_t_13 = PyTuple_GET_ITEM(sequence, 1); 
            } else {
              __pyx_t_6 = PyList_GET_ITEM(sequence, 0); 
              __pyx_t_13 = PyList_GET_ITEM(sequence, 1); 
            }
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(__pyx_t_13);
            #else
            __pyx_t_6 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 858, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_13 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 858, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_13);
            #endif
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else {
            Py_ssize_t index = -1;
            __pyx_t_14 = PyObject_GetIter(__pyx_t_11); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 858, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __pyx_t_22 = Py_TYPE(__pyx_t_14)->tp_iternext;
            index = 0; __pyx_t_6 = __pyx_t_22(__pyx_t_14); if (unlikely(!__pyx_t_6)) goto __pyx_L30_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_6);
            index = 1; __pyx_t_13 = __pyx_t_22(__pyx_t_14); if (unlikely(!__pyx_t_13)) goto __pyx_L30_unpacking_failed;
            __Pyx_GOTREF(__pyx_t_13);
            if (__Pyx_IternextUnpackEndCheck(__pyx_t_22(__pyx_t_14), 2) < 0) __PYX_ERR(0, 858, __pyx_L1_error)
            __pyx_t_22 = NULL;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            goto __pyx_L31_unpacking_done;
            __pyx_L30_unpacking_failed:;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            __pyx_t_22 = NULL;
            if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
            __PYX_ERR(0, 858, __pyx_L1_error)
            __pyx_L31_unpacking_done:;
          }
          __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_6);
          __pyx_t_6 = 0;
          __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_13);
          __pyx_t_13 = 0;

          /* "ddtrace/internal/_encoding.pyx":859
 *                 if ret == 0:
 *                     for k, v in d.items():
 *                        ret = self._pack(k)             # <<<<<<<<<<<<<<
 *                        if ret != 0: break
 *                        ret = self._pack(v)
 */
          __pyx_t_9 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self->__pyx_vtab)->_pack(__pyx_v_self, __pyx_v_k); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 859, __pyx_L1_error)
          __pyx_v_ret = __pyx_t_9;

          /* "ddtrace/internal/_encoding.pyx":860
 *                     for k, v in d.items():
 *                        ret = self._pack(k)
 *                        if ret != 0: break             # <<<<<<<<<<<<<<
 *                        ret = self._pack(v)
 *                        if ret != 0: break
 */
          __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
          if (__pyx_t_2) {
            goto __pyx_L29_break;
          }

          /* "ddtrace/internal/_encoding.pyx":861
 *                        ret = self._pack(k)
 *                        if ret != 0: break
 *                        ret = self._pack(v)             # <<<<<<<<<<<<<<
 *                        if ret != 0: break
 *             elif PyList_CheckExact(o):
 */
          __pyx_t_9 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self->__pyx_vtab)->_pack(__pyx_v_self, __pyx_v_v); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 861, __pyx_L1_error)
          __pyx_v_ret = __pyx_t_9;

          /* "ddtrace/internal/_encoding.pyx":862
 *                        if ret != 0: break
 *                        ret = self._pack(v)
 *                        if ret != 0: break             # <<<<<<<<<<<<<<
 *             elif PyList_CheckExact(o):
 *                 L = Py_SIZE(o)
 */
          __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
          if (__pyx_t_2) {
            goto __pyx_L29_break;
          }

          /* "ddtrace/internal/_encoding.pyx":858
 *                 ret = msgpack_pack_map(&self.pk, L)
 *                 if ret == 0:
 *                     for k, v in d.items():             # <<<<<<<<<<<<<<
 *                        ret = self._pack(k)
 *                        if ret != 0: break
 */
        }
        __pyx_L29_break:;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "ddtrace/internal/_encoding.pyx":857
 *                     raise ValueError("dict is too large")
 *                 ret = msgpack_pack_map(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     for k, v in d.items():
 *                        ret = self._pack(k)
 */
      }

      /* "ddtrace/internal/_encoding.pyx":851
 *                         rawval = o
 *                         ret = msgpack_pack_raw_body(&self.pk, rawval, L)
 *             elif PyDict_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 d = <dict>o
 *                 L = len(d)
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":863
 *                        ret = self._pack(v)
 *                        if ret != 0: break
 *             elif PyList_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 L = Py_SIZE(o)
 *                 if L > ITEM_LIMIT:
 */
    __pyx_t_2 = (PyList_CheckExact(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":864
 *                        if ret != 0: break
 *             elif PyList_CheckExact(o):
 *                 L = Py_SIZE(o)             # <<<<<<<<<<<<<<
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("list is too large")
 */
      __pyx_v_L = Py_SIZE(__pyx_v_o);

      /* "ddtrace/internal/_encoding.pyx":865
 *             elif PyList_CheckExact(o):
 *                 L = Py_SIZE(o)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     raise ValueError("list is too large")
 *                 ret = msgpack_pack_array(&self.pk, L)
 */
      __pyx_t_2 = ((__pyx_v_L > __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT) != 0);
      if (unlikely(__pyx_t_2)) {

        /* "ddtrace/internal/_encoding.pyx":866
 *                 L = Py_SIZE(o)
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("list is too large")             # <<<<<<<<<<<<<<
 *                 ret = msgpack_pack_array(&self.pk, L)
 *                 if ret == 0:
 */
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 866, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_Raise(__pyx_t_10, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __PYX_ERR(0, 866, __pyx_L1_error)

        /* "ddtrace/internal/_encoding.pyx":865
 *             elif PyList_CheckExact(o):
 *                 L = Py_SIZE(o)
 *                 if L > ITEM_LIMIT:             # <<<<<<<<<<<<<<
 *                     raise ValueError("list is too large")
 *                 ret = msgpack_pack_array(&self.pk, L)
 */
      }

      /* "ddtrace/internal/_encoding.pyx":867
 *                 if L > ITEM_LIMIT:
 *                     raise ValueError("list is too large")
 *                 ret = msgpack_pack_array(&self.pk, L)             # <<<<<<<<<<<<<<
 *                 if ret == 0:
 *                     for v in o:
 */
      __pyx_v_ret = msgpack_pack_array((&__pyx_v_self->pk), __pyx_v_L);

      /* "ddtrace/internal/_encoding.pyx":868
 *                     raise ValueError("list is too large")
 *                 ret = msgpack_pack_array(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     for v in o:
 *                         ret = self._pack(v)
 */
      __pyx_t_2 = ((__pyx_v_ret == 0) != 0);
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":869
 *                 ret = msgpack_pack_array(&self.pk, L)
 *                 if ret == 0:
 *                     for v in o:             # <<<<<<<<<<<<<<
 *                         ret = self._pack(v)
 *                         if ret != 0: break
 */
        if (likely(PyList_CheckExact(__pyx_v_o)) || PyTuple_CheckExact(__pyx_v_o)) {
          __pyx_t_10 = __pyx_v_o; __Pyx_INCREF(__pyx_t_10); __pyx_t_18 = 0;
          __pyx_t_21 = NULL;
        } else {
          __pyx_t_18 = -1; __pyx_t_10 = PyObject_GetIter(__pyx_v_o); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 869, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __pyx_t_21 = Py_TYPE(__pyx_t_10)->tp_iternext; if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 869, __pyx_L1_error)
        }
        for (;;) {
          if (likely(!__pyx_t_21)) {
            if (likely(PyList_CheckExact(__pyx_t_10))) {
              if (__pyx_t_18 >= PyList_GET_SIZE(__pyx_t_10)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_11 = PyList_GET_ITEM(__pyx_t_10, __pyx_t_18); __Pyx_INCREF(__pyx_t_11); __pyx_t_18++; if (unlikely(0 < 0)) __PYX_ERR(0, 869, __pyx_L1_error)
              #else
              __pyx_t_11 = PySequence_ITEM(__pyx_t_10, __pyx_t_18); __pyx_t_18++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 869, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              #endif
            } else {
              if (__pyx_t_18 >= PyTuple_GET_SIZE(__pyx_t_10)) break;
              #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
              __pyx_t_11 = PyTuple_GET_ITEM(__pyx_t_10, __pyx_t_18); __Pyx_INCREF(__pyx_t_11); __pyx_t_18++; if (unlikely(0 < 0)) __PYX_ERR(0, 869, __pyx_L1_error)
              #else
              __pyx_t_11 = PySequence_ITEM(__pyx_t_10, __pyx_t_18); __pyx_t_18++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 869, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              #endif
            }
          } else {
            __pyx_t_11 = __pyx_t_21(__pyx_t_10);
            if (unlikely(!__pyx_t_11)) {
              PyObject* exc_type = PyErr_Occurred();
              if (exc_type) {
                if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
                else __PYX_ERR(0, 869, __pyx_L1_error)
              }
              break;
            }
            __Pyx_GOTREF(__pyx_t_11);
          }
          __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_11);
          __pyx_t_11 = 0;

          /* "ddtrace/internal/_encoding.pyx":870
 *                 if ret == 0:
 *                     for v in o:
 *                         ret = self._pack(v)             # <<<<<<<<<<<<<<
 *                         if ret != 0: break
 *             elif PyBool_Check(o):
 */
          __pyx_t_9 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self->__pyx_vtab)->_pack(__pyx_v_self, __pyx_v_v); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 870, __pyx_L1_error)
          __pyx_v_ret = __pyx_t_9;

          /* "ddtrace/internal/_encoding.pyx":871
 *                     for v in o:
 *                         ret = self._pack(v)
 *                         if ret != 0: break             # <<<<<<<<<<<<<<
 *             elif PyBool_Check(o):
 *                 if o:
 */
          __pyx_t_2 = ((__pyx_v_ret != 0) != 0);
          if (__pyx_t_2) {
            goto __pyx_L37_break;
          }

          /* "ddtrace/internal/_encoding.pyx":869
 *                 ret = msgpack_pack_array(&self.pk, L)
 *                 if ret == 0:
 *                     for v in o:             # <<<<<<<<<<<<<<
 *                         ret = self._pack(v)
 *                         if ret != 0: break
 */
        }
        __pyx_L37_break:;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "ddtrace/internal/_encoding.pyx":868
 *                     raise ValueError("list is too large")
 *                 ret = msgpack_pack_array(&self.pk, L)
 *                 if ret == 0:             # <<<<<<<<<<<<<<
 *                     for v in o:
 *                         ret = self._pack(v)
 */
      }

      /* "ddtrace/internal/_encoding.pyx":863
 *                        ret = self._pack(v)
 *                        if ret != 0: break
 *             elif PyList_CheckExact(o):             # <<<<<<<<<<<<<<
 *                 L = Py_SIZE(o)
 *                 if L > ITEM_LIMIT:
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":872
 *                         ret = self._pack(v)
 *                         if ret != 0: break
 *             elif PyBool_Check(o):             # <<<<<<<<<<<<<<
 *                 if o:
 *                     ret = msgpack_pack_true(&self.pk)
 */
    __pyx_t_2 = (PyBool_Check(__pyx_v_o) != 0);
    if (__pyx_t_2) {

      /* "ddtrace/internal/_encoding.pyx":873
 *                         if ret != 0: break
 *             elif PyBool_Check(o):
 *                 if o:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_true(&self.pk)
 *                 else:
 */
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_o); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 873, __pyx_L1_error)
      if (__pyx_t_2) {

        /* "ddtrace/internal/_encoding.pyx":874
 *             elif PyBool_Check(o):
 *                 if o:
 *                     ret = msgpack_pack_true(&self.pk)             # <<<<<<<<<<<<<<
 *                 else:
 *                     ret = msgpack_pack_false(&self.pk)
 */
        __pyx_v_ret = msgpack_pack_true((&__pyx_v_self->pk));

        /* "ddtrace/internal/_encoding.pyx":873
 *                         if ret != 0: break
 *             elif PyBool_Check(o):
 *                 if o:             # <<<<<<<<<<<<<<
 *                     ret = msgpack_pack_true(&self.pk)
 *                 else:
 */
        goto __pyx_L39;
      }

      /* "ddtrace/internal/_encoding.pyx":876
 *                     ret = msgpack_pack_true(&self.pk)
 *                 else:
 *                     ret = msgpack_pack_false(&self.pk)             # <<<<<<<<<<<<<<
 *             else:
 *                 PyErr_Format(TypeError, b"can not serialize '%.200s' object", Py_TYPE(o).tp_name)
 */
      /*else*/ {
        __pyx_v_ret = msgpack_pack_false((&__pyx_v_self->pk));
      }
      __pyx_L39:;

      /* "ddtrace/internal/_encoding.pyx":872
 *                         ret = self._pack(v)
 *                         if ret != 0: break
 *             elif PyBool_Check(o):             # <<<<<<<<<<<<<<
 *                 if o:
 *                     ret = msgpack_pack_true(&self.pk)
 */
      goto __pyx_L5;
    }

    /* "ddtrace/internal/_encoding.pyx":878
 *                     ret = msgpack_pack_false(&self.pk)
 *             else:
 *                 PyErr_Format(TypeError, b"can not serialize '%.200s' object", Py_TYPE(o).tp_name)             # <<<<<<<<<<<<<<
 *             return ret
 * 
 */
    /*else*/ {
      __pyx_t_19 = PyErr_Format(__pyx_builtin_TypeError, ((char *)"can not serialize '%.200s' object"), Py_TYPE(__pyx_v_o)->tp_name); if (unlikely(__pyx_t_19 == ((PyObject *)NULL))) __PYX_ERR(0, 878, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "ddtrace/internal/_encoding.pyx":879
 *             else:
 *                 PyErr_Format(TypeError, b"can not serialize '%.200s' object", Py_TYPE(o).tp_name)
 *             return ret             # <<<<<<<<<<<<<<
 * 
 *     cpdef pack(self, object obj):
 */
    __pyx_r = __pyx_v_ret;
    goto __pyx_L0;
    __pyx_L3_continue:;
  }

  /* "ddtrace/internal/_encoding.pyx":789
 *         self.pk.buf = NULL
 * 
 *     cdef int _pack(self, object o) except -1:             # <<<<<<<<<<<<<<
 *         cdef long long llval
 *         cdef unsigned long long ullval
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer._pack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_d);
  __Pyx_XDECREF(__pyx_v_oe);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_o);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":881
 *             return ret
 * 
 *     cpdef pack(self, object obj):             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         try:
 */

static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_7pack(PyObject *__pyx_v_self, PyObject *__pyx_v_obj); /*proto*/
static PyObject *__pyx_f_7ddtrace_8internal_9_encoding_6Packer_pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_obj, int __pyx_skip_dispatch) {
  int __pyx_v_ret;
  PyObject *__pyx_v_buf = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_pack); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 881, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_7pack)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_obj) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_obj);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 881, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "ddtrace/internal/_encoding.pyx":883
 *     cpdef pack(self, object obj):
 *         cdef int ret
 *         try:             # <<<<<<<<<<<<<<
 *             ret = self._pack(obj)
 *         except:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    /*try:*/ {

      /* "ddtrace/internal/_encoding.pyx":884
 *         cdef int ret
 *         try:
 *             ret = self._pack(obj)             # <<<<<<<<<<<<<<
 *         except:
 *             self.pk.length = 0
 */
      __pyx_t_8 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self->__pyx_vtab)->_pack(__pyx_v_self, __pyx_v_obj); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 884, __pyx_L3_error)
      __pyx_v_ret = __pyx_t_8;

      /* "ddtrace/internal/_encoding.pyx":883
 *     cpdef pack(self, object obj):
 *         cdef int ret
 *         try:             # <<<<<<<<<<<<<<
 *             ret = self._pack(obj)
 *         except:
 */
    }
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "ddtrace/internal/_encoding.pyx":885
 *         try:
 *             ret = self._pack(obj)
 *         except:             # <<<<<<<<<<<<<<
 *             self.pk.length = 0
 * 
 */
    /*except:*/ {
      __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.pack", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 885, __pyx_L5_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GOTREF(__pyx_t_3);

      /* "ddtrace/internal/_encoding.pyx":886
 *             ret = self._pack(obj)
 *         except:
 *             self.pk.length = 0             # <<<<<<<<<<<<<<
 * 
 *             raise
 */
      __pyx_v_self->pk.length = 0;

      /* "ddtrace/internal/_encoding.pyx":888
 *             self.pk.length = 0
 * 
 *             raise             # <<<<<<<<<<<<<<
 *         if ret:  # should not happen.
 *             raise RuntimeError("internal error")
 */
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_2);
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_2, __pyx_t_3);
      __pyx_t_1 = 0; __pyx_t_2 = 0; __pyx_t_3 = 0; 
      __PYX_ERR(0, 888, __pyx_L5_except_error)
    }
    __pyx_L5_except_error:;

    /* "ddtrace/internal/_encoding.pyx":883
 *     cpdef pack(self, object obj):
 *         cdef int ret
 *         try:             # <<<<<<<<<<<<<<
 *             ret = self._pack(obj)
 *         except:
 */
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
    goto __pyx_L1_error;
    __pyx_L8_try_end:;
  }

  /* "ddtrace/internal/_encoding.pyx":889
 * 
 *             raise
 *         if ret:  # should not happen.             # <<<<<<<<<<<<<<
 *             raise RuntimeError("internal error")
 * 
 */
  __pyx_t_9 = (__pyx_v_ret != 0);
  if (unlikely(__pyx_t_9)) {

    /* "ddtrace/internal/_encoding.pyx":890
 *             raise
 *         if ret:  # should not happen.
 *             raise RuntimeError("internal error")             # <<<<<<<<<<<<<<
 * 
 *         # Reset the buffer.
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_RuntimeError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 890, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 890, __pyx_L1_error)

    /* "ddtrace/internal/_encoding.pyx":889
 * 
 *             raise
 *         if ret:  # should not happen.             # <<<<<<<<<<<<<<
 *             raise RuntimeError("internal error")
 * 
 */
  }

  /* "ddtrace/internal/_encoding.pyx":893
 * 
 *         # Reset the buffer.
 *         buf = PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)             # <<<<<<<<<<<<<<
 *         self.pk.length = 0
 *         return buf
 */
  __pyx_t_3 = PyBytes_FromStringAndSize(__pyx_v_self->pk.buf, __pyx_v_self->pk.length); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_buf = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "ddtrace/internal/_encoding.pyx":894
 *         # Reset the buffer.
 *         buf = PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)
 *         self.pk.length = 0             # <<<<<<<<<<<<<<
 *         return buf
 * 
 */
  __pyx_v_self->pk.length = 0;

  /* "ddtrace/internal/_encoding.pyx":895
 *         buf = PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)
 *         self.pk.length = 0
 *         return buf             # <<<<<<<<<<<<<<
 * 
 *     def bytes(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_buf);
  __pyx_r = __pyx_v_buf;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":881
 *             return ret
 * 
 *     cpdef pack(self, object obj):             # <<<<<<<<<<<<<<
 *         cdef int ret
 *         try:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.pack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_buf);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_7pack(PyObject *__pyx_v_self, PyObject *__pyx_v_obj); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_7pack(PyObject *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pack (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_6pack(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self), ((PyObject *)__pyx_v_obj));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_6pack(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, PyObject *__pyx_v_obj) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("pack", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_7ddtrace_8internal_9_encoding_6Packer_pack(__pyx_v_self, __pyx_v_obj, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 881, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.pack", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":897
 *         return buf
 * 
 *     def bytes(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer contents as bytes object"""
 *         return PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_9bytes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_7ddtrace_8internal_9_encoding_6Packer_8bytes[] = "Return internal buffer contents as bytes object";
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_9bytes(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("bytes (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_8bytes(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_8bytes(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bytes", 0);

  /* "ddtrace/internal/_encoding.pyx":899
 *     def bytes(self):
 *         """Return internal buffer contents as bytes object"""
 *         return PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyBytes_FromStringAndSize(__pyx_v_self->pk.buf, __pyx_v_self->pk.length); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":897
 *         return buf
 * 
 *     def bytes(self):             # <<<<<<<<<<<<<<
 *         """Return internal buffer contents as bytes object"""
 *         return PyBytes_FromStringAndSize(self.pk.buf, self.pk.length)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.bytes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_11__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_10__reduce_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_13__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_6Packer_12__setstate_cython__(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_6Packer_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_Raise(__pyx_t_1, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("ddtrace.internal._encoding.Packer.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "ddtrace/internal/_encoding.pyx":902
 * 
 * 
 * def packb(o, **kwargs):             # <<<<<<<<<<<<<<
 *     """
 *     Pack object `o` and return packed bytes
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_1packb(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_7ddtrace_8internal_9_encoding_packb[] = "\n    Pack object `o` and return packed bytes\n    See :class:`Packer` for options.\n    ";
static PyMethodDef __pyx_mdef_7ddtrace_8internal_9_encoding_1packb = {"packb", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7ddtrace_8internal_9_encoding_1packb, METH_VARARGS|METH_KEYWORDS, __pyx_doc_7ddtrace_8internal_9_encoding_packb};
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_1packb(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_o = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("packb (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return NULL;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_o,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_o)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "packb") < 0)) __PYX_ERR(0, 902, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_o = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("packb", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 902, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("ddtrace.internal._encoding.packb", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_packb(__pyx_self, __pyx_v_o, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_packb(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_o, PyObject *__pyx_v_kwargs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("packb", 0);

  /* "ddtrace/internal/_encoding.pyx":907
 *     See :class:`Packer` for options.
 *     """
 *     return Packer(**kwargs).pack(o)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_Packer), __pyx_empty_tuple, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = ((struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer *)((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_t_2)->__pyx_vtab)->pack(((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)__pyx_t_2), __pyx_v_o, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 907, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "ddtrace/internal/_encoding.pyx":902
 * 
 * 
 * def packb(o, **kwargs):             # <<<<<<<<<<<<<<
 *     """
 *     Pack object `o` and return packed bytes
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("ddtrace.internal._encoding.packb", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_StringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_3__pyx_unpickle_StringTable(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7ddtrace_8internal_9_encoding_3__pyx_unpickle_StringTable = {"__pyx_unpickle_StringTable", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7ddtrace_8internal_9_encoding_3__pyx_unpickle_StringTable, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_3__pyx_unpickle_StringTable(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_StringTable (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StringTable", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StringTable", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_StringTable") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StringTable", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_StringTable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_2__pyx_unpickle_StringTable(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_2__pyx_unpickle_StringTable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StringTable", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb9d0cd9, 0x1e618df, 0x3f658ea):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__30, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb9d0cd9, 0x1e618df, 0x3f658ea):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 *     __pyx_result = StringTable.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xb9d0cd9, 0x1e618df, 0x3f658ea):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = StringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xb9d0cd9, 0x1e618df, 0x3f658ea):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 *     __pyx_result = StringTable.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_StringTable), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 *     __pyx_result = StringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = StringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_StringTable__set_state(((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xb9d0cd9, 0x1e618df, 0x3f658ea) = (_next_id, _table))" % __pyx_checksum)
 *     __pyx_result = StringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_StringTable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_StringTable__set_state(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  uint32_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StringTable__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_uint32_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_next_id = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_table);
  __Pyx_DECREF(__pyx_v___pyx_result->_table);
  __pyx_v___pyx_result->_table = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 2) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[2])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[2])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_StringTable__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ListStringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_5__pyx_unpickle_ListStringTable(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_7ddtrace_8internal_9_encoding_5__pyx_unpickle_ListStringTable = {"__pyx_unpickle_ListStringTable", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_7ddtrace_8internal_9_encoding_5__pyx_unpickle_ListStringTable, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_7ddtrace_8internal_9_encoding_5__pyx_unpickle_ListStringTable(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ListStringTable (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ListStringTable", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ListStringTable", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ListStringTable") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ListStringTable", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_ListStringTable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_7ddtrace_8internal_9_encoding_4__pyx_unpickle_ListStringTable(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_7ddtrace_8internal_9_encoding_4__pyx_unpickle_ListStringTable(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ListStringTable", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xae267b1, 0x54e003f, 0xd98eea9):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_t_1, __pyx_tuple__31, Py_NE)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xae267b1, 0x54e003f, 0xd98eea9):
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 *     __pyx_result = ListStringTable.__new__(__pyx_type)
 */
    __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_PickleError);
    __pyx_t_4 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_1, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v___pyx_PickleError = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum not in (0xae267b1, 0x54e003f, 0xd98eea9):
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ListStringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_1 = __pyx_v___pyx_PickleError; __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum not in (0xae267b1, 0x54e003f, 0xd98eea9):             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 *     __pyx_result = ListStringTable.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable), __pyx_n_s_new); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_4 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_5, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 *     __pyx_result = ListStringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_3 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "(tree fragment)":9
 *     __pyx_result = ListStringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_4 = __pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_ListStringTable__set_state(((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (0x%x vs (0xae267b1, 0x54e003f, 0xd98eea9) = (_list, _next_id, _table))" % __pyx_checksum)
 *     __pyx_result = ListStringTable.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ListStringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_ListStringTable", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_7ddtrace_8internal_9_encoding___pyx_unpickle_ListStringTable__set_state(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  uint32_t __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ListStringTable__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_list);
  __Pyx_DECREF(__pyx_v___pyx_result->_list);
  __pyx_v___pyx_result->_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_uint32_t(__pyx_t_1); if (unlikely((__pyx_t_2 == ((uint32_t)-1)) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._next_id = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._table);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._table);
  __pyx_v___pyx_result->__pyx_base._table = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 3) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ListStringTable__set_state(<ListStringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ListStringTable__set_state(ListStringTable __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._list = __pyx_state[0]; __pyx_result._next_id = __pyx_state[1]; __pyx_result._table = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("ddtrace.internal._encoding.__pyx_unpickle_ListStringTable__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable __pyx_vtable_7ddtrace_8internal_9_encoding_StringTable;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_StringTable(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)o);
  p->__pyx_vtab = __pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable;
  p->_table = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_StringTable(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_table);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_StringTable(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)o;
  if (p->_table) {
    e = (*v)(p->_table, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_StringTable(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *)o;
  tmp = ((PyObject*)p->_table);
  p->_table = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_StringTable[] = {
  {"index", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_3index, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_9__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_11__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_StringTable = {
  __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_5__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_7__contains__, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_StringTable = {
  __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_5__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_StringTable = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.StringTable", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_StringTable, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_StringTable, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_StringTable, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_StringTable, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_StringTable, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_StringTable, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7ddtrace_8internal_9_encoding_11StringTable_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_StringTable, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListStringTable __pyx_vtable_7ddtrace_8internal_9_encoding_ListStringTable;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_ListStringTable(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_StringTable(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable*)__pyx_vtabptr_7ddtrace_8internal_9_encoding_ListStringTable;
  p->_list = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_ListStringTable(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_list);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_StringTable(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_ListStringTable(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)o;
  e = __pyx_tp_traverse_7ddtrace_8internal_9_encoding_StringTable(o, v, a); if (e) return e;
  if (p->_list) {
    e = (*v)(p->_list, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_ListStringTable(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable *)o;
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_StringTable(o);
  tmp = ((PyObject*)p->_list);
  p->_list = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_ListStringTable[] = {
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_5__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_7__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_ListStringTable = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.ListStringTable", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListStringTable), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_ListStringTable, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_ListStringTable, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_ListStringTable, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_3__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_ListStringTable, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7ddtrace_8internal_9_encoding_15ListStringTable_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_ListStringTable, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackStringTable __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackStringTable(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_StringTable(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_StringTable*)__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackStringTable;
  p->_lock = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackStringTable(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_lock);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_StringTable(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_MsgpackStringTable(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)o;
  e = __pyx_tp_traverse_7ddtrace_8internal_9_encoding_StringTable(o, v, a); if (e) return e;
  if (p->_lock) {
    e = (*v)(p->_lock, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_MsgpackStringTable(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)o;
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_StringTable(o);
  tmp = ((PyObject*)p->_lock);
  p->_lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_18MsgpackStringTable_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackStringTable[] = {
  {"flush", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_5flush, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackStringTable[] = {
  {(char *)"size", __pyx_getprop_7ddtrace_8internal_9_encoding_18MsgpackStringTable_size, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.MsgpackStringTable", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackStringTable_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackStringTable, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_BufferedEncoder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)o);
  p->_lock = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_BufferedEncoder(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_lock);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)o;
  if (p->_lock) {
    e = (*v)(p->_lock, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder *)o;
  tmp = ((PyObject*)p->_lock);
  p->_lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_1__get__(o);
}

static int __pyx_setprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_8max_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_item_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_1__get__(o);
}

static int __pyx_setprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_item_size(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_13max_item_size_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_BufferedEncoder[] = {
  {"put", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_3put, METH_O, 0},
  {"encode", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_5encode, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_15BufferedEncoder_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7ddtrace_8internal_9_encoding_BufferedEncoder[] = {
  {(char *)"max_size", __pyx_getprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_size, __pyx_setprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_size, (char *)0, 0},
  {(char *)"max_item_size", __pyx_getprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_item_size, __pyx_setprop_7ddtrace_8internal_9_encoding_15BufferedEncoder_max_item_size, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.BufferedEncoder", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_BufferedEncoder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_ListBufferedEncoder __pyx_vtable_7ddtrace_8internal_9_encoding_ListBufferedEncoder;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_ListBufferedEncoder(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_BufferedEncoder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)o);
  p->__pyx_vtab = __pyx_vtabptr_7ddtrace_8internal_9_encoding_ListBufferedEncoder;
  p->_buffer = ((PyObject*)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_ListBufferedEncoder(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_buffer);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_BufferedEncoder(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_ListBufferedEncoder(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)o;
  e = __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder(o, v, a); if (e) return e;
  if (p->_buffer) {
    e = (*v)(p->_buffer, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_ListBufferedEncoder(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *)o;
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder(o);
  tmp = ((PyObject*)p->_buffer);
  p->_buffer = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_ListBufferedEncoder[] = {
  {"put", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_5put, METH_O, __pyx_doc_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_4put},
  {"get", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_7get, METH_NOARGS, __pyx_doc_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_6get},
  {"encode_item", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_9encode_item, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7ddtrace_8internal_9_encoding_ListBufferedEncoder[] = {
  {(char *)"size", __pyx_getprop_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_size, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_ListBufferedEncoder = {
  __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_3__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_ListBufferedEncoder = {
  __pyx_pw_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_3__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.ListBufferedEncoder", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_ListBufferedEncoder, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_ListBufferedEncoder, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_ListBufferedEncoder, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_BufferedEncoder(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  if (unlikely(__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_BufferedEncoder(o);
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderBase[] = {
  {"_decode", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_7_decode, METH_O, 0},
  {"encode", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_9encode, METH_NOARGS, 0},
  {"put", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_11put, METH_O, __pyx_doc_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_10put},
  {"flush", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_13flush, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_15__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_17__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackEncoderBase[] = {
  {(char *)"size", __pyx_getprop_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_size, 0, (char *)"Return the size in bytes of the encoder buffer.", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_MsgpackEncoderBase = {
  __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_5__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  0, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_MsgpackEncoderBase = {
  __pyx_pw_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_5__len__, /*mp_length*/
  0, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.MsgpackEncoderBase", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_MsgpackEncoderBase, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_MsgpackEncoderBase, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackEncoderBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV03(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase*)__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;
  return o;
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderV03[] = {
  {"flush", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_1flush, METH_NOARGS, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_3__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_5__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.MsgpackEncoderV03", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderV03, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV03, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV05(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *p;
  PyObject *o = __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_MsgpackEncoderBase*)__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;
  p->_st = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)Py_None); Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderV05(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_st);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderBase(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_MsgpackEncoderV05(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)o;
  e = __pyx_tp_traverse_7ddtrace_8internal_9_encoding_BufferedEncoder(o, v, a); if (e) return e;
  if (p->_st) {
    e = (*v)(((PyObject *)p->_st), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_MsgpackEncoderV05(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *)o;
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_BufferedEncoder(o);
  tmp = ((PyObject*)p->_st);
  p->_st = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderV05[] = {
  {"flush", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_3flush, METH_NOARGS, 0},
  {"put", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_5put, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_7__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_9__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackEncoderV05[] = {
  {(char *)"size", __pyx_getprop_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_size, 0, (char *)"Return the size in bytes of the encoder buffer.", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.MsgpackEncoderV05", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_MsgpackEncoderV05, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};
static struct __pyx_vtabstruct_7ddtrace_8internal_9_encoding_Packer __pyx_vtable_7ddtrace_8internal_9_encoding_Packer;

static PyObject *__pyx_tp_new_7ddtrace_8internal_9_encoding_Packer(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)o);
  p->__pyx_vtab = __pyx_vtabptr_7ddtrace_8internal_9_encoding_Packer;
  p->_default = Py_None; Py_INCREF(Py_None);
  p->_berrors = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_1__cinit__(o, __pyx_empty_tuple, NULL) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_Packer(PyObject *o) {
  struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_5__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->_default);
  Py_CLEAR(p->_berrors);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_7ddtrace_8internal_9_encoding_Packer(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)o;
  if (p->_default) {
    e = (*v)(p->_default, a); if (e) return e;
  }
  if (p->_berrors) {
    e = (*v)(p->_berrors, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_7ddtrace_8internal_9_encoding_Packer(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *p = (struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *)o;
  tmp = ((PyObject*)p->_default);
  p->_default = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_berrors);
  p->_berrors = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_7ddtrace_8internal_9_encoding_Packer[] = {
  {"pack", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_7pack, METH_O, 0},
  {"bytes", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_9bytes, METH_NOARGS, __pyx_doc_7ddtrace_8internal_9_encoding_6Packer_8bytes},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_11__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_7ddtrace_8internal_9_encoding_6Packer_13__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_7ddtrace_8internal_9_encoding_Packer = {
  PyVarObject_HEAD_INIT(0, 0)
  "ddtrace.internal._encoding.Packer", /*tp_name*/
  sizeof(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_7ddtrace_8internal_9_encoding_Packer, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Slightly modified version of the v0.6.2 msgpack Packer\n    which only supports basic Python types (int, bool, float, dict, list).\n\n    Note that _only_ the basic types can be encoded. Subtypes of these types\n    are not supported.\n\n    - strict_type argument is removed and assumed to be True\n    - use_bin_type argument is removed and assumed to be True (use the msgpack 2.0 bin type fields when possible)\n    - use_single_float is removed and assumed to be False\n    - autoreset is removed and assumed to be True (bytes are always returned from pack and the buffer reset)\n\n    https://github.com/msgpack/msgpack-python/tree/v0.6.2\n    ", /*tp_doc*/
  __pyx_tp_traverse_7ddtrace_8internal_9_encoding_Packer, /*tp_traverse*/
  __pyx_tp_clear_7ddtrace_8internal_9_encoding_Packer, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_7ddtrace_8internal_9_encoding_Packer, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_7ddtrace_8internal_9_encoding_6Packer_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_7ddtrace_8internal_9_encoding_Packer, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
  0, /*tp_pypy_flags*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__encoding(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__encoding},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "_encoding",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_BufferFull, __pyx_k_BufferFull, sizeof(__pyx_k_BufferFull), 0, 0, 1, 1},
  {&__pyx_n_s_BufferItemTooLarge, __pyx_k_BufferItemTooLarge, sizeof(__pyx_k_BufferItemTooLarge), 0, 0, 1, 1},
  {&__pyx_n_s_BufferedEncoder, __pyx_k_BufferedEncoder, sizeof(__pyx_k_BufferedEncoder), 0, 0, 1, 1},
  {&__pyx_kp_s_Cannot_insert_s_string_table_is, __pyx_k_Cannot_insert_s_string_table_is, sizeof(__pyx_k_Cannot_insert_s_string_table_is), 0, 0, 1, 0},
  {&__pyx_kp_s_Couldn_t_pack_span, __pyx_k_Couldn_t_pack_span, sizeof(__pyx_k_Couldn_t_pack_span), 0, 0, 1, 0},
  {&__pyx_kp_s_Couldn_t_pack_trace, __pyx_k_Couldn_t_pack_trace, sizeof(__pyx_k_Couldn_t_pack_trace), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_add_string_to_msgpack, __pyx_k_Failed_to_add_string_to_msgpack, sizeof(__pyx_k_Failed_to_add_string_to_msgpack), 0, 0, 1, 0},
  {&__pyx_kp_s_Failed_to_append_raw_bytes_to_ms, __pyx_k_Failed_to_append_raw_bytes_to_ms, sizeof(__pyx_k_Failed_to_append_raw_bytes_to_ms), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0, __pyx_k_Incompatible_checksums_0x_x_vs_0, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_0x_x_vs_0_2, __pyx_k_Incompatible_checksums_0x_x_vs_0_2, sizeof(__pyx_k_Incompatible_checksums_0x_x_vs_0_2), 0, 0, 1, 0},
  {&__pyx_kp_s_Integer_value_out_of_range, __pyx_k_Integer_value_out_of_range, sizeof(__pyx_k_Integer_value_out_of_range), 0, 0, 1, 0},
  {&__pyx_n_s_ListBufferedEncoder, __pyx_k_ListBufferedEncoder, sizeof(__pyx_k_ListBufferedEncoder), 0, 0, 1, 1},
  {&__pyx_n_s_ListStringTable, __pyx_k_ListStringTable, sizeof(__pyx_k_ListStringTable), 0, 0, 1, 1},
  {&__pyx_n_s_Lock, __pyx_k_Lock, sizeof(__pyx_k_Lock), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_MsgpackEncoderBase, __pyx_k_MsgpackEncoderBase, sizeof(__pyx_k_MsgpackEncoderBase), 0, 0, 1, 1},
  {&__pyx_n_s_MsgpackEncoderV03, __pyx_k_MsgpackEncoderV03, sizeof(__pyx_k_MsgpackEncoderV03), 0, 0, 1, 1},
  {&__pyx_n_s_MsgpackEncoderV05, __pyx_k_MsgpackEncoderV05, sizeof(__pyx_k_MsgpackEncoderV05), 0, 0, 1, 1},
  {&__pyx_n_s_MsgpackStringTable, __pyx_k_MsgpackStringTable, sizeof(__pyx_k_MsgpackStringTable), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_ORIGIN_KEY, __pyx_k_ORIGIN_KEY, sizeof(__pyx_k_ORIGIN_KEY), 0, 0, 1, 1},
  {&__pyx_n_s_OverflowError, __pyx_k_OverflowError, sizeof(__pyx_k_OverflowError), 0, 0, 1, 1},
  {&__pyx_n_s_Packer, __pyx_k_Packer, sizeof(__pyx_k_Packer), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_kp_s_Pickling_of_struct_members_such, __pyx_k_Pickling_of_struct_members_such, sizeof(__pyx_k_Pickling_of_struct_members_such), 0, 0, 1, 0},
  {&__pyx_n_s_RLock, __pyx_k_RLock, sizeof(__pyx_k_RLock), 0, 0, 1, 1},
  {&__pyx_n_s_RuntimeError, __pyx_k_RuntimeError, sizeof(__pyx_k_RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s_StringTable, __pyx_k_StringTable, sizeof(__pyx_k_StringTable), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Unable_to_allocate_internal_buff, __pyx_k_Unable_to_allocate_internal_buff, sizeof(__pyx_k_Unable_to_allocate_internal_buff), 0, 0, 1, 0},
  {&__pyx_kp_s_Unhandled_meta_type_r, __pyx_k_Unhandled_meta_type_r, sizeof(__pyx_k_Unhandled_meta_type_r), 0, 0, 1, 0},
  {&__pyx_kp_s_Unhandled_metrics_type_r, __pyx_k_Unhandled_metrics_type_r, sizeof(__pyx_k_Unhandled_metrics_type_r), 0, 0, 1, 0},
  {&__pyx_kp_s_Unhandled_numeric_type_r, __pyx_k_Unhandled_numeric_type_r, sizeof(__pyx_k_Unhandled_numeric_type_r), 0, 0, 1, 0},
  {&__pyx_kp_s_Unhandled_text_type_r, __pyx_k_Unhandled_text_type_r, sizeof(__pyx_k_Unhandled_text_type_r), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 0, 1, 0},
  {&__pyx_kp_s_application_msgpack, __pyx_k_application_msgpack, sizeof(__pyx_k_application_msgpack), 0, 0, 1, 0},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_content_type, __pyx_k_content_type, sizeof(__pyx_k_content_type), 0, 0, 1, 1},
  {&__pyx_n_s_context, __pyx_k_context, sizeof(__pyx_k_context), 0, 0, 1, 1},
  {&__pyx_n_s_dd_origin, __pyx_k_dd_origin, sizeof(__pyx_k_dd_origin), 0, 0, 1, 1},
  {&__pyx_n_s_ddtrace_constants, __pyx_k_ddtrace_constants, sizeof(__pyx_k_ddtrace_constants), 0, 0, 1, 1},
  {&__pyx_n_s_ddtrace_internal__encoding, __pyx_k_ddtrace_internal__encoding, sizeof(__pyx_k_ddtrace_internal__encoding), 0, 0, 1, 1},
  {&__pyx_kp_s_ddtrace_internal__encoding_pyx, __pyx_k_ddtrace_internal__encoding_pyx, sizeof(__pyx_k_ddtrace_internal__encoding_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_default, __pyx_k_default, sizeof(__pyx_k_default), 0, 0, 1, 1},
  {&__pyx_kp_s_default_must_be_a_callable, __pyx_k_default_must_be_a_callable, sizeof(__pyx_k_default_must_be_a_callable), 0, 0, 1, 0},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_kp_s_dict_is_too_large, __pyx_k_dict_is_too_large, sizeof(__pyx_k_dict_is_too_large), 0, 0, 1, 0},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_duration_ns, __pyx_k_duration_ns, sizeof(__pyx_k_duration_ns), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_encode_item, __pyx_k_encode_item, sizeof(__pyx_k_encode_item), 0, 0, 1, 1},
  {&__pyx_n_s_enter, __pyx_k_enter, sizeof(__pyx_k_enter), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_s_exit, __pyx_k_exit, sizeof(__pyx_k_exit), 0, 0, 1, 1},
  {&__pyx_n_s_flush, __pyx_k_flush, sizeof(__pyx_k_flush), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_index, __pyx_k_index, sizeof(__pyx_k_index), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_kp_s_internal_error, __pyx_k_internal_error, sizeof(__pyx_k_internal_error), 0, 0, 1, 0},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_kwargs, __pyx_k_kwargs, sizeof(__pyx_k_kwargs), 0, 0, 1, 1},
  {&__pyx_kp_s_list_is_too_large, __pyx_k_list_is_too_large, sizeof(__pyx_k_list_is_too_large), 0, 0, 1, 0},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_max_item_size, __pyx_k_max_item_size, sizeof(__pyx_k_max_item_size), 0, 0, 1, 1},
  {&__pyx_n_s_max_size, __pyx_k_max_size, sizeof(__pyx_k_max_size), 0, 0, 1, 1},
  {&__pyx_n_s_meta, __pyx_k_meta, sizeof(__pyx_k_meta), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_metrics, __pyx_k_metrics, sizeof(__pyx_k_metrics), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_msgpack, __pyx_k_msgpack, sizeof(__pyx_k_msgpack), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_kp_s_no_default___reduce___due_to_non, __pyx_k_no_default___reduce___due_to_non, sizeof(__pyx_k_no_default___reduce___due_to_non), 0, 0, 1, 0},
  {&__pyx_n_s_o, __pyx_k_o, sizeof(__pyx_k_o), 0, 0, 1, 1},
  {&__pyx_n_s_pack, __pyx_k_pack, sizeof(__pyx_k_pack), 0, 0, 1, 1},
  {&__pyx_n_s_packb, __pyx_k_packb, sizeof(__pyx_k_packb), 0, 0, 1, 1},
  {&__pyx_n_s_parent_id, __pyx_k_parent_id, sizeof(__pyx_k_parent_id), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_put, __pyx_k_put, sizeof(__pyx_k_put), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ListStringTable, __pyx_k_pyx_unpickle_ListStringTable, sizeof(__pyx_k_pyx_unpickle_ListStringTable), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_StringTable, __pyx_k_pyx_unpickle_StringTable, sizeof(__pyx_k_pyx_unpickle_StringTable), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_raw, __pyx_k_raw, sizeof(__pyx_k_raw), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_resource, __pyx_k_resource, sizeof(__pyx_k_resource), 0, 0, 1, 1},
  {&__pyx_n_s_service, __pyx_k_service, sizeof(__pyx_k_service), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_span_id, __pyx_k_span_id, sizeof(__pyx_k_span_id), 0, 0, 1, 1},
  {&__pyx_n_s_span_type, __pyx_k_span_type, sizeof(__pyx_k_span_type), 0, 0, 1, 1},
  {&__pyx_n_s_start_ns, __pyx_k_start_ns, sizeof(__pyx_k_start_ns), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_threading, __pyx_k_threading, sizeof(__pyx_k_threading), 0, 0, 1, 1},
  {&__pyx_n_s_trace_id, __pyx_k_trace_id, sizeof(__pyx_k_trace_id), 0, 0, 1, 1},
  {&__pyx_kp_s_unicode_string_is_too_large, __pyx_k_unicode_string_is_too_large, sizeof(__pyx_k_unicode_string_is_too_large), 0, 0, 1, 0},
  {&__pyx_n_s_unpackb, __pyx_k_unpackb, sizeof(__pyx_k_unpackb), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_version, __pyx_k_version, sizeof(__pyx_k_version), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_OverflowError = __Pyx_GetBuiltinName(__pyx_n_s_OverflowError); if (!__pyx_builtin_OverflowError) __PYX_ERR(0, 99, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 108, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 121, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 193, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 214, __pyx_L1_error)
  __pyx_builtin_RuntimeError = __Pyx_GetBuiltinName(__pyx_n_s_RuntimeError); if (!__pyx_builtin_RuntimeError) __PYX_ERR(0, 240, __pyx_L1_error)
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(0, 317, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "ddtrace/internal/_encoding.pyx":100
 *             return msgpack_pack_long_long(pk, <long long> n)
 *         except OverflowError as oe:
 *             raise OverflowError("Integer value out of range")             # <<<<<<<<<<<<<<
 * 
 *     if PyInt_Check(n):
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_Integer_value_out_of_range); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "ddtrace/internal/_encoding.pyx":131
 *             ret = msgpack_pack_unicode(pk, text, ITEM_LIMIT)
 *             if ret == -2:
 *                 raise ValueError("unicode string is too large")             # <<<<<<<<<<<<<<
 *         ELSE:
 *             text = PyUnicode_AsEncodedString(text, "utf-8", NULL)
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_unicode_string_is_too_large); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "ddtrace/internal/_encoding.pyx":214
 *         self.pk.buf = <char*> PyMem_Malloc(self.pk.buf_size)
 *         if self.pk.buf == NULL:
 *             raise MemoryError("Unable to allocate internal buffer.")             # <<<<<<<<<<<<<<
 *         self.max_size = max_size
 *         self.pk.length = MSGPACK_STRING_TABLE_LENGTH_PREFIX_SIZE
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_Unable_to_allocate_internal_buff); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "ddtrace/internal/_encoding.pyx":240
 *         ret = pack_text(&self.pk, string)
 *         if ret != 0:
 *             raise RuntimeError("Failed to add string to msgpack string table")             # <<<<<<<<<<<<<<
 * 
 *     cdef savepoint(self):
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_Failed_to_add_string_to_msgpack); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 240, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "ddtrace/internal/_encoding.pyx":257
 *         cdef int old_pos = self.pk.length
 * 
 *         with self._lock:             # <<<<<<<<<<<<<<
 *             # Update table size prefix
 *             self.pk.length = offset
 */
  __pyx_tuple__6 = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "ddtrace/internal/_encoding.pyx":283
 *             res = msgpack_pack_raw_body(&self.pk, <char *>PyLong_AsLong(src), size)
 *             if res != 0:
 *                 raise RuntimeError("Failed to append raw bytes to msgpack string table")             # <<<<<<<<<<<<<<
 * 
 *     cdef reset(self):
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_kp_s_Failed_to_append_raw_bytes_to_ms); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "(tree fragment)":4
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("Pickling of struct members such as self.pk must be explicitly requested with @auto_pickle(True)")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_Pickling_of_struct_members_such); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "ddtrace/internal/_encoding.pyx":396
 *     cpdef _decode(self, data):
 *         import msgpack
 *         if msgpack.version[:2] < (0, 6):             # <<<<<<<<<<<<<<
 *             return msgpack.unpackb(data)
 *         return msgpack.unpackb(data, raw=True)
 */
  __pyx_slice__14 = PySlice_New(Py_None, __pyx_int_2, Py_None); if (unlikely(!__pyx_slice__14)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__14);
  __Pyx_GIVEREF(__pyx_slice__14);
  __pyx_tuple__15 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_6); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "ddtrace/internal/_encoding.pyx":442
 *         L = len(trace)
 *         if L > ITEM_LIMIT:
 *             raise ValueError("list is too large")             # <<<<<<<<<<<<<<
 * 
 *         ret = msgpack_pack_array(&self.pk, L)
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_list_is_too_large); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "ddtrace/internal/_encoding.pyx":445
 * 
 *         ret = msgpack_pack_array(&self.pk, L)
 *         if ret != 0: raise RuntimeError("Couldn't pack trace")             # <<<<<<<<<<<<<<
 * 
 *         if L > 0 and trace[0].context is not None and trace[0].context.dd_origin is not None:
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_Couldn_t_pack_trace); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "ddtrace/internal/_encoding.pyx":452
 *         for span in trace:
 *             ret = self.pack_span(span, dd_origin)
 *             if ret != 0: raise RuntimeError("Couldn't pack span")             # <<<<<<<<<<<<<<
 * 
 *         return ret
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_Couldn_t_pack_span); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "ddtrace/internal/_encoding.pyx":466
 *                 ret = self._pack_trace(trace)
 *                 if ret:  # should not happen.
 *                     raise RuntimeError("internal error")             # <<<<<<<<<<<<<<
 * 
 *                 # DEV: msgpack avoids buffer overflows by calling PyMem_Realloc so
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_internal_error); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "ddtrace/internal/_encoding.pyx":522
 *                 L += 1
 *             if L > ITEM_LIMIT:
 *                 raise ValueError("dict is too large")             # <<<<<<<<<<<<<<
 * 
 *             ret = msgpack_pack_map(&self.pk, L)
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_dict_is_too_large); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 522, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "ddtrace/internal/_encoding.pyx":777
 *         if default is not None:
 *             if not PyCallable_Check(default):
 *                 raise TypeError("default must be a callable.")             # <<<<<<<<<<<<<<
 *         self._default = default
 * 
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_default_must_be_a_callable); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 777, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "(tree fragment)":4
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError("no default __reduce__ due to non-trivial __cinit__")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_no_default___reduce___due_to_non); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_tuple__30 = PyTuple_Pack(3, __pyx_int_194841817, __pyx_int_31856863, __pyx_int_66476266); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);
  __pyx_tuple__31 = PyTuple_Pack(3, __pyx_int_182609841, __pyx_int_88997951, __pyx_int_228126377); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "ddtrace/internal/_encoding.pyx":902
 * 
 * 
 * def packb(o, **kwargs):             # <<<<<<<<<<<<<<
 *     """
 *     Pack object `o` and return packed bytes
 */
  __pyx_tuple__32 = PyTuple_Pack(2, __pyx_n_s_o, __pyx_n_s_kwargs); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);
  __pyx_codeobj__33 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS|CO_VARKEYWORDS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__32, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_ddtrace_internal__encoding_pyx, __pyx_n_s_packb, 902, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__33)) __PYX_ERR(0, 902, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_StringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__34 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_StringTable, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__36 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ListStringTable, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  __pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_6 = PyInt_FromLong(6); if (unlikely(!__pyx_int_6)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_31856863 = PyInt_FromLong(31856863L); if (unlikely(!__pyx_int_31856863)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_66476266 = PyInt_FromLong(66476266L); if (unlikely(!__pyx_int_66476266)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_88997951 = PyInt_FromLong(88997951L); if (unlikely(!__pyx_int_88997951)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_182609841 = PyInt_FromLong(182609841L); if (unlikely(!__pyx_int_182609841)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_194841817 = PyInt_FromLong(194841817L); if (unlikely(!__pyx_int_194841817)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_228126377 = PyInt_FromLong(228126377L); if (unlikely(!__pyx_int_228126377)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable = &__pyx_vtable_7ddtrace_8internal_9_encoding_StringTable;
  __pyx_vtable_7ddtrace_8internal_9_encoding_StringTable.insert = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_insert;
  __pyx_vtable_7ddtrace_8internal_9_encoding_StringTable._index = (uint32_t (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_11StringTable__index;
  __pyx_vtable_7ddtrace_8internal_9_encoding_StringTable.index = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_index;
  __pyx_vtable_7ddtrace_8internal_9_encoding_StringTable.reset = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *))__pyx_f_7ddtrace_8internal_9_encoding_11StringTable_reset;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_StringTable) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_StringTable.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_StringTable.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_StringTable.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_StringTable.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_StringTable.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StringTable, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_StringTable) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_StringTable) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_StringTable = &__pyx_type_7ddtrace_8internal_9_encoding_StringTable;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_ListStringTable = &__pyx_vtable_7ddtrace_8internal_9_encoding_ListStringTable;
  __pyx_vtable_7ddtrace_8internal_9_encoding_ListStringTable.__pyx_base = *__pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable;
  __pyx_vtable_7ddtrace_8internal_9_encoding_ListStringTable.__pyx_base.insert = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_15ListStringTable_insert;
  __pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_StringTable;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_ListStringTable) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ListStringTable, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable) < 0) __PYX_ERR(0, 188, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_ListStringTable = &__pyx_type_7ddtrace_8internal_9_encoding_ListStringTable;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackStringTable = &__pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.__pyx_base = *__pyx_vtabptr_7ddtrace_8internal_9_encoding_StringTable;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.__pyx_base.insert = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_insert;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.__pyx_base.reset = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_StringTable *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_reset;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.savepoint = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_savepoint;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.rollback = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_rollback;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.get_bytes = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_get_bytes;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.append_raw = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *, long, Py_ssize_t))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_append_raw;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackStringTable.flush = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackStringTable *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackStringTable_flush;
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_StringTable;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackStringTable) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MsgpackStringTable, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable) < 0) __PYX_ERR(0, 202, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackStringTable = &__pyx_type_7ddtrace_8internal_9_encoding_MsgpackStringTable;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder) < 0) __PYX_ERR(0, 302, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_BufferedEncoder, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder) < 0) __PYX_ERR(0, 302, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder) < 0) __PYX_ERR(0, 302, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder = &__pyx_type_7ddtrace_8internal_9_encoding_BufferedEncoder;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_ListBufferedEncoder = &__pyx_vtable_7ddtrace_8internal_9_encoding_ListBufferedEncoder;
  __pyx_vtable_7ddtrace_8internal_9_encoding_ListBufferedEncoder.put = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_put;
  __pyx_vtable_7ddtrace_8internal_9_encoding_ListBufferedEncoder.get = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_ListBufferedEncoder *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_19ListBufferedEncoder_get;
  __pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_ListBufferedEncoder) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ListBufferedEncoder, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder) < 0) __PYX_ERR(0, 323, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_ListBufferedEncoder = &__pyx_type_7ddtrace_8internal_9_encoding_ListBufferedEncoder;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase = &__pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase._decode = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__decode;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase._reset_buffer = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__reset_buffer;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.encode = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_encode;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase._update_array_len = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__update_array_len;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.get_bytes = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_bytes;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.get_buffer = (char *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_buffer;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.get_dd_origin_ref = (void *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_get_dd_origin_ref;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase._pack_trace = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase__pack_trace;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.put = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_put;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.flush = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_flush;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.pack_span = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, void *))__pyx_f_7ddtrace_8internal_9_encoding_18MsgpackEncoderBase_pack_span;
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MsgpackEncoderBase, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase) < 0) __PYX_ERR(0, 369, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase = &__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 = &__pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.__pyx_base = *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.__pyx_base.get_dd_origin_ref = (void *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_get_dd_origin_ref;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.__pyx_base.flush = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_flush;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.__pyx_base.pack_span = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, void *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03_pack_span;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03._pack_meta = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *, char *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_meta;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV03._pack_metrics = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV03__pack_metrics;
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03) < 0) __PYX_ERR(0, 500, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV03) < 0) __PYX_ERR(0, 500, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MsgpackEncoderV03, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03) < 0) __PYX_ERR(0, 500, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03) < 0) __PYX_ERR(0, 500, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV03 = &__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV03;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 = &__pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.__pyx_base = *__pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.__pyx_base.get_dd_origin_ref = (void *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_get_dd_origin_ref;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.__pyx_base.put = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_put;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.__pyx_base.flush = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_flush;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.__pyx_base.pack_span = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderBase *, PyObject *, void *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05_pack_span;
  __pyx_vtable_7ddtrace_8internal_9_encoding_MsgpackEncoderV05._pack_string = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_17MsgpackEncoderV05__pack_string;
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_base = __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_MsgpackEncoderV05) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_MsgpackEncoderV05, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderV05 = &__pyx_type_7ddtrace_8internal_9_encoding_MsgpackEncoderV05;
  __pyx_vtabptr_7ddtrace_8internal_9_encoding_Packer = &__pyx_vtable_7ddtrace_8internal_9_encoding_Packer;
  __pyx_vtable_7ddtrace_8internal_9_encoding_Packer._pack = (int (*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *, PyObject *))__pyx_f_7ddtrace_8internal_9_encoding_6Packer__pack;
  __pyx_vtable_7ddtrace_8internal_9_encoding_Packer.pack = (PyObject *(*)(struct __pyx_obj_7ddtrace_8internal_9_encoding_Packer *, PyObject *, int __pyx_skip_dispatch))__pyx_f_7ddtrace_8internal_9_encoding_6Packer_pack;
  if (PyType_Ready(&__pyx_type_7ddtrace_8internal_9_encoding_Packer) < 0) __PYX_ERR(0, 746, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_7ddtrace_8internal_9_encoding_Packer.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_7ddtrace_8internal_9_encoding_Packer.tp_dictoffset && __pyx_type_7ddtrace_8internal_9_encoding_Packer.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_7ddtrace_8internal_9_encoding_Packer.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_7ddtrace_8internal_9_encoding_Packer.tp_dict, __pyx_vtabptr_7ddtrace_8internal_9_encoding_Packer) < 0) __PYX_ERR(0, 746, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_Packer, (PyObject *)&__pyx_type_7ddtrace_8internal_9_encoding_Packer) < 0) __PYX_ERR(0, 746, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_7ddtrace_8internal_9_encoding_Packer) < 0) __PYX_ERR(0, 746, __pyx_L1_error)
  __pyx_ptype_7ddtrace_8internal_9_encoding_Packer = &__pyx_type_7ddtrace_8internal_9_encoding_Packer;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(2, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(3, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(4, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC init_encoding(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC init_encoding(void)
#else
__Pyx_PyMODINIT_FUNC PyInit__encoding(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__encoding(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__encoding(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_encoding' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit__encoding(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_encoding", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_ddtrace__internal___encoding) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "ddtrace.internal._encoding")) {
      if (unlikely(PyDict_SetItemString(modules, "ddtrace.internal._encoding", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "ddtrace/internal/_encoding.pyx":5
 * from libc cimport stdint
 * from libc.string cimport strlen
 * import threading             # <<<<<<<<<<<<<<
 * from ._utils cimport PyBytesLike_Check
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_threading, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_threading, __pyx_t_1) < 0) __PYX_ERR(0, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "ddtrace/internal/_encoding.pyx":8
 * from ._utils cimport PyBytesLike_Check
 * 
 * from ddtrace.constants import ORIGIN_KEY             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ORIGIN_KEY);
  __Pyx_GIVEREF(__pyx_n_s_ORIGIN_KEY);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ORIGIN_KEY);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_ddtrace_constants, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ORIGIN_KEY); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ORIGIN_KEY, __pyx_t_1) < 0) __PYX_ERR(0, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":43
 * 
 * 
 * cdef long long ITEM_LIMIT = (2**32)-1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_v_7ddtrace_8internal_9_encoding_ITEM_LIMIT = 0xFFFFFFFF;

  /* "ddtrace/internal/_encoding.pyx":50
 * 
 * 
 * class BufferFull(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_BufferFull, __pyx_n_s_BufferFull, (PyObject *) NULL, __pyx_n_s_ddtrace_internal__encoding, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_BufferFull, __pyx_t_2, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BufferFull, __pyx_t_4) < 0) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":54
 * 
 * 
 * class BufferItemTooLarge(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_BufferItemTooLarge, __pyx_n_s_BufferItemTooLarge, (PyObject *) NULL, __pyx_n_s_ddtrace_internal__encoding, (PyObject *) NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_BufferItemTooLarge, __pyx_t_2, __pyx_t_3, NULL, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_BufferItemTooLarge, __pyx_t_4) < 0) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":67
 * # This is a borrowed reference but should be fine as we don't expect ORIGIN_KEY
 * # to get GC'd.
 * cdef const char * _ORIGIN_KEY = string_to_buff(ORIGIN_KEY)             # <<<<<<<<<<<<<<
 * cdef size_t _ORIGIN_KEY_LEN = <size_t> len(ORIGIN_KEY)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_ORIGIN_KEY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 67, __pyx_L1_error)
  __pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY = __pyx_f_7ddtrace_8internal_9_encoding_string_to_buff(((PyObject*)__pyx_t_2));
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":68
 * # to get GC'd.
 * cdef const char * _ORIGIN_KEY = string_to_buff(ORIGIN_KEY)
 * cdef size_t _ORIGIN_KEY_LEN = <size_t> len(ORIGIN_KEY)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_ORIGIN_KEY); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyObject_Length(__pyx_t_2); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_7ddtrace_8internal_9_encoding__ORIGIN_KEY_LEN = ((size_t)__pyx_t_5);

  /* "ddtrace/internal/_encoding.pyx":303
 * 
 * cdef class BufferedEncoder(object):
 *     content_type: str = None             # <<<<<<<<<<<<<<
 * 
 *     cdef public size_t max_size
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder->tp_dict, __pyx_n_s_content_type, Py_None) < 0) __PYX_ERR(0, 303, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_7ddtrace_8internal_9_encoding_BufferedEncoder);

  /* "ddtrace/internal/_encoding.pyx":370
 * 
 * cdef class MsgpackEncoderBase(BufferedEncoder):
 *     content_type = "application/msgpack"             # <<<<<<<<<<<<<<
 * 
 *     cdef msgpack_packer pk
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase->tp_dict, __pyx_n_s_content_type, __pyx_kp_s_application_msgpack) < 0) __PYX_ERR(0, 370, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_7ddtrace_8internal_9_encoding_MsgpackEncoderBase);

  /* "ddtrace/internal/_encoding.pyx":902
 * 
 * 
 * def packb(o, **kwargs):             # <<<<<<<<<<<<<<
 *     """
 *     Pack object `o` and return packed bytes
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7ddtrace_8internal_9_encoding_1packb, NULL, __pyx_n_s_ddtrace_internal__encoding); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_packb, __pyx_t_2) < 0) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StringTable(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7ddtrace_8internal_9_encoding_3__pyx_unpickle_StringTable, NULL, __pyx_n_s_ddtrace_internal__encoding); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_StringTable, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_StringTable__set_state(<StringTable> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StringTable__set_state(StringTable __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._next_id = __pyx_state[0]; __pyx_result._table = __pyx_state[1]
 *     if len(__pyx_state) > 2 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_7ddtrace_8internal_9_encoding_5__pyx_unpickle_ListStringTable, NULL, __pyx_n_s_ddtrace_internal__encoding); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ListStringTable, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "ddtrace/internal/_encoding.pyx":1
 * from cpython cimport *             # <<<<<<<<<<<<<<
 * from cpython.bytearray cimport PyByteArray_CheckExact
 * from libc cimport stdint
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init ddtrace.internal._encoding", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init ddtrace.internal._encoding");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* KeywordStringCheck */
static int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* SliceObject */
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* UnpackUnboundCMethod */
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    target->method = method;
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION >= 3
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    #endif
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    }
#endif
    return 0;
}

/* CallUnboundCMethod0 */
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *args, *result = NULL;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_ASSUME_SAFE_MACROS
    args = PyTuple_New(1);
    if (unlikely(!args)) goto bad;
    Py_INCREF(self);
    PyTuple_SET_ITEM(args, 0, self);
#else
    args = PyTuple_Pack(1, self);
    if (unlikely(!args)) goto bad;
#endif
    result = __Pyx_PyObject_Call(cfunc->method, args, NULL);
    Py_DECREF(args);
bad:
    return result;
}

/* py_dict_items */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    if (PY_MAJOR_VERSION >= 3)
        return __Pyx_CallUnboundCMethod0(&__pyx_umethod_PyDict_Type_items, d);
    else
        return PyDict_Items(d);
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_NeObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_FALSE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a != b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_TRUE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_TRUE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal != 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a != (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_NE));
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_n_s_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_uint32_t(uint32_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint32_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(uint32_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint32_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(uint32_t),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint32_t __Pyx_PyInt_As_uint32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint32_t neg_one = (uint32_t) -1, const_zero = (uint32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 2 * PyLong_SHIFT) {
                            return (uint32_t) (((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 3 * PyLong_SHIFT) {
                            return (uint32_t) (((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) >= 4 * PyLong_SHIFT) {
                            return (uint32_t) (((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint32_t) ((((((uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) (((uint32_t)-1)*(((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint32_t) ((((((((((uint32_t)digits[3]) << PyLong_SHIFT) | (uint32_t)digits[2]) << PyLong_SHIFT) | (uint32_t)digits[1]) << PyLong_SHIFT) | (uint32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint32_t) -1;
        }
    } else {
        uint32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint32_t) -1;
        val = __Pyx_PyInt_As_uint32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint32_t");
    return (uint32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint32_t");
    return (uint32_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_As_unsigned_PY_LONG_LONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG) -1, const_zero = (unsigned PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned PY_LONG_LONG) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned PY_LONG_LONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned PY_LONG_LONG) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) >= 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned PY_LONG_LONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned PY_LONG_LONG) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) (((unsigned PY_LONG_LONG)-1)*(((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned PY_LONG_LONG) ((((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned PY_LONG_LONG) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned PY_LONG_LONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned PY_LONG_LONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned PY_LONG_LONG) -1;
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned PY_LONG_LONG) -1;
        val = __Pyx_PyInt_As_unsigned_PY_LONG_LONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned PY_LONG_LONG");
    return (unsigned PY_LONG_LONG) -1;
}

/* CIntFromPy */
static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_As_PY_LONG_LONG(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const PY_LONG_LONG neg_one = (PY_LONG_LONG) -1, const_zero = (PY_LONG_LONG) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(PY_LONG_LONG) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (PY_LONG_LONG) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (PY_LONG_LONG) 0;
                case  1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, digit, digits[0])
                case 2:
                    if (8 * sizeof(PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) >= 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (PY_LONG_LONG) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(PY_LONG_LONG) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(PY_LONG_LONG) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (PY_LONG_LONG) 0;
                case -1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(PY_LONG_LONG,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(PY_LONG_LONG) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(PY_LONG_LONG) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((((PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) (((PY_LONG_LONG)-1)*(((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(PY_LONG_LONG) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(PY_LONG_LONG, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                            return (PY_LONG_LONG) ((((((((((PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (PY_LONG_LONG)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(PY_LONG_LONG) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(PY_LONG_LONG) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(PY_LONG_LONG, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            PY_LONG_LONG val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (PY_LONG_LONG) -1;
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (PY_LONG_LONG) -1;
        val = __Pyx_PyInt_As_PY_LONG_LONG(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to PY_LONG_LONG");
    return (PY_LONG_LONG) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to PY_LONG_LONG");
    return (PY_LONG_LONG) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE uint64_t __Pyx_PyInt_As_uint64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 2 * PyLong_SHIFT) {
                            return (uint64_t) (((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 3 * PyLong_SHIFT) {
                            return (uint64_t) (((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) >= 4 * PyLong_SHIFT) {
                            return (uint64_t) (((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint64_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint64_t) ((((((uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) (((uint64_t)-1)*(((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint64_t) ((((((((((uint64_t)digits[3]) << PyLong_SHIFT) | (uint64_t)digits[2]) << PyLong_SHIFT) | (uint64_t)digits[1]) << PyLong_SHIFT) | (uint64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint64_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint64_t) -1;
        }
    } else {
        uint64_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint64_t) -1;
        val = __Pyx_PyInt_As_uint64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint64_t");
    return (uint64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint64_t");
    return (uint64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int64_t __Pyx_PyInt_As_int64_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int64_t neg_one = (int64_t) -1, const_zero = (int64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int64_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int64_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int64_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int64_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 2 * PyLong_SHIFT) {
                            return (int64_t) (((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 3 * PyLong_SHIFT) {
                            return (int64_t) (((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) >= 4 * PyLong_SHIFT) {
                            return (int64_t) (((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int64_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int64_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int64_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int64_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int64_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int64_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int64_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int64_t) ((((((int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int64_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int64_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int64_t) ((((((((int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int64_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int64_t) (((int64_t)-1)*(((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int64_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int64_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int64_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int64_t) ((((((((((int64_t)digits[3]) << PyLong_SHIFT) | (int64_t)digits[2]) << PyLong_SHIFT) | (int64_t)digits[1]) << PyLong_SHIFT) | (int64_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int64_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int64_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int64_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int64_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int64_t) -1;
        }
    } else {
        int64_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int64_t) -1;
        val = __Pyx_PyInt_As_int64_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int64_t");
    return (int64_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int64_t");
    return (int64_t) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int32_t __Pyx_PyInt_As_int32_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int32_t neg_one = (int32_t) -1, const_zero = (int32_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int32_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int32_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int32_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int32_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 2 * PyLong_SHIFT) {
                            return (int32_t) (((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 3 * PyLong_SHIFT) {
                            return (int32_t) (((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) >= 4 * PyLong_SHIFT) {
                            return (int32_t) (((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int32_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int32_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int32_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int32_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int32_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int32_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int32_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                            return (int32_t) ((((((int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int32_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int32_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                            return (int32_t) ((((((((int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int32_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) (((int32_t)-1)*(((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int32_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int32_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int32_t) - 1 > 4 * PyLong_SHIFT) {
                            return (int32_t) ((((((((((int32_t)digits[3]) << PyLong_SHIFT) | (int32_t)digits[2]) << PyLong_SHIFT) | (int32_t)digits[1]) << PyLong_SHIFT) | (int32_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int32_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int32_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int32_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int32_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int32_t) -1;
        }
    } else {
        int32_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int32_t) -1;
        val = __Pyx_PyInt_As_int32_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int32_t");
    return (int32_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int32_t");
    return (int32_t) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
